// acis.hxx: ACIS class.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: acis_module.hxx,v 1.16.2.1 2001/08/16 16:19:07 scmfcl Exp $

#ifndef ACIS_MODULE_HXX
#define ACIS_MODULE_HXX 1

#include "module.hxx"
#include "list.hxx"

class Material;

class ACIS : public Module {
public:
  ACIS ();
  ~ACIS ();
  void add_symbol (void *object, char *name, bool visible);
  void del_object (void *object);
  void show (void *object);
  void hide (void *object);
  void facet_init ();
  void draw_init ();
  void facet_object (void *object);
  void draw_object (void *object, SimpleList<void> const &hirefs);
  void draw_fixed_object (void *object, SimpleList<void> const &hirefs);
  void load_file (FILE *fp, const char *format, bool ext, bool replace);
  void save_file (FILE *fp, const char *format, bool ext);
  const char **file_formats () const;
  const char *name () const;
private:
  void del_symbol (ENTITY *e);
  void facet_body (BODY *b);
  void facet_lump (LUMP *l);
  void facet_shell (SHELL *s);
  void facet_face (FACE *f);
  void unfacet_body (BODY *b);
  void unfacet_lump (LUMP *l);
  void unfacet_shell (SHELL *s);
  void unfacet_face (FACE *f);
  void draw_entity (ENTITY *e, SimpleList<void> const &hirefs);
  void draw_body (BODY *b, Material *a, bool inv,
		  SimpleList<void> const &hirefs);
  void draw_lump (LUMP *l, Material *a, bool inv, 
		  SimpleList<void> const &hirefs,
		  transf const *trans = NULL);
  void draw_shell (SHELL *s, Material *a, bool inv, 
		   SimpleList<void> const &hirefs,
		   transf const *trans = NULL);
  void draw_face (FACE *f, Material *a, bool inv, 
		  SimpleList<void> const &hirefs,
		  transf const *trans = NULL);
  void draw_wire (WIRE *w, Material *a, bool inv, 
		  SimpleList<void> const &hirefs,
		  transf const *trans = NULL);
  void draw_loop (LOOP *w, Material *a, bool inv, 
		  SimpleList<void> const &hirefs,
		  transf const *trans = NULL);
  void draw_edge (EDGE *e, Material *a, bool inv, 
		  SimpleList<void> const &hirefs,
		  VERTEX *start = NULL, VERTEX *end = NULL,
		  bool sense = false,
		  transf const *trans = NULL);
  void draw_vertex (VERTEX *v, Material *a, bool inv, 
		    SimpleList<void> const &hirefs,
		    transf const *trans = NULL);
  double max_ll;
  int init_freelist_size;
  static const char *fl_formats[9];
  static const char *nm;
  SimpleList<ENTITY> drawn;
};

#endif // ACIS_MODULE_HXX

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
