// app.hxx: app class.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: app.hxx,v 1.23.2.1 2001/08/16 16:19:04 scmfcl Exp $

#ifndef APP_HXX
#define APP_HXX 1

class GUI;
class Symbols;
class Symbol;
class Renderer;
class PluginInterface;
class ModuleList;
class AppWindow;
class ToolWindow;

class App
{
public:
  App (char const *nam, const char *mods, const char *path);
  ~App ();
  static App *instance () { 
    return this_app; 
  }
  inline const char *name () const {
    return nm;
  }
  void run ();
  inline unsigned long tmp_name () {
    if (tmp_name_counter == ULONG_MAX) {
      // If we reached the maximum, we start at the beginning. It's
      // slighlty dangerous as this can lead to an eternal loop, but
      // we don't really expect to have ULONG_MAX symbols.  The
      // functions calling this can check for an eternal loop by
      // remembering the first value from the function call and report
      // an error if that value is returned a 2nd time.
      tmp_name_counter = 0UL;
      return tmp_name_counter;
    } 
    return ++tmp_name_counter;
  }
protected:
  void reset_tmp_name () {
    tmp_name_counter = 0UL;
    return;
  }
  ModuleList *modules;
  GUI *gui;
  Symbols *symbols;
  friend PluginInterface;
  friend GUI;
  friend Symbols;
  friend Symbol;
  friend Renderer;
  friend AppWindow;
  friend ToolWindow;
  friend void sil_register_update_handler (void (*u) (void));
  friend void sil_deregister_update_handler (void (*u) (void));
private:
  static App *this_app;
  static unsigned long tmp_name_counter;
  const char *nm;
  PluginInterface *interface;
};

#endif // !APP_HXX

/// Local Variables:
/// exe-comm: "./sil"
/// compile-command: "make"
/// End:
