// basetypes.hxx: Points class.
//
// Copyright (C) 1999-2000 Frank C Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: basetypes.hxx,v 1.8.2.1 2001/08/16 16:19:08 scmfcl Exp $

#ifndef BASETYPES_HXX
#define BASETYPES_HXX 1

#include "sil_light.hxx"
#include "material.hxx"

// General base type class.
class Base {
public:
  Base () { 
  };
  virtual ~Base () { 
  }
  virtual inline const char *get_type () {
    return type;
  };
  inline bool is (const char *s) {
    return (strcmp (get_type (), s) == 0);
  };
private:
  static const char *type;
};

// Base light class.
class Light : public Base, public SiL_Light {
public:
  Light (GLfloat ar=0.0, GLfloat ag=0.0, GLfloat ab=0.0, GLfloat aa=1.0,
	 GLfloat dr=1.0, GLfloat dg=1.0, GLfloat db=1.0, GLfloat da=1.0,
	 GLfloat sr=1.0, GLfloat sg=1.0, GLfloat sb=1.0, GLfloat sa=1.0,
	 GLfloat x=-0.0, GLfloat y=0.0, GLfloat z=-1.0, GLfloat w=0.0,
	 GLfloat sx=0.0, GLfloat sy=0.0, GLfloat sz=-1.0,
	 GLfloat se=0.0,
	 GLfloat sc=180.0,
	 GLfloat ca=1.0,
	 GLfloat la=0.0,
	 GLfloat qa=0.0) :
    Base (),
    SiL_Light (ar, ag, ab, aa,
	       dr, dg, db, da,
	       sr, sg, sb, sa,
	       x, y, z, w,
	       sx, sy, sz,
	       se, sc, ca, la, qa) {
  };
  ~Light () { };
  inline const char *get_type () {
    return type;
  };
private:
  static const char *type;
};

class Vectors : public Base {
public:
  inline Vectors () {
    head = NULL;
    material = new Material ();
  };
  ~Vectors();
  void add (double a, double b, double c);
  void gl_render (bool inv);
  inline const char *get_type () {
    return type;
  };
  inline void set_material (GLfloat ar, GLfloat ag, GLfloat ab, GLfloat aa,
			    GLfloat dr, GLfloat dg, GLfloat db, GLfloat da,
			    GLfloat sr, GLfloat sg, GLfloat sb, GLfloat sa,
			    GLfloat er, GLfloat eg, GLfloat eb, GLfloat ea,
			    GLfloat ss) {
    if (!material)
      material = new Material(ar, ag, ab, aa,
			      dr, dg, db, da,
			      sr, sg, sb, sa,
			      er, eg, eb, ea,
			      ss);
    else
      material->set(ar, ag, ab, aa,
		    dr, dg, db, da,
		    sr, sg, sb, sa,
		    er, eg, eb, ea,
		    ss);
  }
private:
  class VectorElement {
  protected:
    VectorElement (double a,double b, double c, VectorElement *n) {
      u = a;
      v = b;
      w = c;
      next = n;
    }
    double u, v, w;
    VectorElement *next;
    friend class Vectors;
  };
  VectorElement *head;
  Material *material;
  static const char *type;
};

#endif // !BASETYPES_HXX

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
