/* config.h: Master configuration header file.
 * 
 * Copyright (C) 1999-2001 Frank Langbein <frank@langbein.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: config.h 1 2005-08-10 23:35:07Z frank $
 */

/* NOTE: This header file should be included first by all source files, but
 * not by any of the other header files. 
 *
 * There is no need to edit this file for changing the configuration. This
 * should always be done by the autoconf script configure and the header
 * file generated by that script.
 */

#ifndef CONFIG_H
#define CONFIG_H 1

#ifdef __cplusplus
using namespace std;
#endif /* __cplusplus */

/* Include the configuration parameters from configure/autoconf first. */
#ifdef HAVE_CONFIG_H
#include "c-auto.h"
#endif /* HAVE_CONFIG_H */

/* ACIS and C++ defines the type vector. To avoid namespace conflicts
 * we redefine the C++ vector to _internal_vector. That also means that
 * we can't use the C++ vector type without taking special care. */
#define vector _interval_vector

/* If debugging is not enabled by configure, define the no-debug symbols
 * for assert, ...etc. */
#ifndef DEBUG
#define NDEBUG
#define NO_DEBUG
#endif /* !DEBUG */

/* If we are using glibc this enables some useful extensions, otherwise
 * it should have no effect. */
#define _GNU_SOURCE 1

/* Standard C header files */

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */

#ifdef HAVE_MATH_H
#include <math.h>
#endif /* HAVE_MATH_H */

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#ifdef HAVE_STRING_H
#include <string>
#endif /* HAVE_STRING_H */

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif /* HAVE_LIMITS_H */

#ifdef __cplusplus
/* We only need this in the C++ code and that way we avoid some warnings. */
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */
#endif /* __cplusplus */

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif /* HAVE_SYS_TIME_H */

#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif /* HAVE_SYS_RESOURCE_H */

/* Standard C++ header files */

#ifdef __cplusplus

#ifdef HAVE_NEW_H
#include <new.h>
#endif /* HAVE_NEW_H */

#ifdef HAVE_IOSTREAM_H
#include <iostream.h>
#endif /* HAVE_IOSTRAM_H */

#ifdef HAVE_FSTREAM_H
#include <fstream.h>
#endif /* HAVE_FSTREAM_H */

#endif /* __cplusplus */

/* Compiler characteristics. */

/* The NULL pointer should be defined on most systems. But there are
 * still some which don't provide it. */
#ifndef NULL
#ifdef __cplusplus
#define NULL 0
#else /* __cplusplus */
#define NULL (void *)0
#endif /* !__cplusplus */
#endif /* !NULL */

/* Define const to be empty if it is not already defined or provided
 * by the compiler. */
#if !defined(__STDC__) || !__STDC__
/* This is a separate conditional since some standard C systems reject
 * `defined (const)'. */
#ifndef const
#define const
#endif /* !const */
#endif /* !defined(__STDC__) || !__STDC__ */

/* Define the RealF32 type which should always represent a 4 byte (32 bit)
 * floating point number. It is not available if NO_REALF32 is defined. */
#ifdef REALF32_DOUBLE
typedef double RealF32;
#else /* REALF32_DOUBLE */
#ifdef REALF32_FLOAT
typedef float RealF32;
#else /* REALF32_DOUBLE */
#ifndef NO_REALF32
/* Configure should define this in c-auto.h, but that way we are sure it
 * is there. */
#define NO_REALF32
#endif /* !NO_REALF32 */
#endif /* REALF32_FLOAT */
#endif /* REALF32_DOUBLE */

/* Check if maximum for unsigned long is known. If not, use a reasonable
 * value. */
#ifndef ULONG_MAX
#ifdef __STDC__
#define ULONG_MAX 4294967295UL
#else /* __STDC__ */
#define ULONG_MAX   4294967295L
#endif /* __STDC__ */
#endif /* !ULONG_MAX */

/* Directory */

#ifdef HAVE_DIRENT_H
#include <dirent.h>
#define NAMLEN(dirent) strlen((dirent)->d_name)
#else // HAVE_DIRENT_H
#define dirent direct
#define NAMLEN(dirent) (dirent)->d_namlen
#ifdef HAVE_SYS_NDIR_H
#include <sys/ndir.h>
#endif // HAVE_SYS_NDIR_H
#if HAVE_SYS_DIR_H
#include <sys/dir.h>
#endif // HAVE_SYS_DIR_H
#if HAVE_NDIR_H
#include <ndir.h>
#endif // HAVE_NDIR_H
#endif // HAVE_DIRENT_H

/* Not-a-number definitions. */

#ifdef HAVE_NAN_H
/* The IEEE NAN code seems to be supported on this machine, so include
 * the nan.h header file. */
#include <nan.h>
#ifndef HAVE_ISNAN
/* If we don't have a isnan function, we simulate it with this comparison.
 * IEEE standard says that NAN != NAN. */
#define isnan(x) ((x) != (x))
#endif /* HAVE_ISNAN */
#else /* !HAVE_NAN_H */
#if !(defined(HAVE_ISNAN) && defined(NAN))
/* We don't have the nan.h header file and NAN and isnan is not
 * available. So we try to simulate the IEEE NAN code. But note that
 * this can have some strange effects even if the code is very close
 * to the real IEEE code. */
#ifdef WORDS_BIGENDIAN
#define sil_nan_bytes { (unsigned char) 0x7f, (unsigned char) 0xf8, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0 }
#else /* !WORDS_BIGENDIAN */
#define sil_nan_bytes { (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0xf8, (unsigned char) 0x7f }
#endif /* !WORDS_BIGENDIAN */
static const unsigned char sil_nan[8] = sil_nan_bytes;
#ifdef NAN
#undef NAN
#endif /* NAN */
#define NAN (*(const double *) sil_nan)
#define USE_SIL_ISNAN 1
#endif /* !(defined(HAVE_ISNAN) && defined(NAN)) */
#endif /* !HAVE_NAN_H */

/* Assertions are too useful to not make them generally available. 
 * We only use them if debugging is enabled and if the system provides
 * them. We define an extra macro assert_exe(expr) which will always
 * execute expr, no matter if the assertions are enabled or not. But
 * the result will only be checked when the assertions are enabled. */
#ifdef HAVE_ASSERT_H

#include <assert.h> 

/** Execute assertion.
 *
 * This first executes the given expression even if assertions are
 * disabled. If the assertions are enabled, assert is called with the
 * result for the expression. This is used to run assertions on the
 * results of certain function calls without creating lots of extra
 * code storing these results.
 *
 * @param expr Expression. */
#ifdef NO_DEBUG 
#define assert_exe(expr) expr
#else /* !NO_DEBUG */
#define assert_exe(expr) assert (expr)
#endif /* !NO_DEBUG */

#else /* !HAVE_ASSERT_H */

#define assert(expr)
#define assert_exe(expr) expr

#endif /* !HAVE_ASSERT_H */

/* Include the dynamic loader definitions. */
#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#endif /* HAVE_DLFCN_H */

/* GTKMM headers */

#ifdef __cplusplus

#ifdef HAVE_GDK_GDKX_H
#include <gdk/gdkx.h>
#endif /* HAVE_GDK_H */

#ifdef HAVE_GTK___H
#include <gtk--.h>
#endif /* HAVE_GTK___H */

#ifdef HAVE_GTKGL___GDKGL_H
#include <gtkgl--/gdkgl.h>
#endif /* HAVE_GTK___GDKGL_H */

#ifdef HAVE_GTKGL___GLAREA_H
#include <gtkgl--/glarea.h>
#endif /* HAVE_GTK___GLAREA_H */

#endif /* __cplusplus */

/* OpenGL and GLUT header files. */

#ifdef __cplusplus
/* There is no real need to do this, but we only use OpenGL in the C++ code
 * and this avoids a few warnings. */

#ifdef USE_MESAGL

#ifdef HAVE_MESAGL_GL_H
#include <MesaGL/gl.h>
#else /* !HAVE_MESAGL_GL_H */
#ifdef HAVE_GL_GL_H
#include <GL/gl.h>
#endif /* HAVE_GL_GL_H */
#endif /* !HAVE_MESAGL_GL_H */

#ifdef HAVE_MESAGL_GLX_H
#include <MesaGL/glx.h>
#else /* !HAVE_MESAGL_GLX_H */
#ifdef HAVE_GL_GLX_H
#include <GL/glx.h>
#endif /* HAVE_GL_GLX_H */
#endif /* !HAVE_MESAGL_GLX_H */

#ifdef HAVE_MESAGL_GLU_H
#include <MesaGL/glu.h>
#else /* !HAVE_MESAGL_GLU_H */
#ifdef HAVE_GL_GLU_H
#include <GL/glu.h>
#endif /* HAVE_GL_GLU_H */
#endif /* !HAVE_MESAGL_GLU_H */

#ifdef HAVE_MESAGL_GLUT_H
#include <MesaGL/glut.h>
#else /* !HAVE_MESAGL_GLUT_H */
#ifdef HAVE_GL_GLUT_H
#include <GL/glut.h>
#endif /* !HAVE_GL_GLUT_H */
#endif /* !HAVE_MESAGL_GLUT_H */

#else /* !USE_MEASGL */

#ifdef HAVE_GL_GL_H
#include <GL/gl.h>
#else /* !HAVE_GL_GL_H */
#ifdef HAVE_MESAGL_GL_H
#include <MesaGL/gl.h>
#endif /* HAVE_MESAGL_GL_H */
#endif /* !HAVE_GL_GL_H */

#ifdef HAVE_GL_GLX_H
#include <GL/glx.h>
#else /* !HAVE_GL_GLX_H */
#ifdef HAVE_MESAGL_GLX_H
#include <MesaGL/glx.h>
#endif /* HAVE_MESAGL_GLX_H */
#endif /* !HAVE_GL_GLX_H */

#ifdef HAVE_GL_GLU_H
#include <GL/glu.h>
#else /* !HAVE_GL_GLU_H */
#ifdef HAVE_MESAGL_GLU_H
#include <MesaGL/glu.h>
#endif /* HAVE_MESAGL_GLU_H */
#endif /* !HAVE_GL_GLU_H */

#ifdef HAVE_GL_GLUT_H
#include <GL/glut.h>
#else /* !HAVE_GL_GLUT_H */
#ifdef HAVE_MESAGL_GLUT_H
#include <MesaGL/glut.h>
#endif /* !HAVE_MESAGL_GLUT_H */
#endif /* !HAVE_GL_GLUT_H */

#endif /* !USE_MESAGL */

#endif /* __cplusplus */

/** OpenGL name.
 *
 * String constant for the name of the used OpenGL version Currently
 * it's set to "OpenGL" for the default OpenGL version on the system
 * and to "MesaGL" when we are using MesaGL explicitly.
 */
#ifdef USE_MESAGL
#define OPENGLNAME "MesaGL"
#else /* !USE_MESAGL */
#define OPENGLNAME "OpenGL"
#endif /* !USE_MESAGL */

/** Set RCS id.
 *
 * Set the RCS id for the module. This defines a variable
 * rcsid_##module which points to the RCS id string. This way of
 * setting the string will avoid any Warnings about it and still
 * provide all the RCS id strings in the binary.
 *
 * @param module  Name of the module.
 * @param idstr   RCS id string.  */
#define RCSID(module, idstr) const char *rcsid_##module = idstr

/* Precision
 */
#define DEFAULT_EPSILON 1e-10

/* Mouse sensitivity */
#define DEFAULT_SENS_ANGLE 0.6
#define DEFAULT_SENS_MOVE 0.01
#define DEFAULT_SENS_DIST 0.1

/* Render box dimensions (min/max) */
#define DEFAULT_RENDER_BOX_MIN -10000.0
#define DEFAULT_RENDER_BOX_MAX 10000.0

/* Default tmp directory */
#define DEFAULT_TMPDIR "/tmp"

/* Undefine vector. From here vector should always refer to the ACIS
 * vector type unless vector is redefined to _internal_vector. It's
 * a mess and I wish ACIS wouldn't use those symbols. */
#undef vector

#endif /* !CONFIG_H */

/***
 *** Local Variables:
 *** exe-comm: "./sil"
 *** compile-command: "make"
 *** End:
 ***/
