// gui.cc: GUI class implementation.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: gui.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (gui, "$Id: gui.cc 1 2005-08-10 23:35:07Z frank $");

#include "xmath.hxx"

#include "interface.hxx"

#include "list.hxx"
#include "symbols.hxx"
#include "interface.hxx"
#include "renderer.hxx"
#include "gui.hxx"
#include "app.hxx"
#include "module.hxx"
#include "modules.hxx"

#include "pixmaps/new.xpm"
#include "pixmaps/open.xpm"
#include "pixmaps/save.xpm"
#include "pixmaps/config.xpm"
#include "pixmaps/init_view.xpm"
#include "pixmaps/plugin.xpm"
#include "pixmaps/exit.xpm"

using namespace SigC;

extern char *plugin_path;

const gchar *sym_titles[2] = { "V", "Name" };
const gchar *sym_vis = "*";
const gchar *sym_non_vis = " ";

SimpleList<Plugin> sil_plugins;
SimpleList<void> sil_open_plugins;

void sil_register_tool (void (*f) (void),
			const char *n,
			const char *v,
			const char *a,
			const char *c,
			const char *d,
			const char *cl,
			const char *m)
{
  // Check if modules are available
  const char *ms = m;
  char *mod = new char[strlen (ms) + 2];
  strcpy (mod, ms);
  mod[strlen (ms) + 1] = '\0';
  char *mm = mod;
  while (*mm != '\0') {
    int l = 0;
    while (mm[l] != ':' && mm[l] != '\0')
      ++l;
    mm[l] = '\0';
    if (!sil_get_module (mm)) {
      char *str = new char[strlen (mm) + 25];
      strcpy (str, "module ");
      strcpy (str, mm);
      strcat (str, " is not available");
      sil_msg_error("plugin", str);
      delete []str;
      delete []mod;
      return;
    }
    mm += l + 1;
  }
  delete []mod;

  Plugin *p = new Plugin (f, n, v, a, c, d, cl, m);
  sil_plugins.add_last (p);

  return;
}

void sil_register_tool (void (*f) (SimpleList<Symbol const> *), 
			const char *n,
			const char *v,
			const char *a,
			const char *c,
			const char *d,
			const char *cl,
			const char *m)
{
  // Check if modules are available
  const char *ms = m;
  char *mod = new char[strlen (ms) + 2];
  strcpy (mod, ms);
  mod[strlen (ms) + 1] = '\0';
  char *mm = mod;
  while (*mm != '\0') {
    int l = 0;
    while (mm[l] != ':' && mm[l] != '\0')
      ++l;
    mm[l] = '\0';
    if (!sil_get_module (mm)) {
      char *str = new char[strlen (mm) + 25];
      strcpy (str, "module ");
      strcpy (str, mm);
      strcat (str, " is not available");
      sil_msg_error("plugin", str);
      delete []str;
      delete []mod;
      return;
    }
    mm += l + 1;
  }
  delete []mod;

  Plugin *p = new Plugin (f, n, v, a, c, d, cl, m);
  sil_plugins.add_last (p);

  return;
}

void sil_register_update_handler (void (*u) (void))
{
  assert (u);
  App::instance () -> symbols->add_update_handler (u);
  return;
}

void sil_deregister_update_handler (void (*u) (void))
{
  assert (u);
  App::instance () -> symbols->del_update_handler (u);
  return;
}

ToolWindow::ToolWindow () : 
  Gtk::Window (GTK_WINDOW_TOPLEVEL),
       vbox (),
       hbox (true),
       nb (),
       b_rescan ("Rescan"),
       b_close ("Close")
{
  set_title ("SiL Tools");
  set_border_width (3);
  set_usize (450,400);
  nb.set_tab_pos (GTK_POS_TOP);
  unsigned long num_pages = 0;
  const char *m = App::instance () -> modules -> first_name ();
  while (m) {
    ++num_pages;
    m = App::instance () -> modules -> next_name ();
  }
  if (num_pages > 0) {
    scan_plugins (plugin_path);
    sym_list = new Gtk::CList*[num_pages];
    i_t = new Gtk::Table*[num_pages];
    ni_t = new Gtk::Table*[num_pages];
    unsigned long i = 0;
    m = App::instance () -> modules -> first_name ();
    while (m) {
      Gtk::Frame *frame = manage (new Gtk::Frame (m));
      frame->set_border_width (5);
      frame->show ();
      nb.pages () . push_back (Gtk::Notebook_Helpers::TabElem (*frame, m));
      
      Gtk::HBox *hb = manage (new Gtk::HBox ());
      
      Gtk::ScrolledWindow *swin = manage (new Gtk::ScrolledWindow ());
      swin->set_usize (140, 100);
      sym_list[i] = NULL;
      make_sym_list (sym_list[i], m);
      swin->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
      swin->add_with_viewport (*sym_list[i]);    
      hb->pack_start (*swin, true, true, 0);
      
      i_t[i] = NULL;
      ni_t[i] = NULL;
      make_buttons (i_t[i], ni_t[i], m);

      swin = manage (new Gtk::ScrolledWindow ());
      swin->set_usize (145, 100);
      swin->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
      swin->add_with_viewport (*i_t[i]);    
      hb->pack_start (*swin, false, true, 0);
  
      swin = manage (new Gtk::ScrolledWindow ());
      swin->set_usize (145, 100);
      swin->set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
      swin->add_with_viewport (*ni_t[i]);    
      hb->pack_start (*swin, false, true, 0);

      frame->add (*hb);
      
      ++i;
      m = App::instance () -> modules -> next_name ();
    }
  } else {
    sym_list = NULL;
    i_t = NULL;
    ni_t = NULL;
  }

  add (vbox);
  vbox.pack_start (nb, true, true, 2);
  vbox.pack_start (hbox, false, true, 2);
  hbox.pack_start (b_rescan, true, true, 2);
  hbox.pack_start (b_close, true, true, 2);

  b_close.clicked.connect (slot (this, &ToolWindow::closeit));
  b_rescan.clicked.connect (slot (this, &ToolWindow::rescan));
  destroy.connect (slot (this, &ToolWindow::destroyit));

  show_all ();
}

ToolWindow::~ToolWindow()
{
  App::instance()->gui->closed_tool_window ();
  if (sym_list) {
    delete []sym_list;
    sym_list = NULL;
  }
  if (i_t) {
    delete []i_t;
    i_t = NULL;
  }
  if (ni_t) {
    delete []ni_t;
    ni_t = NULL;
  }
}

void ToolWindow::update ()
{
  if (sym_list) {
    unsigned long i = 0;
    char const *m = App::instance () -> modules -> first_name ();
    while (m) {
      sym_list[i]->freeze ();
      make_sym_list (sym_list[i], m);
      sym_list[i]->thaw ();
      ++i;
      m = App::instance () -> modules -> next_name ();
    }
  }
  return;
}

void ToolWindow::rescan ()
{
  if (i_t && ni_t) {
    scan_plugins (plugin_path);
    unsigned long i = 0;
    char const *m = App::instance () -> modules -> first_name ();
    while (m) {
      make_buttons (i_t[i], ni_t[i], m);
      ++i;
      m = App::instance () -> modules -> next_name ();
    }
    show_all ();
  }
  return;
}

void ToolWindow::closeit ()
{
  hide_all ();
  destroy ();
  return;
}

void ToolWindow::destroyit ()
{
  void *h = sil_open_plugins.unlink_first ();
  while (h) {
    void (*p)(void) = (void (*)(void)) dlsym (h, "plugin_close__Fv");
    const char *e = dlerror ();
    if (p && !e) {
      p ();
    }
    dlclose (h);
    h = sil_open_plugins.unlink_first ();
  }
  sil_plugins.purge ();
  App::instance()->gui->closed_tool_window ();
  if (sym_list) {
    delete sym_list;
    sym_list = NULL;
  }
  if (i_t) {
    delete i_t;
    i_t = NULL;
  }
  if (ni_t) {
    delete ni_t;
    ni_t = NULL;
  }
  return;
}

void ToolWindow::make_sym_list (Gtk::CList *&list, char const *n)
{
  if (!list) {
    list = manage (new Gtk::CList (2, sym_titles));
  } else {
    list->clear ();
  }

  list->column_titles_passive ();
  list->set_column_auto_resize (0, true);
  list->set_column_justification (0, GTK_JUSTIFY_CENTER);
  list->set_column_auto_resize (1, true);
  list->set_column_justification (1, GTK_JUSTIFY_LEFT);
  list->set_auto_sort (false);
  list->set_selection_mode (GTK_SELECTION_MULTIPLE);

  const Symbol *s = sil_first_symbol ();
  while (s) {
    if (strcmp (n, sil_get_symbol_type (s)) == 0) {
      const gchar *text[2];
      if (sil_symbol_is_visible (s))
	text[0] = sym_vis;
      else
	text[0] = sym_non_vis;
      text[1] = sil_get_symbol_name (s);
      list->append (text);
    }
    s = sil_next_symbol ();
  }

  return;
}

void ToolWindow::make_buttons (Gtk::Table *&ti, 
			       Gtk::Table *&tni,
			       char const *n)
{
  unsigned long sti = 1;
  if (ti) {
    ti->resize (sti, 1);
  } else {
    ti = manage (new Gtk::Table (1, sti, true));
  }
  
  unsigned long stni = 1;
  if (tni) {
    tni->resize (stni, 1);
  } else {
    tni = manage (new Gtk::Table (1, stni, true));
  }

  unsigned long pti = 0;
  unsigned long ptni = 0;
  Plugin *p = sil_plugins.first ();
  while (p) {
    if (strcmp (p->module_class (), n) == 0) {
      Gtk::Button *b = manage (new Gtk::Button (p->name ()));
      b->clicked.connect (bind (slot (this, &ToolWindow::call), p));
      b->set_usize (120, 23);
      if (p->interactive ()) {
	if (pti >= sti) {
	  ++sti;
	  ti->resize (sti, 1);
	}
	ti->attach (*b, 0, 1, pti, pti + 1, 
		    GTK_SHRINK|GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 0);
	++pti;
      } else {
	if (ptni >= stni) {
	  ++stni;
	  tni->resize (stni, 1);
	}
	tni->attach (*b, 0, 1, ptni, ptni + 1, 
		     GTK_SHRINK|GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 0);
	++ptni;
      }
    }
    p = sil_plugins.next ();
  }

  return;
}

void ToolWindow::scan_plugins (const char *p)
{
  sil_progress_bar (true, false);
  sil_progress_bar_text ("Scanning plugins");

  sil_plugins.purge ();
  void *h = sil_open_plugins.unlink_first ();
  while (h) {
    void (*p)(void) = (void (*)(void)) dlsym (h, "plugin_close__Fv");
    const char *e = dlerror ();
    if (p && !e)
      p ();
    dlclose (h);
    h = sil_open_plugins.unlink_first ();
  }

  // Scan directories
  double prog = 0.0;
  double inc_prog = 0.1;
  char *path = new char[strlen (p) + 2];
  strcpy (path, p);
  path[strlen(p) + 1] = '\0';
  char *m = path;
  while (*m != '\0') {
    int l = 0;
    while (m[l] != ':' && m[l] != '\0')
      ++l;
    m[l] = '\0';
    DIR *d = opendir (m);
    if (d) {
      unsigned long ld = strlen (m);
      struct dirent *de = readdir (d);
      while (de) {
	unsigned long l = strlen (de->d_name);
	if (l > 11) {
	  if (strcmp (de->d_name + l - 11, ".sil_plugin") == 0) {
	    sil_progress_bar_percentage (prog += inc_prog);
	    inc_prog = (1.0 - prog) * 0.1;
	    char *n = new char[ld + l + 2];
	    strcpy (n, m);
	    strcat (n, "/");
	    strcat (n, de->d_name);
	    register_plugin (n);
	    delete []n;
	  }
	}
	de = readdir (d);
      }
      closedir (d);
    }

    m += l + 1;
  }
  delete []path;

  sil_progress_bar_percentage (1.0);

  sil_progress_bar (false);
  
  return;
}

void ToolWindow::register_plugin (char const *n)
{
  struct stat finfo;
  if (stat (n, &finfo) != 0 || !(S_ISREG (finfo.st_mode) ||
				 S_ISLNK (finfo.st_mode)))
    return;
  void *h = dlopen (n, RTLD_LAZY);
  if (!h) {
    sil_msg_error ("plugin", dlerror());
    return;
  }
  void (*p)(void) = (void (*)(void)) dlsym (h, "plugin__Fv");
  const char *e = dlerror ();
  if (e || !p) {
    sil_msg_error ("plugin", e);
    dlclose (h);
    return;
  }
  char *str = new char[strlen (n) + 13];
  strcpy(str, "Registering ");
  strcat(str, n);
  sil_progress_bar_text (str);
  delete []str;
  p ();

  sil_open_plugins.add_first (h);

  return;
}

void ToolWindow::call (Plugin *p)
{
  assert (p);
  if (p->interactive ()) {
    // Get List
    SimpleList<Symbol const> sl;
    unsigned long cp = nb.get_current_page_num ();
    for (Gtk::CList::SelectionList::iterator iter = 
	   sym_list[cp]->selection () . begin ();
	 iter != sym_list[cp]->selection () . end ();
	 ++iter) {
      int r = (*iter) . get_row_num ();
      string text;
      if (sym_list[cp]->get_text (r, 1, text)) {
	const Symbol *s = sil_find_symbol ((char *)text.c_str ());
	if (s) {
	  sl.add_last (s);
	}
      }
    }
    p->call (&sl);
    sym_list[cp]->selection () . clear ();
    sym_list[cp]->show_all ();
  } else {
    p->call ();
  }
  return;
}

OGLDisplay::OGLDisplay () : Gtk::GLArea ()
{
  set_events (GDK_EXPOSURE_MASK|
	      GDK_BUTTON_MOTION_MASK|
              GDK_POINTER_MOTION_HINT_MASK|
              GDK_BUTTON_PRESS_MASK|
              GDK_BUTTON_RELEASE_MASK);

  renderer = new Renderer;

  // Initialize the mouse state.
  mouse_last_x = mouse_last_y = 0;
}

OGLDisplay::~OGLDisplay ()
{
  if (renderer)
    delete renderer;
}

void OGLDisplay::realize_impl ()
{
  Gtk::GLArea::realize_impl ();
  if (make_current ()) {
    renderer->init ();
    renderer->size (width(), height());
    renderer->show ();
  }
  swap_buffers ();
  return;
}

gint OGLDisplay::motion_notify_event_impl(GdkEventMotion *event)
{
  GdkModifierType state;
  double dx, dy;
  if (event->is_hint) {
    gint tx, ty;
    gdk_window_get_pointer (event->window, &tx, &ty, &state);
    dx = tx - mouse_last_x;
    dy = ty - mouse_last_y;
  } else {
    dx = event->x - mouse_last_x;
    dy = event->y - mouse_last_y;
    state = GdkModifierType (event->state);
  }
  if (state&GDK_BUTTON3_MASK) {
    if (state&GDK_BUTTON2_MASK) {
      renderer->change_obj_rot (dy, 0, dx);
    } else if (state&GDK_BUTTON1_MASK) {
      renderer->change_obj_rot (0, dx, dy);
    } else {
      renderer->change_obj_rot (dy, dx, 0);
    }
  } else if (state&GDK_BUTTON2_MASK) {
    if (state&GDK_BUTTON1_MASK) {
      renderer->change_eye_pos (dx, 0);
    } else {
      renderer->change_eye_pos (0, dy);
    }
  } else if (state&GDK_BUTTON1_MASK) {
    renderer->change_eye_dir (dx, dy);
  } else {
    return TRUE;
  }
  if (make_current ())
    renderer->show ();
  swap_buffers ();
  // Remember the current mouse position.
  mouse_last_x = event->x;
  mouse_last_y = event->y;
  return TRUE;
}

gint OGLDisplay::button_press_event_impl (GdkEventButton *event)
{
  // A button has been pressed at the mouse position x,y.
  mouse_last_x = event->x;
  mouse_last_y = event->y;
  return TRUE;
}

gint OGLDisplay::button_release_event_impl (GdkEventButton *event)
{
  return TRUE;
}

gint OGLDisplay::configure_event_impl (GdkEventConfigure *event)
{
  if (make_current())
    renderer->size (width (), height ());
  swap_buffers ();
  return TRUE;
}

gint OGLDisplay::expose_event_impl (GdkEventExpose *event)
{
  if (event->count > 0)
    return 0;
  if (make_current ())
    renderer->show ();
  swap_buffers ();
  return TRUE;
}

GUI::GUI () : 
  Gtk::Window (GTK_WINDOW_TOPLEVEL),
       vbox (false, 0),
       toolbar (),
       display (),
       status_text (),
       status_scroll (),
       status_scrollbar (*status_text.get_vadjustment ()),
       status_font ("fixed"),
       hb_progress (),
       pb_progress (),
       b_pcancel ("Cancel")
{
  // Setup toolbar
  new_pix = new Gtk::Pixmap (new_xpm);
  open_pix = new Gtk::Pixmap (open_xpm);
  save_pix = new Gtk::Pixmap (save_xpm);
  config_pix = new Gtk::Pixmap (config_xpm);
  init_view_pix = new Gtk::Pixmap (init_view_xpm);
  plugin_pix = new Gtk::Pixmap (plugin_xpm);
  exit_pix = new Gtk::Pixmap (exit_xpm);
  toolbar.set_button_relief ( GTK_RELIEF_NONE );
  toolbar.set_tooltips (true);
  toolbar.set_space_size (30);
  toolbar.tools () . push_back 
    (Gtk::Toolbar_Helpers::ButtonElem ("New",
				       *new_pix,
				       slot (this, &GUI::newlist),
				       "New symbol table", ""));
  toolbar.tools () . push_back 
    (Gtk::Toolbar_Helpers::ButtonElem ("Open",
				       *open_pix,
				       slot (this, &GUI::open),
				       "Open file", ""));
  toolbar.tools () . push_back 
    (Gtk::Toolbar_Helpers::ButtonElem ("Save",
				       *save_pix,
				       slot (this, &GUI::save),
				       "Save file", ""));
  toolbar.tools () . push_back (Gtk::Toolbar_Helpers::Space ());
  toolbar.tools () . push_back 
    (Gtk::Toolbar_Helpers::ButtonElem ("Config",
				       *config_pix,
				       slot (this, &GUI::config),
				       "SiL config", ""));
  toolbar.tools () . push_back 
    (Gtk::Toolbar_Helpers::ButtonElem ("Init View",
				       *init_view_pix,
				       slot (this, &GUI::init_view),
				       "Initial view", ""));
  toolbar.tools () . push_back (Gtk::Toolbar_Helpers::Space ());
  toolbar.tools () . push_back 
    (Gtk::Toolbar_Helpers::ButtonElem ("Tools",
				       *plugin_pix,
				       slot (this, &GUI::plugin),
				       "Show tool window", ""));
  toolbar.tools () . push_back (Gtk::Toolbar_Helpers::Space ());
  toolbar.tools () . push_back 
    (Gtk::Toolbar_Helpers::ButtonElem ("Exit",
				       *exit_pix,
				       slot (this, &GUI::exit),
				       "Exit SiL", ""));
  toolbar.set_orientation (GTK_ORIENTATION_HORIZONTAL); 
  toolbar.set_style (GTK_TOOLBAR_BOTH);

  status_text.set_editable (FALSE);
  status_text.set_word_wrap (TRUE);
  status_text.set_line_wrap (TRUE);
  status_scroll.set_usize (600, 80);
  status_scroll.pack_start (status_text, TRUE, TRUE, 0);
  status_scroll.pack_start (status_scrollbar, FALSE, FALSE, 0);

  pb_progress.set_percentage (0.0);
  b_pcancel.set_sensitive (false);
  b_pcancel.clicked.connect (SigC::slot (this, &GUI::pcancel_handler));
  progress_flag = false;
  progress_cancel_flag = false;
  hb_progress.pack_start (pb_progress, true, true, (gint)2);
  hb_progress.pack_start (b_pcancel, false, false, (gint)2);

  set_title ("SiL");
  set_usize (600, 600);
  add (vbox);

  vbox.pack_start (toolbar, false, false, (gint)2);
  vbox.pack_start (display, true, true, (gint)0);
  vbox.pack_start (status_scroll, false, false, (gint)2);
  vbox.pack_start (hb_progress, false, true, (gint)2);

  destroy.connect (SigC::slot (this, &GUI::win_dest));

  tool_win = NULL;
  file_sel = NULL;
  last_file_dir = NULL;
  replace_sym = true;
  sil_file_ext = true;
  configwin = NULL;

  show_all ();

  win_visible = true;
}

GUI::~GUI ()
{
  if (tool_win)
    tool_win->closeit ();
  if (file_sel) {
    file_sel->hide ();
    delete file_sel;
  }
  if (configwin) {
    configwin->hide ();
    delete configwin;
  }
  if (open_pix)
    delete open_pix;
  if (save_pix)
    delete save_pix;
  if (plugin_pix)
    delete plugin_pix;
  if (exit_pix)
    delete exit_pix;
  if (last_file_dir)
    delete []last_file_dir;
}

bool GUI::status_available ()
{
  return win_visible;
}

void GUI::newlist ()
{
  App::instance ()->symbols->clear ();
  App::instance ()->reset_tmp_name ();
  status_text.backward_delete (status_text.get_length ());
  if (display.make_current ()) {
    display.renderer->init ();
    display.renderer->draw ();
    display.renderer->show ();
  }
  display.swap_buffers ();
  sil_updated ();
  return;
}

void GUI::config ()
{
  if (configwin)
    return;
  configwin = new Gtk::Window (GTK_WINDOW_TOPLEVEL);
  configwin->set_title ("SiL Configuration");

  configwin->delete_event.connect (slot (this, &GUI::config_close));

  config_table = new Gtk::Table (7, 7, true);
  config_table->set_row_spacings (2);
  config_table->set_col_spacings (2);
  config_table->set_border_width (4);
  configwin->add (*config_table);

  double an, as, t, b, n, f, l, r;
  bool ortho;
  display.renderer->projection (an, as, l, r, b, t, n, f, ortho);

  cb_persp = new Gtk::CheckButton ("perspective");
  cb_ortho = new Gtk::CheckButton ("orthographic");

  label_proj[2] = new Gtk::Label ("top");
  label_proj[3] = new Gtk::Label ("bottom");
  label_proj[4] = new Gtk::Label ("near");
  label_proj[5] = new Gtk::Label ("far");

  adj_proj[2] = new Gtk::Adjustment (t,
				     -10000.0, 10000.0,
				     1.0, 5.0,
				     0.0);
  sb_proj[2] = new Gtk::SpinButton (*adj_proj[2], 1.0, 2);
  adj_proj[3] = new Gtk::Adjustment (b,
				     -10000.0, 10000.0,
				     1.0, 5.0,
				     0.0);
  sb_proj[3] = new Gtk::SpinButton (*adj_proj[3], 1.0, 2);
  adj_proj[4] = new Gtk::Adjustment (n,
				     0.01, 10000.0,
				     1.0, 5.0,
				     0.0);
  sb_proj[4] = new Gtk::SpinButton (*adj_proj[4], 1.0, 2);
  adj_proj[5] = new Gtk::Adjustment (f,
				     0.1, 10000.0,
				     1.0, 5.0,
				     0.0);
  sb_proj[5] = new Gtk::SpinButton (*adj_proj[5], 1.0, 2);
  
  adj_proj[0] = new Gtk::Adjustment (an,
				     0.0, 180.0,
				     1.0, 5.0,
				     0.0);
  adj_proj[1] = new Gtk::Adjustment (as,
				     0.0, 100.0,
				     0.1, 1.0,
				     0.0);
  adj_proj[6] = new Gtk::Adjustment (l,
				     -10000.0, 10000.0,
				     1.0, 5.0,
				     0.0);
  adj_proj[7] = new Gtk::Adjustment (r,
				     -10000.0, 10000.0,
				     1.0, 5.0,
				     0.0);
  
  if (ortho) {
    cb_persp->set_active (false);
    cb_ortho->set_active (true);
    label_proj[0] = new Gtk::Label ("left");
    label_proj[1] = new Gtk::Label ("right");
    sb_proj[0] = new Gtk::SpinButton (*adj_proj[6], 1.0, 2);
    sb_proj[1] = new Gtk::SpinButton (*adj_proj[7], 1.0, 2);   
    sb_proj[2]->set_sensitive (true);
    sb_proj[3]->set_sensitive (true);
  } else {
    cb_persp->set_active (true);
    cb_ortho->set_active (false);
    label_proj[0] = new Gtk::Label ("angel");
    label_proj[1] = new Gtk::Label ("aspect");
    sb_proj[0] = new Gtk::SpinButton (*adj_proj[0], 1.0, 2);
    sb_proj[1] = new Gtk::SpinButton (*adj_proj[1], 1.0, 2);
    sb_proj[2]->set_sensitive (false);
    sb_proj[3]->set_sensitive (false);
  }

  config_table->attach (*cb_persp, 
			0, 1, 0, 1,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*cb_ortho, 
			0, 1, 1, 2,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*label_proj[0], 
			1, 2, 0, 1,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_proj[0], 
			2, 3, 0, 1,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*label_proj[1], 
			1, 2, 1, 2,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_proj[1], 
			2, 3, 1, 2,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*label_proj[2], 
			5, 6, 0, 1,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_proj[2], 
			6, 7, 0, 1,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*label_proj[3], 
			5, 6, 1, 2,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_proj[3], 
			6, 7, 1, 2,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*label_proj[4], 
			3, 4, 0, 1,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_proj[4], 
			4, 5, 0, 1,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*label_proj[5], 
			3, 4, 1, 2,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_proj[5], 
			4, 5, 1, 2,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  cb_persp->show ();
  cb_ortho->show ();

  cb_persp->toggled.connect (slot (this, &GUI::config_persp));
  cb_ortho->toggled.connect (slot (this, &GUI::config_ortho));

  for (int i=0; i<6; ++i) {
    sb_proj[i]->show ();
    label_proj[i]->set_justify (GTK_JUSTIFY_RIGHT);
    label_proj[i]->show ();
  }

  cb_local_viewer = new Gtk::CheckButton ("local viewer");
  cb_solid = new Gtk::CheckButton ("solid");
  cb_sub_parts = new Gtk::CheckButton ("draw sub parts");

  cb_local_viewer->set_active (display.renderer->local_viewer ());
  cb_solid->set_active (display.renderer->solid ());
  cb_sub_parts->set_active (display.renderer->draw_sub_parts ());

  config_table->attach (*cb_local_viewer,
			0, 1, 2, 3,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*cb_solid,
			0, 1, 3, 4,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*cb_sub_parts,
			0, 1, 4, 5,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  cb_local_viewer->show ();
  cb_solid->show ();
  cb_sub_parts->show ();

  double surf_tol;
  double normal_tol;
  SiL_TriangMode t_mode;
  SiL_GridMode g_mode;
  int max_l;
  int min_l;
  double max_ll;
  double ar;
  SiL_AdjustMode am;
  sil_get_refinement (surf_tol,
		      normal_tol,
		      t_mode,
		      g_mode,
		      max_l,
		      min_l,
		      max_ll,
		      ar,
		      am);
  
  label_ref[0] = new Gtk::Label ("Surface tol.");
  adj_ref[0] = new Gtk::Adjustment (surf_tol,
				   0.0, 100.0,
				   0.1, 1.0,
				   0.0);
  sb_ref[0] = new Gtk::SpinButton (*adj_ref[0], 1.0, 2);
  label_ref[1] = new Gtk::Label ("Normal tol.");
  adj_ref[1] = new Gtk::Adjustment (normal_tol,
				    0.0, 100.0,
				    1.0, 5.0,
				    0.0);
  sb_ref[1] = new Gtk::SpinButton (*adj_ref[1], 1.0, 2);

  config_table->attach (*label_ref[0],
			1, 2, 2, 3,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_ref[0],
			2, 3, 2, 3,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*label_ref[1],
			5, 6, 2, 3,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_ref[1],
			6, 7, 2, 3,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  rb_tri_ref[0] = new Gtk::RadioButton ("TRIANG_NONE");
  rb_tri_ref[1] = new Gtk::RadioButton ("TRIANG_FRINGE_1");
  rb_tri_ref[1]->set_group (rb_tri_ref[0]->group ());
  rb_tri_ref[2] = new Gtk::RadioButton ("TRIANG_FRINGE_2");
  rb_tri_ref[2]->set_group (rb_tri_ref[0]->group ());
  rb_tri_ref[3] = new Gtk::RadioButton ("TRIANG_FRINGE_3");
  rb_tri_ref[3]->set_group (rb_tri_ref[0]->group ());
  rb_tri_ref[4] = new Gtk::RadioButton ("TRIANG_FRINGE_4");
  rb_tri_ref[4]->set_group (rb_tri_ref[0]->group ());
  rb_tri_ref[5] = new Gtk::RadioButton ("TRIANG_ALL");
  rb_tri_ref[5]->set_group (rb_tri_ref[0]->group ());

  rb_grid_ref[0] = new Gtk::RadioButton ("GRID_NONE");
  rb_grid_ref[1] = new Gtk::RadioButton ("GRID_INTERIOR");
  rb_grid_ref[1]->set_group (rb_grid_ref[0]->group ());
  rb_grid_ref[2] = new Gtk::RadioButton ("GRID_TO_EDGES");
  rb_grid_ref[2]->set_group (rb_grid_ref[0]->group ());

  rb_adj_ref[0] = new Gtk::RadioButton ("ADJUST_NONE");
  rb_adj_ref[1] = new Gtk::RadioButton ("ADJUST_NON_GRID");
  rb_adj_ref[1]->set_group (rb_adj_ref[0]->group ());
  rb_adj_ref[2] = new Gtk::RadioButton ("ADJUST_ALL");
  rb_adj_ref[2]->set_group (rb_adj_ref[0]->group ());

  for (int i=0; i<6; ++i) {
    config_table->attach (*rb_tri_ref[i],
			  i+1, i+2, 3, 4,
			  GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			  GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			  (gint)0, (gint)0);
    rb_tri_ref[i]->set_active (false);
    rb_tri_ref[i]->show ();
  }
  switch (t_mode) {
  case SIL_TRIANG_NONE:
    rb_tri_ref[0]->set_active (true);
    break;
  case SIL_TRIANG_FRINGE_1:
    rb_tri_ref[1]->set_active (true);
    break;
  case SIL_TRIANG_FRINGE_2:
    rb_tri_ref[2]->set_active (true);
    break;
  case SIL_TRIANG_FRINGE_3:
    rb_tri_ref[3]->set_active (true);
    break;
  case SIL_TRIANG_FRINGE_4:
    rb_tri_ref[4]->set_active (true);
    break;
  case SIL_TRIANG_ALL:
    rb_tri_ref[5]->set_active (true);
    break;
  }
  for (int i=0; i<3; ++i) {
    config_table->attach (*rb_grid_ref[i],
			  i+1, i+2, 4, 5,
			  GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			  GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			  (gint)0, (gint)0);
    rb_grid_ref[i]->set_active (false);
    rb_grid_ref[i]->show ();
  }
  switch (g_mode) {
  case SIL_GRID_NONE:
    rb_grid_ref[0]->set_active (true);
    break;
  case SIL_GRID_INTERIOR:
    rb_grid_ref[1]->set_active (true);
    break;
  case SIL_GRID_TO_EDGES:
    rb_grid_ref[2]->set_active (true);
    break;
  }
  for (int i=0; i<3; ++i) {
    config_table->attach (*rb_adj_ref[i],
			  i+4, i+5, 4, 5,
			  GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			  GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			  (gint)0, (gint)0);
    rb_adj_ref[i]->set_active (false);
    rb_adj_ref[i]->show ();
  }
  switch (am) {
  case SIL_ADJUST_NONE:
    rb_adj_ref[0]->set_active (true);
    break;
  case SIL_ADJUST_NON_GRID:
    rb_adj_ref[1]->set_active (true);
    break;
  case SIL_ADJUST_ALL:
    rb_adj_ref[2]->set_active (true);
    break;
  }
  label_ref[2] = new Gtk::Label ("Max grid lines");
  adj_ref[2] = new Gtk::Adjustment (max_l,
				    0.0, 1000.0,
				    1.0, 5.0,
				    0.0);
  sb_ref[2] = new Gtk::SpinButton (*adj_ref[2], 1.0, 0);
  config_table->attach (*label_ref[2],
			1, 2, 5, 6,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_ref[2],
			2, 3, 5, 6,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  label_ref[3] = new Gtk::Label ("Min grid lines");
  adj_ref[3] = new Gtk::Adjustment (min_l,
				    0.0, 1000.0,
				    1.0, 5.0,
				    0.0);
  sb_ref[3] = new Gtk::SpinButton (*adj_ref[3], 1.0, 0);
  config_table->attach (*label_ref[3],
			3, 4, 5, 6,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_ref[3],
			4, 5, 5, 6,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  label_ref[4] = new Gtk::Label ("Max line length");
  adj_ref[4] = new Gtk::Adjustment (max_ll,
				    0.0, 1000.0,
				    1.0, 5.0,
				    0.0);
  sb_ref[4] = new Gtk::SpinButton (*adj_ref[4], 1.0, 2);
  config_table->attach (*label_ref[4],
			5, 6, 5, 6,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_ref[4],
			6, 7, 5, 6,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  label_ref[5] = new Gtk::Label ("Grid aspect ratio");
  adj_ref[5] = new Gtk::Adjustment (ar,
				    0.0, 100.0,
				    1.0, 5.0,
				    0.0);
  sb_ref[5] = new Gtk::SpinButton (*adj_ref[5], 1.0, 2);
  config_table->attach (*label_ref[5],
			1, 2, 6, 7,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_ref[5],
			2, 3, 6, 7,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  for (int i=0; i<6; ++i) {
    sb_ref[i]->show ();
    label_ref[i]->set_justify (GTK_JUSTIFY_RIGHT);
    label_ref[i]->show();
  }

  label_gl[0] = new Gtk::Label ("Point size");
  adj_gl[0] = new Gtk::Adjustment (display.renderer->point_size (),
				   display.renderer->min_point_size (),
				   display.renderer->max_point_size (),
				   1.0, 5.0,
				   0.0);
  sb_gl[0] = new Gtk::SpinButton (*adj_gl[0], 1.0, 2);
  config_table->attach (*label_gl[0],
			3, 4, 6, 7,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_gl[0],
			4, 5, 6, 7,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  label_gl[1] = new Gtk::Label ("Line width");
  adj_gl[1] = new Gtk::Adjustment (display.renderer->line_width (),
				   display.renderer->min_line_width (),
				   display.renderer->max_line_width (),
				   1.0, 5.0,
				   0.0);
  sb_gl[1] = new Gtk::SpinButton (*adj_gl[1], 1.0, 2);
  config_table->attach (*label_gl[1],
			5, 6, 6, 7,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  config_table->attach (*sb_gl[1],
			6, 7, 6, 7,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);
  
  for (int i=0; i<2; ++i) {
    sb_gl[i]->show ();
    label_gl[i]->set_justify (GTK_JUSTIFY_RIGHT);
    label_gl[i]->show();
  }

  config_ok_button = new Gtk::Button ("OK");
  config_apply_button = new Gtk::Button ("Apply");
  config_cancel_button = new Gtk::Button ("Cancel");

  config_ok_button->clicked.connect (slot (this, &GUI::config_ok));
  config_apply_button->clicked.connect (slot (this, &GUI::config_apply));
  config_cancel_button->clicked.connect (slot (this, &GUI::config_cancel));

  config_hbox = new Gtk::HBox (FALSE, 0);

  config_hbox->pack_start (*config_ok_button, false, false, (gint)2);
  config_hbox->pack_start (*config_apply_button, false, false, (gint)2);
  config_hbox->pack_start (*config_cancel_button, false, false, (gint)2);
  
  config_table->attach (*config_hbox, 
			0, 1, 6, 7,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			GTK_FILL|GTK_EXPAND|GTK_SHRINK,
			(gint)0, (gint)0);

  config_ok_button->show ();
  config_apply_button->show ();
  config_cancel_button->show ();
  config_hbox->show ();

  config_table->show ();

  configwin->show ();

  redraw ();
  
  return;
}

void GUI::config_persp () {
  if (cb_persp->get_active ()) {
    cb_ortho->set_active (false);
    label_proj[0]->set ("angel");
    label_proj[1]->set ("aspect");
    sb_proj[0]->set_adjustment (*adj_proj[0]);
    sb_proj[0]->set_value (adj_proj[0]->get_value ());
    sb_proj[1]->set_adjustment (*adj_proj[1]);
    sb_proj[1]->set_value (adj_proj[1]->get_value ());
    sb_proj[2]->set_sensitive (false);
    sb_proj[3]->set_sensitive (false);
  } else {
    cb_ortho->set_active (true);
  }
  return;
}

void GUI::config_ortho () {
  if (cb_ortho->get_active ()) {
    cb_persp->set_active (false);
    label_proj[0]->set ("left");
    label_proj[1]->set ("right");
    sb_proj[0]->set_adjustment (*adj_proj[6]);
    sb_proj[0]->set_value (adj_proj[6]->get_value ());
    sb_proj[1]->set_adjustment (*adj_proj[7]);
    sb_proj[1]->set_value (adj_proj[7]->get_value ());
    sb_proj[2]->set_sensitive (true);
    sb_proj[3]->set_sensitive (true);
  } else {
    cb_persp->set_active (true);
  }
  return;
}

void GUI::config_ok ()
{
  config_apply ();
  config_cancel ();
  return;
}

void GUI::config_apply ()
{
  display.renderer->set_projection (adj_proj[0]->get_value (),
				    adj_proj[1]->get_value (),
				    adj_proj[6]->get_value (),
				    adj_proj[7]->get_value (),
				    adj_proj[3]->get_value (),
				    adj_proj[2]->get_value (),
				    adj_proj[4]->get_value (),
				    adj_proj[5]->get_value (),
				    cb_ortho->get_active ());
  display.renderer->set_local_viewer (cb_local_viewer->get_active ());
  display.renderer->set_solid (cb_solid->get_active ());
  display.renderer->set_draw_sub_parts (cb_sub_parts->get_active ());

  SiL_GridMode g_mode = SIL_GRID_NONE;
  if (rb_grid_ref[0]->get_active ()) {
    g_mode = SIL_GRID_NONE;
  } else if (rb_grid_ref[1]->get_active ()) {
    g_mode = SIL_GRID_INTERIOR;
  } else if (rb_grid_ref[2]->get_active ()) {
    g_mode = SIL_GRID_TO_EDGES;
  }
  SiL_TriangMode t_mode  = SIL_TRIANG_NONE;
  if (rb_tri_ref[0]->get_active ()) {
    t_mode = SIL_TRIANG_NONE;
  } else if (rb_tri_ref[1]->get_active ()) {
    t_mode = SIL_TRIANG_FRINGE_1;
  } else if (rb_tri_ref[2]->get_active ()) {
    t_mode = SIL_TRIANG_FRINGE_2;
  } else if (rb_tri_ref[3]->get_active ()) {
    t_mode = SIL_TRIANG_FRINGE_3;
  } else if (rb_tri_ref[4]->get_active ()) {
    t_mode = SIL_TRIANG_FRINGE_4;
  } else if (rb_tri_ref[5]->get_active ()) {
    t_mode = SIL_TRIANG_ALL;
  }
  SiL_AdjustMode am = SIL_ADJUST_NONE;
  if (rb_adj_ref[0]->get_active ()) {
    am = SIL_ADJUST_NONE;
  } else if (rb_tri_ref[1]->get_active ()) {
    am = SIL_ADJUST_NON_GRID;
  } else if (rb_tri_ref[2]->get_active ()) {
    am = SIL_ADJUST_ALL;
  }
  
  display.renderer->set_refinement (sb_ref[0]->get_value_as_float (),
				    sb_ref[1]->get_value_as_float (),
				    t_mode,
				    g_mode,
				    sb_ref[2]->get_value_as_int (),
				     sb_ref[3]->get_value_as_int (),
				    sb_ref[4]->get_value_as_float (),
				    sb_ref[5]->get_value_as_float (),
				    am);
  display.renderer->set_point_size (sb_gl[0]->get_value_as_float ());
  display.renderer->set_line_width (sb_gl[1]->get_value_as_float ());
  
  display.renderer->size (display.width (),
			  display.height ());

  refacet ();
  return;
}

void GUI::config_cancel ()
{
  if (configwin) {
    configwin->hide ();
    delete configwin;
    delete config_hbox;
    delete config_table;
    delete cb_persp;
    delete cb_ortho;
    for (int i=0; i<6; ++i) {
      delete sb_proj[i];
      delete adj_proj[i];
      delete label_proj[i];
    }
    delete adj_proj[6];
    delete adj_proj[7];
    for (int i=0; i<6; ++i) {
      delete sb_ref[i];
      delete adj_ref[i];
      delete label_ref[i];
    }
    for (int i=0; i<6; ++i) {
      delete rb_tri_ref[i];
    }
    for (int i=0; i<3; ++i) {
      delete rb_grid_ref[i];
    }
    for (int i=0; i<3; ++i) {
      delete rb_adj_ref[i];
    }
    delete cb_local_viewer;
    delete cb_solid;
    delete cb_sub_parts;
    delete config_ok_button;
    delete config_apply_button;
    delete config_cancel_button;
    configwin = NULL;
  }
  return;
}

gint GUI::config_close (GdkEventAny *e)
{
  config_cancel ();
  return TRUE;
}

void GUI::init_view ()
{
  display.renderer->init_view ();
  display.renderer->size (display.width (),
			   display.height ());
  redraw ();
  return;
}

void GUI::open ()
{
  if (file_sel) 
    return;

  file_sel = new Gtk::FileSelection ("Open File");
  file_replace_cb = new Gtk::CheckButton ("Replace Symbols");
  file_sil_cb = new Gtk::CheckButton ("Load SiL extensions");
  m_filetypes = new Gtk::Menu ();

  mi_n = 0;
  const char *c = App::instance () -> modules -> first_format_name ();
  while (c) {
    ++mi_n;
    c = App::instance () -> modules -> next_format_name ();
  }
  mi_filetypes = new Gtk::MenuItem *[mi_n];
  c = App::instance () -> modules -> first_format_name ();
  mi_n = 0;
  while (c) {
    mi_filetypes[mi_n] = new Gtk::MenuItem (App::instance () -> modules -> 
					    current_format_desc ());
    m_filetypes->append (*mi_filetypes[mi_n]);
    mi_filetypes[mi_n]->select.
      connect (bind (slot (this,
			   &GUI::sel_filetypes), (int)mi_n));
    mi_filetypes[mi_n]->show ();
    ++mi_n;
    c = App::instance () -> modules -> next_format_name ();
  }

  om_filetypes = new Gtk::OptionMenu ();
  om_filetypes->set_menu (*m_filetypes);
  m_filetypes->set_title ("File Types");
  filetypes_id = 0;
  file_sil_cb->set_active (sil_file_ext);
  file_replace_cb->set_active (replace_sym);
  file_sel->get_action_area () -> pack_end (*file_sil_cb,
					    false, false, 0);
  file_sel->get_action_area () -> pack_end (*file_replace_cb,
					    false, false, 0);
  file_sel->get_action_area () -> pack_start (*om_filetypes,
					      true, true, 0);
  file_sel->get_ok_button()->clicked.connect (slot (this, &GUI::open_ok));
  file_sel->get_cancel_button()->clicked.connect (slot (this,
  							&GUI::open_cancel));
  file_sel->delete_event.connect (slot (this, &GUI::open_close));
  file_sel->hide_fileop_buttons ();
  if (last_file_dir)
    file_sel->set_filename (last_file_dir);
  file_sil_cb->show ();
  file_replace_cb->show ();
  m_filetypes->show ();
  om_filetypes->show ();
  file_sel->show ();
  
  return;
}

void GUI::sel_filetypes (int i)
{
  filetypes_id = i;
  return;
}

void GUI::open_ok ()
{
  int l = file_sel->get_filename () . length ();
  char *name = new char[l + 1];
  strcpy (name, file_sel->get_filename().c_str());
  struct stat finfo;
  if (stat (name, &finfo) != 0 || !(S_ISREG (finfo.st_mode) ||
				    S_ISLNK (finfo.st_mode))) {
    delete []name;
    return;
  }
  for (--l; l > -1 && name[l] != '/'; l--);
  if (l != -1) {
    if (last_file_dir)
      delete []last_file_dir;
    last_file_dir = new char[l + 2];
    strncpy (last_file_dir, name, l);
    last_file_dir[l] = '/';
    last_file_dir[l+1] = '\0';
  }
  sil_file_ext = file_sil_cb->get_active ();
  replace_sym = file_replace_cb->get_active ();
  file_sel->hide ();
  delete file_sel;
  delete m_filetypes;
  for (int i = 0; i < mi_n; ++i)
    delete mi_filetypes[i];
  delete []mi_filetypes;
  file_sel = NULL;

  char *str = new char[strlen (name) + 14];
  strcpy (str, "opening file ");
  strcat (str, name);
  sil_msg ("open", str);
  delete []str;
  FILE *fp = fopen (name, "r");
  if (fp != NULL) {
    const char *c = App::instance () -> modules -> first_format_name ();
    int n = 0;
    while (c) {
      if (n == filetypes_id) {
	App::instance () -> modules -> load_file (fp, c, sil_file_ext,
						  replace_sym);
	sil_updated ();
	c = NULL;
      } else {
	c = App::instance () -> modules -> next_format_name ();
	++n;
      }
    }
    fclose (fp);
  } else {
    sil_msg_error ("open", "failed");
  }
  delete []name;

  refacet ();
  sil_updated ();

  return;
}

void GUI::open_cancel ()
{
  if (file_sel) {
    file_sel->hide ();
    delete file_sel;
    delete m_filetypes;
    for (int i = 0; i < mi_n; ++i)
      delete mi_filetypes[i];
    delete []mi_filetypes;
    file_sel = NULL;
  }
  return;
}

gint GUI::open_close (GdkEventAny *e)
{
  if (file_sel) {
    file_sel->hide ();
    delete file_sel;
    delete m_filetypes;
    for (int i = 0; i < mi_n; ++i)
      delete mi_filetypes[i];
    delete []mi_filetypes;
    file_sel = NULL;
  }
  return TRUE;
}

void GUI::save ()
{
  if (file_sel)
    return;
  file_sel = new Gtk::FileSelection ("Save Selection");
  file_sil_cb = new Gtk::CheckButton ("Save SiL extensions");
  m_filetypes = new Gtk::Menu ();

  mi_n = 0;
  const char *c = App::instance () -> modules -> first_format_name ();
  while (c) {
    ++mi_n;
    c = App::instance () -> modules -> next_format_name ();
  }
  mi_filetypes = new Gtk::MenuItem *[mi_n];
  c = App::instance () -> modules -> first_format_name ();
  mi_n = 0;
  while (c) {
    mi_filetypes[mi_n] = new Gtk::MenuItem (App::instance () -> modules ->
					    current_format_desc ());
    m_filetypes->append (*mi_filetypes[mi_n]);
    mi_filetypes[mi_n]->select. connect (bind (slot (this, 
						     &GUI::sel_filetypes),
					       (int)mi_n));
    mi_filetypes[mi_n]->show ();
    ++mi_n;
    c = App::instance () -> modules -> next_format_name ();
  }

  om_filetypes = new Gtk::OptionMenu ();
  om_filetypes->set_menu (*m_filetypes);
  m_filetypes->set_title ("File Types");
  filetypes_id = 0;
  file_sil_cb->set_active (sil_file_ext);
  file_sel->get_action_area () -> pack_end (*file_sil_cb,
  					    false, false, 0);
  file_sel->get_action_area () -> pack_start (*om_filetypes,
					      true, true, 0); 
  file_sel->get_ok_button()->clicked.connect (slot (this, &GUI::save_ok));
  file_sel->get_cancel_button()->clicked.connect (slot (this,
  							&GUI::save_cancel));
  file_sel->delete_event.connect (slot (this, &GUI::save_close));
  file_sel->show_fileop_buttons ();
  if (last_file_dir)
    file_sel->set_filename(last_file_dir);
  file_sil_cb->show ();
  m_filetypes->show ();
  om_filetypes->show ();
  file_sel->show ();
  return;
}

void GUI::save_ok ()
{
  int l = file_sel->get_filename () . length ();
  if (l == 0)
    return;
  char *name = new char[l + 1];
  strcpy (name, file_sel->get_filename () . c_str());
  struct stat finfo;
  if (stat (name, &finfo) == 0) {
    if (S_ISDIR (finfo.st_mode)) {
      delete []name;
      return;
    }
  }
  for (--l; l > -1 && name[l] != '/'; l--);
  if (l != -1) {
    if (last_file_dir)
      delete []last_file_dir;
    last_file_dir = new char[l + 2];
    strncpy (last_file_dir, name, l);
    last_file_dir[l] = '/';
    last_file_dir[l+1] = '\0';
  }
  sil_file_ext = file_sil_cb->get_active ();
  file_sel->hide();
  delete file_sel;
  for (int i = 0; i < mi_n; ++i)
    delete mi_filetypes[i];
  delete []mi_filetypes;
  delete m_filetypes;

  file_sel = NULL;
  
  char *str = new char[strlen (name) + 16];
  strcpy (str, "saving to file ");
  strcat (str, name);
  sil_msg ("save", str);
  delete []str;

  FILE *fp = fopen (name, "w");
  if (fp != NULL) {
    const char *c = App::instance () -> modules -> first_format_name ();
    int n = 0;
    while (c) {
      if (n == filetypes_id) {
	App::instance () -> modules -> save_file (fp, c, sil_file_ext);
	c = NULL;
      } else {
	c = App::instance () -> modules -> next_format_name ();
	++n;
      }
    }
    fclose (fp);
  } else {
    sil_msg_error ("save", "failed");
  }
  delete []name;

  return;
}

void GUI::save_cancel ()
{
  if (file_sel) {
    file_sel->hide();
    delete file_sel;
    delete m_filetypes;
    for (int i = 0; i < mi_n; ++i)
      delete mi_filetypes[i];
    delete []mi_filetypes;
    file_sel = NULL;
  }
  return;
}

gint GUI::save_close (GdkEventAny *e)
{
  if (file_sel) {
    file_sel->hide();
    delete file_sel;
    delete m_filetypes;
    for (int i = 0; i < mi_n; ++i)
      delete mi_filetypes[i];
    delete []mi_filetypes;
    file_sel = NULL;
  }
  return TRUE;
}

void GUI::plugin ()
{
  if (tool_win)
    return;
  tool_win = new ToolWindow ();
  tool_win->show ();
  return;
}

void GUI::update_tool_window ()
{
  if (tool_win)
    tool_win->update ();
  return;
}

void GUI::win_dest ()
{
  win_visible = false;
  return;
}

void GUI::exit ()
{
  Gtk::Main::quit ();
  return;
}

void GUI::refacet ()
{
  display.renderer->facet ();
  if (display.make_current ()) {
    display.renderer->draw ();
    display.renderer->show ();
  }
  display.swap_buffers ();
  return;
}

void GUI::redraw ()
{
  if (display.make_current ()) {
    display.renderer->draw ();
    display.renderer->show ();
  }
  display.swap_buffers ();
  return;
}

void GUI::refresh ()
{
  if (display.make_current ())
    display.renderer->show ();
  display.swap_buffers ();
  return;
}


void GUI::status_msg (string const &text)
{
  Gtk::Text_Helpers::Context cntxt = status_text.get_context ();
  cntxt.set_foreground (Gdk_Color ("Black"));
  status_text.insert (cntxt, text);

  return;
}

void GUI::status_msg (string const &name, string const &text)
{
  Gtk::Text_Helpers::Context cntxt = status_text.get_context ();
  cntxt.set_foreground (Gdk_Color ("DarkBlue"));
  status_text.insert (cntxt, name);
  status_text.insert (cntxt, ": ");
  cntxt.set_foreground (Gdk_Color ("Black"));
  status_text.insert (cntxt, text);
  status_text.insert (cntxt, "\n");
  return;
}

void GUI::status_msg_warn (string const &name, string const &text)
{
  Gtk::Text_Helpers::Context cntxt = status_text.get_context ();
  cntxt.set_foreground (Gdk_Color ("DarkBlue"));
  status_text.insert (cntxt, name);
  status_text.insert (cntxt, ": ");
  cntxt.set_foreground (Gdk_Color ("Orange"));
  status_text.insert (cntxt, "warning: ");
  cntxt.set_foreground (Gdk_Color ("Black"));
  status_text.insert (cntxt, text);
  status_text.insert (cntxt, "\n");
  return;
}

void GUI::status_msg_error (string const &name, string const &text)
{
  Gtk::Text_Helpers::Context cntxt = status_text.get_context ();
  cntxt.set_foreground (Gdk_Color ("DarkBlue"));
  status_text.insert (cntxt, name);
  status_text.insert (cntxt, ": ");
  cntxt.set_foreground (Gdk_Color ("Red"));
  status_text.insert (cntxt, "error: ");
  cntxt.set_foreground (Gdk_Color ("Black"));
  status_text.insert (cntxt, text);
  status_text.insert (cntxt, "\n");
  return;
}

void GUI::progress_bar (bool flag, bool cancel_flag)
{
  pb_progress.set_show_text (false);
  pb_progress.set_percentage (0.0);
  b_pcancel.set_sensitive (flag && cancel_flag);
  progress_flag = flag;
  progress_cancel_flag = false;
  while (gtk_main_iteration_do (FALSE));
  return;
}

void GUI::progress_bar_percentage (gfloat p)
{
  if (progress_flag) {
    while (gtk_main_iteration_do (FALSE));
    pb_progress.set_percentage (p);
    while (gtk_main_iteration_do (FALSE));
  }
  return;
}

void GUI::progress_bar_text (string const &text)
{
  if (progress_flag) {
    while (gtk_main_iteration_do (FALSE));
    pb_progress.set_format_string (text);
    pb_progress.set_show_text (true);
    while (gtk_main_iteration_do (FALSE));
  }
  return;
}

bool GUI::progress_cancel ()
{
  return progress_cancel_flag;
}

void GUI::pcancel_handler ()
{
  progress_cancel_flag = true;
  return;
}

/// Local Variables:
/// exe-comm: "./sil"
/// compile-command: "make"
/// End:
