// entity.hxx: Kernel base class.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: entity.hxx,v 1.13.2.1 2001/08/16 16:19:09 scmfcl Exp $

#ifndef ENTITY_HXX
#define ENTITY_HXX 1

#include "types.hxx"

class MemoryManager;

class Entity {
public:
  Entity ();
  virtual ~Entity ();
  bool is (EntityType t) const;
  bool is (Entity const *e) const;
  virtual string const type_id () const {
    return tid;
  }
  void use () {
    ++use_cnt;
    return;
  }
  void unuse () {
    assert (use_cnt > 0);
    --use_cnt;
  }
  unsigned long used () {
    return use_cnt;
  }
  inline EntityType type (unsigned int l = MAX_TYPE_LEVEL) const {
    unsigned long k = 0;
    while (k < l && types[k + 1] != SIL_UNKNOWN_TYPE)
      ++k;
    return types[k];
  }
  bool &visible () {
    return vis;
  }
  bool visible () const {
    return vis;
  }
protected:
  inline void type (unsigned int l, EntityType t) {
    assert (l < MAX_TYPE_LEVEL);
    types[l] = t;
    types[l+1] = SIL_UNKNOWN_TYPE;
    return;
  }
  void reset_use () {
    use_cnt = 0;
  }
  friend class MemoryManager;
private:
  static string const tid;
  EntityType *types;
  unsigned long use_cnt;
  bool vis;
};

#endif // !ENTITY_HXX

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
