// kernel_moudle.hxx: Kernel module.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: kernel_module.hxx,v 1.13.2.1 2001/08/16 16:19:09 scmfcl Exp $

#ifndef KERNEL_MODULE_HXX
#define KERNEL_MODULE_HXX 1

#include "module.hxx"

#include "kernel/memory.hxx"

class Boundary;
class Cell;
class Vertex;
class Edge;
class Face;
class Lump;
class Material;

class KernelModule : public Module {
public:
  KernelModule ();
  ~KernelModule ();
  void add_symbol (void *object, char *name, bool visible);
  void show (void *object);
  void hide (void *object);
  void facet_init ();
  void draw_init ();
  void facet_object (void *object);
  void draw_object (void *object, SimpleList<void> const &hirefs);
  void draw_fixed_object (void *object, SimpleList<void> const &hirefs);
  void del_object (void *object);
  //void load_file (FILE *fp, const char *format, bool ext, bool replace);
  //void save_file (FILE *fp, const char *format, bool ext);
  //const char **get_file_formats ();
  const char *name () const;
private:
  void draw_boundary (Entity *b, SimpleList<void> const &hirefs);
  void draw_cell (Entity *c, SimpleList<void> const &hirefs);
  void draw_vertex (Vertex *v, Material *m, bool inv,
		    SimpleList<void> const &hirefs);
  void draw_edge (Edge *e, Material *m, bool inv,
		  SimpleList<void> const &hirefs);
  void draw_face (Face *f, Material *m, bool inv,
		  SimpleList<void> const &hirefs);
  void draw_lump (Lump *l, Material *m, bool inv,
		  SimpleList<void> const &hirefs);
  static const char *nm;
  SimpleList<Entity> *drawn;
  double max_line_length;
};

#endif // !SCANNER_MODULE_HXX

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
