// memory.hxx: Kernel memory management.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: memory.hxx,v 1.9.2.1 2001/08/16 16:19:09 scmfcl Exp $

#ifndef MEMORY_HXX
#define MEMORY_HXX 1

#include "list.hxx"
#include "interface.hxx"
#include "kernel/entity.hxx"

class MemoryManager {
public:
  MemoryManager () {
    list = new SimpleList<Entity>;
  }
  ~MemoryManager ();
  void add (Entity *e) {
    list->add_first (e);
  }
  void del (Entity *e) {
    list->del (e);
  }
private:
  SimpleList<Entity> *list;
};

extern MemoryManager *MM;

inline void new_entity (Entity *e) {
  MM->add (e);
  return;
}

inline void del_entity (Entity *e) {
  MM->del (e);
  return;
}

inline void drop (Entity *e) {
  if (e) {
    e->unuse ();
    if (!e->used ())
      delete e;
  }
  return;
}

inline void take (Entity *e) {
  if (e)
    e->use ();
  return;
}

#endif // !MEMORY_HXX

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
