// types.hxx: Entity types.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: types.hxx,v 1.57.2.2 2001/08/16 16:19:09 scmfcl Exp $

#ifndef TYPES_HXX
#define TYPES_HXX 1

// Max level of derivation for active types.
#define MAX_TYPE_LEVEL 5

// Precision
#ifdef USE_DOUBLE
typedef double Real;
#else
typedef float Real;
#endif

// SiL kernel active types.
typedef enum {
  SIL_UNKNOWN_TYPE = 0,

  // Base type.
  SIL_ENTITY_TYPE,

  // Matrix type.
  SIL_RMATRIX_TYPE,

  // Combinatorics types.
  SIL_SOLID_TYPE,
  SIL_LUMP_TYPE,
  SIL_SHELL_TYPE,
  SIL_FACE_TYPE,
  SIL_LOOP_TYPE,
  SIL_EDGE_TYPE,
  SIL_VERTEX_TYPE,

  // Geometry types.
  SIL_GEOMETRY_TYPE,
  SIL_SPACE_TYPE,
  SIL_SURFACE_TYPE,
  SIL_PLANE_TYPE,
  SIL_CONE_TYPE,
  SIL_CYLINDER_TYPE,
  SIL_SPHERE_TYPE,
  SIL_TORUS_TYPE,
  SIL_CURVE_TYPE,
  SIL_STRAIGHT_TYPE,
  SIL_CIRCLE_TYPE,
  SIL_ELLIPSE_TYPE,
  SIL_SPLINECURVE_TYPE,
  SIL_POINT_TYPE,

  // Feature types.
  SIL_FEATURE_TYPE,
  SIL_SCALAR_TYPE,
  SIL_ANGLE_TYPE,
  SIL_LENGTH_TYPE,
  SIL_POSITION_TYPE,
  SIL_INTERSECTIONPOSITION_TYPE,
  SIL_ALIGNEDAXES_TYPE,
  SIL_DIRECTION_TYPE,
  SIL_DIRECTIONCONE_TYPE,
  SIL_DIRECTIONPLANE_TYPE,
  SIL_PARALLELLINE_TYPE,
  SIL_POINTCIRCLE_TYPE,
  SIL_PARTIALLYPLANEEQUAL_TYPE,
  SIL_PARTIALLYLINEEQUAL_TYPE,
  SIL_LOOPPOLYGON_TYPE,

  // Analyser types.
  SIL_ANALYSE_TYPE,
  SIL_RATIO_TYPE,
  SIL_PLANEPOSITION_TYPE,

  // Regularity types.
  SIL_REGULARITY_TYPE,
  SIL_CLUSTER_TYPE,
  SIL_SPECIALVALUE_TYPE,
  SIL_SPECIALRATIO_TYPE,
  SIL_DIRECTIONANGLE_TYPE,
  SIL_PLANARANGLEREGULAR_TYPE,
  SIL_CONICALANGLEREGULAR_TYPE,
  SIL_POINTLINE_TYPE,
  SIL_REGULARPOINTPOSITION_TYPE,
  SIL_REGULARPOINTCIRCLE_TYPE,
  SIL_SUBREGULARITY_TYPE,
  SIL_PARALLELDIRECTION_TYPE,
  SIL_ORTHOGONALDIRECTION_TYPE,
  SIL_ALTGEOMETRY_TYPE
} EntityType;

#endif // !TYPES_HXX

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
