// check.cc: sil check plugin.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: check.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (check, "$Id: check.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include "list.hxx"
#include "interface.hxx"
#include "acis/silapi.hxx"

class CheckWindow;

SimpleList<CheckWindow> check_windows;

class CheckWindow : public Gtk::Window {
public:
  CheckWindow (const char *name, ENTITY *e, bool c);
  ~CheckWindow ();
  void close ();  
  void check ();
  void debug ();
private:
  Gdk_Font check_font;
  Gtk::ScrolledWindow check_window;
  Gtk::Text check_text;
  Gtk::Button close_button;
  Gtk::VBox vbox;
  string tmpfile;
  const char *sym_name;
  ENTITY *ent;
  bool color;
  ENTITY_LIST *el;
};

CheckWindow::CheckWindow (const char *name, ENTITY *e, bool c) 
  : Gtk::Window (GTK_WINDOW_TOPLEVEL), 
  check_font ("fixed"),
  check_window (),
  check_text (),
  close_button ("Close"),
  vbox (FALSE, 0)
{
  ent = e;
  sym_name = name;
  color = c;
  el = NULL;
  // tmp file
  tmpfile = sil_tmp_filename ();

  check_window.set_usize (400, 300);
  check_text.set_editable (FALSE);
  check_text.set_word_wrap (TRUE);
  check_text.set_line_wrap (TRUE);
  check_window.add (check_text);
  check_window.set_hadjustment (*check_text.get_hadjustment ());
  check_window.set_vadjustment (*check_text.get_vadjustment ());
  check_window.set_policy (GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

  vbox.pack_start (check_window, true, true, (gint)2);
  vbox.pack_start (close_button, false, false, (gint)2);
  add (vbox);

  close_button.clicked.connect (slot (this, &CheckWindow::close));
  destroy.connect (slot (this, &CheckWindow::close));

  show_all ();
}

CheckWindow::~CheckWindow()
{
  check_windows.del (this);
  if (el) {
    el->init();
    ENTITY *l = el->next();
    while (l) {
      sil_hilite (false, sym_name, (void *)l);
      l = el->next();
    }
    sil_redraw ();
    delete el;
    el = NULL;
  }
  if (el)
    delete el;
  unlink (tmpfile.c_str ());
}

void CheckWindow::close ()
{
  hide_all ();
  destroy ();
  return;
}

void CheckWindow::check ()
{
  sil_msg ("check", sym_name);

  if (el)
    delete el;
  ENTITY_LIST *el = new ENTITY_LIST ();
  FILE *df = fopen (tmpfile.c_str (), "w");
  sil_acis_check (api_check_entity (ent, el, df),
		  "check", "check entity");
  fclose (df);

  check_text.freeze ();

  Gtk::Text_Helpers::Context cntxt = check_text.get_context ();
  cntxt.set_foreground (Gdk_Color ("black"));
  check_text.insert (cntxt, sym_name);
  check_text.insert (cntxt, "\n\n");

  char *buf = new char[512];
  df = fopen (tmpfile.c_str (), "r");
  while (!feof (df)) {
    size_t bs = fread (buf, sizeof(char), 511, df);
    buf[bs] = '\0';
    cntxt.set_foreground (Gdk_Color ("black"));
    check_text.insert (cntxt, buf);
  }
  fclose (df);

  check_text.thaw ();

  delete []buf;
  unlink (tmpfile.c_str ());

  if (color) {
    el->init();
    ENTITY *l = el->next();
    while (l) {
      sil_hilite (true, sym_name, (void *)l);
      l = el->next();
    }
    sil_redraw ();
  } else {
    delete el;
    el = NULL;
  }

  return;
}

void CheckWindow::debug ()
{
  sil_msg ("debug", sym_name);

  FILE *df = fopen (tmpfile.c_str (), "w");
  debug_entity (ent, df);
  fclose (df);

  check_text.freeze ();

  Gtk::Text_Helpers::Context cntxt = check_text.get_context ();
  cntxt.set_foreground (Gdk_Color ("black"));
  check_text.insert (cntxt, sym_name);
  check_text.insert (cntxt, "\n\n");

  char *buf = new char[512];
  df = fopen (tmpfile.c_str (), "r");
  while (!feof (df)) {
    size_t bs = fread (buf, sizeof(char), 511, df);
    buf[bs] = '\0';
    cntxt.set_foreground (Gdk_Color ("black"));
    check_text.insert (cntxt, buf);
  }
  fclose (df);

  check_text.thaw ();

  delete []buf;
  unlink (tmpfile.c_str ());

  return ;
}

void check_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (sil_symbol_is (s, "acis")) {
      CheckWindow *w = manage (new CheckWindow 
			       (sil_get_symbol_name (s),
				(ENTITY *)sil_get_symbol_data (s),
				true));
      check_windows.add_first (w);
      w->check ();
    }
    s = sl->next ();
  }
  return;
}

void unmark_check_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (sil_symbol_is (s, "acis")) {
      CheckWindow *w = manage (new CheckWindow 
			       (sil_get_symbol_name (s),
				(ENTITY *)sil_get_symbol_data (s),
				false));
      check_windows.add_first (w);
      w->check ();
    }
    s = sl->next ();
  }
  return;
}

void debug_symbol (SimpleList<Symbol const> *sl)
{
  Symbol const *s = sl->first ();
  while (s) {
    if (sil_symbol_is (s, "acis")) {
      CheckWindow *w = manage (new CheckWindow 
			       (sil_get_symbol_name (s),
				(ENTITY *)sil_get_symbol_data (s),
				false));
      check_windows.add_first (w);
      w->debug ();
    }
    s = sl->next ();
  }
  return;
}

SIL_PLUGIN ()
{
  sil_register_tool (&check_symbol, "Check Color", "0.1",
		     "Frank C Langbein", "1999-2000 Frank C Langbein",
		     "Check ACIS ENTITY symbols.", "acis", "acis");
  sil_register_tool (&unmark_check_symbol, "Check", "0.1",
		     "Frank C Langbein", "1999-2000 Frank C Langbein",
		     "Check ACIS ENTITY symbols.", "acis", "acis");
  sil_register_tool (&debug_symbol, "Debug", "0.1",
		     "Frank C Langbein", "1999-2000 Frank C Langbein",
		     "Debug ACIS ENTITY symbols.", "acis", "acis");
  return;
}

SIL_CLOSE_PLUGIN ()
{
  CheckWindow *w = check_windows.unlink_first ();
  while (w) {
    w->hide_all ();
    w->destroy ();
    w = check_windows.unlink_first ();
  }
  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
