// construct.cc: construct example.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: construct.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (construct, "$Id: construct.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include "interface.hxx"
#include "acis/silapi.hxx"

inline VERTEX *make_vertex(position a) 
{
  return new VERTEX (new APOINT (a));
}

inline EDGE *make_edge(VERTEX *a, VERTEX *b)
{

  return new EDGE (a, b,
		   new STRAIGHT(a->geometry()->coords(), 
				normalise(b->geometry()->coords() -
					  a->geometry()->coords())),
		   false);
}

FACE *make_polygon (VERTEX **vl, unsigned int n)
{
  if (n < 3)
    return NULL;
  unsigned int i = 0;
  EDGE **e = new EDGE *[n];
  COEDGE **c = new COEDGE *[n];
  while (i < n) {    
    VERTEX *a = vl[i];
    VERTEX *b = vl[(i+1)%n];
    e[i] = new EDGE (a, b,
		     new STRAIGHT (a->geometry()->coords (),
				   normalise(b->geometry()->coords() -
					     a->geometry()->coords())),
		     FORWARD);
    c[i] = new COEDGE (e[i], FORWARD, NULL, NULL);
    e[i]->set_coedge (c[i]);
    ++i;
  }
  c[0]->set_next (c[1]);
  c[0]->set_previous (c[n-1]);
  i = 1;
  while (i < n - 1) {
    c[i]->set_next (c[i+1]);
    c[i]->set_previous (c[i-1]);
    ++i;
  }
  c[n-1]->set_next (c[0]);
  c[n-1]->set_previous (c[n-2]);

  LOOP *l = new LOOP(c[0], NULL);

  vector n1 = vl[1]->geometry()->coords() - vl[0]->geometry()->coords();
  vector n2 = vl[n-1]->geometry()->coords() - vl[0]->geometry()->coords();
  vector o = vector( n1.y() * n2.z() - n1.z() * n2.y(),
		     n1.z() * n2.x() - n1.x() * n2.z(),
		     n1.x() * n2.y() - n1.y() * n2.x() );
  FACE *f = new FACE(l, NULL, 
		     new PLANE (vl[0]->geometry()->coords(),
				normalise(o)),
		     FORWARD);
  delete []e;
  delete []c;
  return f;
}

void join_face (FACE *a, FACE *b)
{
  // Find vertices
  ENTITY_LIST *vl = new ENTITY_LIST;

  LOOP *la = a->loop ();
  while (la) {
    COEDGE *ca = la->start ();
    COEDGE *caf = ca;
    while (ca) {
      // FIXME: check for duplicates
      vl->init ();
      VERTEX *v = (VERTEX *)vl->next ();
      bool add_new = true;
      while (v) {
	if (v->geometry()->coords() == 
	    ca->edge()->start()->geometry()->coords()) {
	  add_new = false;
	  if (v != ca->edge () -> start ()) {
	    ca->edge()->start()->delete_edge(ca->edge());
	    ca->edge()->set_start(v);
	    v->set_edge (ca->edge());
	  }
	}
	v = (VERTEX *)vl->next ();
      }
      if (add_new)
	vl->add (ca->start());
      vl->init ();
      v = (VERTEX *)vl->next ();
      add_new = true;
      while (v) {
	if (v->geometry()->coords() == 
	    ca->edge()->end()->geometry()->coords()) {
	  add_new = false;
	  if (v != ca->edge()->end()) {
	    ca->edge()->end()->delete_edge(ca->edge());
	    ca->edge()->set_end(v);
	    v->set_edge (ca->edge());
	  }
	}
	v = (VERTEX *)vl->next ();
      }
      if (add_new)
	vl->add (ca->end());
      ca = ca->next();
      if (ca == caf)
	ca = NULL;
    }
    la = la->next ();
  }

  LOOP *lb = b->loop ();
  while (lb) {
    COEDGE *cb = lb->start ();
    COEDGE *cbf = cb;
    while (cb) {
      // FIXME: check for duplicates
      vl->init ();
      VERTEX *v = (VERTEX *)vl->next ();
      bool add_new = true;
      while (v) {
	if (v->geometry()->coords() == 
	    cb->edge()->start()->geometry()->coords()) {
	  add_new = false;
	  if (v != cb->edge()->start()) {
	    cb->edge()->start()->delete_edge(cb->edge());
	    cb->edge()->set_start(v);
	    v->set_edge (cb->edge());
	  }
	}
	v = (VERTEX *)vl->next ();
      }
      if (add_new)
	vl->add (cb->edge()->start());
      vl->init ();
      v = (VERTEX *)vl->next ();
      add_new = true;
      while (v) {
	if (v->geometry()->coords() == 
	    cb->edge()->end()->geometry()->coords()) {
	  add_new = false;
	  if (v != cb->edge()->end()) {
	    cb->edge()->end()->delete_edge(cb->edge());
	    cb->edge()->set_end(v);
	    v->set_edge (cb->edge());
	  }
	}
	v = (VERTEX *)vl->next ();
      }
      if (add_new)
	vl->add (cb->edge()->end());
      cb = cb->next ();
      if (cb == cbf)
	cb = NULL;
    }
    lb = lb->next ();
  }
  
  delete vl;

  la = a->loop();
  while (la) {
    COEDGE *ca = la->start ();
    COEDGE *caf = ca;
    while (ca) {
      lb = b->loop ();
      while (lb) {
	COEDGE *cb = lb->start ();
	COEDGE *cbf = cb;
	bool reverse = false;
	while (cb) {
	  if (cb->start()->geometry()->coords() ==
	      ca->end()->geometry()->coords() &&
	      cb->end()->geometry()->coords() ==
	      ca->start()->geometry()->coords()) {
	    if (ca->edge() != cb->edge()) {
	      cb->set_edge (ca->edge());
	      if (ca->sense() == FORWARD)
		cb->set_sense (REVERSED);
	      else
		cb->set_sense (FORWARD);
	      cb->set_partner(ca);
	      ca->set_partner(cb);
	    }
	  }
	  // Reversed!!!
	  if (cb->end()->geometry()->coords() ==
	      ca->end()->geometry()->coords() &&
	      cb->start()->geometry()->coords() ==
	      ca->start()->geometry()->coords()) {
	    if (ca->edge() != cb->edge()) {
	      cb->set_edge (ca->edge());
	      cb->set_partner(ca);
	      ca->set_partner(cb);
	      reverse = true;
	    }
	  }
	  cb = cb->next();
	  if (cb == cbf)
	    cb = NULL;
	}
	if (reverse) {
	  cb = lb->start();
	  cbf = cb;
	  while (cb) {
	    if (cb->sense() == FORWARD)
	      cb->set_sense (REVERSED);
	    else
	      cb->set_sense (FORWARD);
	    COEDGE *cnext = cb->next ();
	    cb->set_next (cb->previous());
	    cb->set_previous (cnext);
	    cb = cnext;
	    if (cb == cbf)
	      cb = NULL;
	  }
	  PLANE *p = ((PLANE *)b->geometry());
	  p->set_normal(-p->normal());
	  b->set_geometry (p);
	}
	lb = lb->next ();
      }
      ca = ca->next ();
      if (ca == caf) 
	ca = NULL;
    }
    la = la->next ();
  }


  return;
}

void gconstruct ()
{

  API_BEGIN;
  
  VERTEX *b[4] = {
    make_vertex (position (0.0,0.0,0.0)),
    make_vertex (position (20.0,0.0,0.0)),
    make_vertex (position (20.0,20.0,0.0)),
    make_vertex (position (0.0,20.0,0.0))
  };
  FACE *B = make_polygon (b, 4);

  VERTEX *f1[3] = {
    make_vertex (position (0.0,0.0,0.0)),
    make_vertex (position (20.0,0.0,0.0)),
    make_vertex (position (10.0,10.0,20.0))
  };
  FACE *F1 = make_polygon (f1, 3);

  VERTEX *f2[3] = {
    make_vertex (position (20.0,0.0,0.0)),
    make_vertex (position (20.0,20.0,0.0)),
    make_vertex (position (10.0,10.0,20.0))
  };
  FACE *F2 = make_polygon (f2, 3);

  VERTEX *f3[3] = {
    make_vertex (position (20.0,20.0,0.0)),
    make_vertex (position (0.0,20.0,0.0)),
    make_vertex (position (10.0,10.0,20.0))
  };
  FACE *F3 = make_polygon (f3, 3);

  VERTEX *f4[3] = {
    make_vertex (position (0.0,20.0,0.0)),
    make_vertex (position (0.0,0.0,0.0)),
    make_vertex (position (10.0,10.0,20.0))
  };
  FACE *F4 = make_polygon (f4, 3);

  B->set_next (F1);
  F1->set_next (F2);
  F2->set_next (F3);
  F3->set_next (F4);

  join_face (F1, B);
  join_face (F2, B);
  join_face (F3, B);
  join_face (F4, B);
  join_face (F2, F3);
  join_face (F3, F4);
  join_face (F4, F1);
  join_face (F1, F2);

  SHELL *S = new SHELL(B, NULL, NULL);
  
  LUMP *L = new LUMP (S, NULL);
  S->set_lump (L);
  BODY *BB = new BODY (L);
  while (!sil_add_symbol ("pyramid", BB, "acis", true)) {
    sil_del_symbol ("pyramid");
  }
  sil_acis_check (api_set_material (BB,
				    0.0, 0.0, 0.8, 1.0,
				    0.0, 0.0, 0.8, 1.0,
				    0.0, 0.0, 0.0, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    1.0),
		  "construct", "set material");

  API_END;

  sil_refacet ();
  sil_updated ();

  return;
}

SIL_PLUGIN ()
{
  sil_register_tool (&gconstruct, "construct", "0.1",
		     "Frank C Langbein", "1999 Frank C Langbein",
		     "Construction example.", "acis", "acis");
  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
