// examples.cc: ACIS examples.
//
// Copyright (C) 2000 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: examples.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (examples, "$Id: examples.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"
#include "interface.hxx"
#include "acis/light.hxx"
#include "acis/silapi.hxx"
#include "base/base.hxx"

#define PI 3.14159265358979323846

void example1 ()
{
  BODY *B;

  sil_acis_check (api_make_torus ( 20, 4, B),
		  "example", "make torus");
  sil_acis_check (api_set_material (B,
				    0.7, 0.0, 0.0, 0.7,
				    0.7, 0.0, 0.0, 0.7,
				    0.0, 0.0, 0.0, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    1.0),
		  "example", "set material");
  while (!sil_add_symbol ("Torus", (void *)B, "acis", true)) {
    sil_del_symbol ("Torus");
  }

  position pts[4]; 
  pts[0] = position (0, 0, 0); 
  pts[1] = position (12, 0, 0); 
  pts[2] = position (12, 6, 0); 
  pts[3] = position (0, 6, 0); 
  EDGE *my_linear[4]; 
  sil_acis_check (api_curve_line( pts[0], pts[1], my_linear[0]),
		  "example", "curve line");
  sil_acis_check (api_curve_line( pts[1], pts[2], my_linear[1]),
		  "example", "curve line");
  sil_acis_check (api_curve_line( pts[2], pts[3], my_linear[2]),
		  "example", "curve line");
  sil_acis_check (api_curve_line( pts[3], pts[0], my_linear[3]),
		  "example", "curve line");
  BODY *w = NULL; 
  sil_acis_check (api_make_ewire (4, my_linear, w),
		  "example", "make ewire");
  while (!sil_add_symbol ("Wire", (void *)w,  "acis", true)) {
    sil_del_symbol ("Wire");
  }

  BODY *a, *b, *aa, *aaa;
  sil_acis_check (api_solid_block (position(0,0,0), position (9,9,9), a),
		  "example", "solid block");
  sil_acis_check (api_solid_block (position(0,0,0), position (9,9,-9), aa),
  		  "example", "solid block2");
  sil_acis_check (api_solid_block (position(0,0,0), position (9,-9,9), aaa),
  		  "example", "solid block2");
  sil_acis_check (api_solid_sphere (position(0,0,0), 6, b),
		  "example", "solid sphere");
  sil_acis_check (api_subtract (a, b),
		  "example", "subtract");
  sil_acis_check (api_subtract (aa, b),
  		  "example", "subtract");
  sil_acis_check (api_subtract (aaa, b),
  		  "example", "subtract");
  sil_acis_check(api_set_material (b,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.9, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("Sphere", (void *)b, "acis", true)) {
    sil_del_symbol ("Sphere");
  }

  LIGHT *l;
  sil_acis_check (api_make_light (0.0, 0.0, 0.0, 1.0,
				  1.0, 1.0, 1.0, 1.0,
				  1.0, 1.0, 1.0, 1.0,
				  50.0, 0.0, 50.0, 0.0,
				  0.0, 0.0, -1.0,
				  0.0,
				  180.0,
				  0.1,
				  0.0,
				  0.0, l),
		  "example", "creating light");
  while (!sil_add_symbol ("Example Light", (void *)l, "acis", true)) {
    sil_del_symbol ("Example Light");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example2 ()
{
  BODY *B, *B2;

  sil_acis_check (api_make_pyramid ( 20, 20, 20, 5, 3, B),
		  "example2", "make pyramid");
  sil_acis_check (api_set_material (B,
				    0.7, 0.0, 0.0, 1.0,
				    0.7, 0.0, 0.0, 1.0,
				    0.8, 0.3, 0.3, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    20.0),
		  "example2", "set material");
  while (!sil_add_symbol ("Pyramid3", (void *)B, "acis", true)) {
    sil_del_symbol ("Pyramid3");
  }

  sil_acis_check (api_make_pyramid ( 20, 20, 20, 5, 4, B),
		  "example2", "make pyramid");
  sil_acis_check (api_set_material (B,
				    0.7, 0.0, 0.0, 1.0,
				    0.7, 0.0, 0.0, 1.0,
				    0.8, 0.3, 0.3, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    20.0),
		  "example2", "set material");
  while (!sil_add_symbol ("Pyramid4", (void *)B, "acis", true)) {
    sil_del_symbol ("Pyramid4");
  }

  sil_acis_check (api_make_pyramid ( 15, 15, 15, 5, 6, B),
		  "example2", "make pyramid");
  sil_acis_check (api_transform_entity (B, 
					rotate_transf (35.0 / 180.0 * PI, 
						       vector (0, 0, 1)) *
					translate_transf (vector (0.0, 0.0,
								  -7.5))),
		  "example2", "api_transform_entity");
  sil_acis_check (api_make_pyramid ( 30, 30, 30, 10, 5, B2),
		  "example2", "make pyramid");
  sil_acis_check (api_subtract (B, B2),
		  "example2", "subtract");
  sil_acis_check (api_set_material (B2,
				    0.7, 0.0, 0.0, 1.0,
				    0.7, 0.0, 0.0, 1.0,
				    0.8, 0.3, 0.3, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    20.0),
		  "example2", "set material");
  while (!sil_add_symbol ("Pyramid56", (void *)B2, "acis", true)) {
    sil_del_symbol ("Pyramid56");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example3 ()
{
  BODY *B, *B2;

  sil_acis_check (api_make_prism (6.0, 10.0, 10.0, 8, B),
		  "example3", "make prism");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, 3.0),
					   position (10.0, 0.0, 3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, 3.0),
					   position (10.0, 0.0, 3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (90.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example3", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, 3.0),
					   position (10.0, 0.0, 3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (180.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example3", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, 3.0),
					   position (10.0, 0.0, 3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (270.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example3", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, -3.0),
					   position (10.0, 0.0, -3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (45.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example3", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, -3.0),
					   position (10.0, 0.0, -3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (135.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example3", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, -3.0),
					   position (10.0, 0.0, -3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (225.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example3", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (3.0, 0.0, -3.0),
					   position (10.0, 0.0, -3.0),
					   1.0, 1.0, 1.0, NULL, B2),
		  "example3", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (315.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example3", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example3", "subtract");

  sil_acis_check (api_set_material (B,
				    0.7, 0.0, 0.0, 1.0,
				    0.7, 0.0, 0.0, 1.0,
				    0.8, 0.3, 0.3, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    20.0),
		  "example3", "set material");

  while (!sil_add_symbol ("PlanarHolesBlock", (void *)B, "acis", true)) {
    sil_del_symbol ("PlanarHolesBlock");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example4 ()
{
  BODY *B, *B2;

  sil_acis_check (api_make_prism (6.0, 10.0, 10.0, 8, B),
		  "example4", "make prism");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (90.0 / 180.0 * PI, 
						       vector (0, 0, 2))),
		  "example4", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (180.0 / 180.0 * PI, 
						       vector (0, 0, 2))),
		  "example4", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (270.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example4", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, -5.0),
					   position (10.0, 0.0, -3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (45.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example4", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, -5.0),
					   position (10.0, 0.0, -3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (135.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example4", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, -5.0),
					   position (10.0, 0.0, -3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (225.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example4", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, -5.0),
					   position (10.0, 0.0, -3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example4", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (315.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example4", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example4", "subtract");

  sil_acis_check (api_set_material (B,
				    0.7, 0.0, 0.0, 1.0,
				    0.7, 0.0, 0.0, 1.0,
				    0.8, 0.3, 0.3, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    20.0),
		  "example4", "set material");

  while (!sil_add_symbol ("ConicalRegBlock", (void *)B, "acis", true)) {
    sil_del_symbol ("ConicalRegBlock");
  }

  sil_refacet ();
  sil_updated ();

  return;
}


#define N 3
#define dN 6
#define M 5
#define dM 4

void example5 ()
{
  BODY *b1, *c1;
  double a = (double)(dN * N);
  double b = (double)(dM * M);
  sil_acis_check (api_solid_block (position (-a, -b, -2), 
				   position (a, b, 2), 
				   b1),
		  "example", "solid block");
  for (int j = -N + 1; j < N; ++j) {
    for (int i = -M + 1; i < M; ++i) {
      sil_acis_check (api_solid_cylinder_cone (position(dN * i,
							dM * j, 
							-2), 
					       position(dN * i,
							dM * j,
							2), 
					       1.5,1.5,1.5, NULL, c1),
		      "example", "solid cylinder cone");
      sil_acis_check (api_subtract (c1, b1),
		      "example", "subtract");
    }
  }
  sil_acis_check(api_set_material (b1,
				   0.7, 0.0, 0.7, 1.0,
				   0.7, 0.0, 0.7, 1.0,
				   0.9, 0.0, 0.9, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("BlockN", (void *)b1, "acis", true)) {
    sil_del_symbol ("BlockN");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example6 ()
{
  BODY *b1, *b2, *b3, *c1, *c2, *c3, *c4;
  sil_acis_check (api_solid_block (position(-5,-5,-2), position (5,5,2), b1),
		  "example", "solid block");
  sil_acis_check (api_solid_block (position(-4,-2,-2), position (4,2,2), b2),
		  "example", "solid block");
  sil_acis_check (api_subtract (b2, b1),
		  "example", "subtract");
  sil_acis_check (api_solid_block (position(-2,-3,-2), position (2,4,2), b3),
		  "example", "solid block");
  sil_acis_check (api_subtract (b3, b1),
		  "example", "subtract");
  sil_acis_check (api_solid_cylinder_cone (position(-2,-2,-2), 
					   position(-2,-2,2), 
					   2.0,1.0,2.0, NULL, c1),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (c1, b1),
		  "example", "subtract");
  sil_acis_check (api_solid_cylinder_cone (position(2,-2,-2), 
					   position(2,-2,2), 
					   2.0,1.0,2.0, NULL, c2),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (c2, b1),
		  "example", "subtract");
  sil_acis_check (api_solid_cylinder_cone (position(-2,2,-2), 
					   position(-2,2,2), 
					   2.0,2.0,2.0, NULL, c3),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (c3, b1),
		  "example", "subtract");
  sil_acis_check (api_solid_cylinder_cone (position(2,2,-2), 
					   position(2,2,2), 
					   2.0,2.0,2.0, NULL, c4),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (c4, b1),
		  "example", "subtract");
  sil_acis_check(api_set_material (b1,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.9, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("BlockCylindricalHole", (void *)b1, "acis", true)) {
    sil_del_symbol ("BlockCylindricalHole");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example7 ()
{
  BODY *b1, *c1;

  sil_acis_check (api_solid_block (position(-5,-5,-2), position (5,5,2), b1),
		  "example", "solid block");
  sil_acis_check (api_solid_cylinder_cone (position(0,0,-2), 
					   position(0,0,2), 
					   4.0,4.0,4.0, NULL, c1),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (c1, b1),
		  "example", "subtract");
  sil_acis_check(api_set_material (b1,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.9, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("BlockRoundHole", (void *)b1, "acis", true)) {
    sil_del_symbol ("BlockRoundHole");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example8 ()
{
  BODY *b1, *c1;

  sil_acis_check (api_solid_block (position(-5,-5,-2), position (5,5,2), b1),
		  "example", "solid block");
  sil_acis_check (api_solid_cylinder_cone (position(-2,0,0), 
					   position(-2,0,2), 
					   1.0,1.0,1.0, NULL, c1),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (c1, b1),
		  "example", "subtract");
  sil_acis_check (api_solid_cylinder_cone (position(0,2,-2), 
					   position(0,2,0), 
					   1.0,1.0,1.0, NULL, c1),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (c1, b1),
		  "example", "subtract");
  sil_acis_check(api_set_material (b1,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.7, 1.0,
				   0.0, 0.0, 0.9, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("BlockTwoHole", (void *)b1, "acis", true)) {
    sil_del_symbol ("BlockTwoHole");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example9 ()
{
  BODY *B, *c1;

  sil_acis_check (api_solid_cylinder_cone (position(0,0,10), 
					   position(0,0,-10), 
					   5.0,5.0,5.0, NULL, B),
		  "example", "solid cylinder cone");
  sil_acis_check (api_solid_torus (position (0, 0, -8),
				   5, 1, c1),
		  "example", "solid torus");
  sil_acis_check (api_subtract (c1, B),
		  "example", "subtract");
  sil_acis_check (api_solid_torus (position (0, 0, -5),
				   5, 1, c1),
		  "example", "solid torus");
  sil_acis_check (api_subtract (c1, B),
		  "example", "subtract");
  sil_acis_check (api_solid_torus (position (0, 0, -2),
				   5, 1, c1),
		  "example", "solid torus");
  sil_acis_check (api_subtract (c1, B),
		  "example", "subtract");
  sil_acis_check (api_solid_torus (position (0, 0, 1),
				   5, 1, c1),
		  "example", "solid torus");
  sil_acis_check (api_subtract (c1, B),
		  "example", "subtract");

  sil_acis_check (api_solid_torus (position (0, 0, 4),
				   5, 1, c1),
		  "example", "solid torus");
  sil_acis_check (api_subtract (c1, B),
		  "example", "subtract");
  sil_acis_check (api_solid_torus (position (0, 0, 7),
				   5, 1, c1),
		  "example", "solid torus");
  sil_acis_check (api_subtract (c1, B),
		  "example", "subtract");
  sil_acis_check (api_solid_torus (position (0, 0, 10),
				   5, 1, c1),
		  "example", "solid torus");
  sil_acis_check (api_subtract (c1, B),
		  "example", "subtract");
  sil_acis_check(api_set_material (B,
				   0.7, 0.0, 0.0, 1.0,
				   0.7, 0.0, 0.0, 1.0,
				   0.9, 0.0, 0.0, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("ToriCylinder", (void *)B, "acis", true)) {
    sil_del_symbol ("ToriCylinder");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example10 ()
{
  BODY *b1, *c1;

  sil_acis_check (api_solid_block (position(-10,-10,-2), position (10,10,2), 
				   b1),
		  "example", "solid block");
  for (int j = 1; j < 5; ++j) {
    for (int i = 1; i < 5; ++i) {
      sil_acis_check (api_solid_cylinder_cone (position(-10 + 4 * i,
							-10 + 4 * j, 
							-2), 
					       position(-10 + 4 * i,
							-10 + 4 * j,
							2), 
					       1.5,1.5,1.5, NULL, c1),
		      "example", "solid cylinder cone");
      sil_acis_check (api_subtract (c1, b1),
		      "example", "subtract");
    }
  }
  sil_acis_check(api_set_material (b1,
				   0.7, 0.0, 0.7, 1.0,
				   0.7, 0.0, 0.7, 1.0,
				   0.9, 0.0, 0.9, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("HoleBlock", (void *)b1, "acis", true)) {
    sil_del_symbol ("HoleBlock");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example11 ()
{
  BODY *B1, *B2;
  sil_acis_check (api_solid_block (position(-5,-5,-2), position (5,5,2), B1),
		  "example", "solid block");
  sil_acis_check (api_solid_block (position(-2,-2,-2), position (2,2,2), B2),
		  "example", "solid block");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (PI / 4.0, 
						       vector (0, 0, 1)) *
					translate_transf (vector (5,5,0))),
		  "example", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B1),
		  "example", "subtract");
  sil_acis_check (api_solid_block (position(-2,-2,-2), position (2,2,2), B2),
		  "example", "solid block");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (PI / 4.0,
						       vector (0, 0, 1)) *
					translate_transf (vector (-5,5,0))),
		  "example", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B1),
		  "example", "subtract");

  BODY *B3;
  sil_acis_check (api_solid_block (position(-2,-2,-2), position (2,2,2), B2),
		  "example", "solid block");
  sil_acis_check (api_solid_block (position(-1.0,-1.0,-2),
				   position (1.0,1.0,2), B3),
		  "example", "solid block");
  sil_acis_check (api_transform_entity (B3, 
					rotate_transf (PI / 4.0, 
						      vector (0, 0, 1)) *
					translate_transf (vector (2,2,0))),
		  "example", "api_transform_entity");
  sil_acis_check (api_subtract (B3, B2),
		  "example", "subtract");
  sil_acis_check (api_solid_block (position(-1.0,-1.0,-2),
				   position (1.0,1.0,2), B3),
		  "example", "solid block");
  sil_acis_check (api_transform_entity (B3, 
					rotate_transf (PI / 4.0, 
						      vector (0, 0, 1)) *
					translate_transf (vector (-2,2,0))),
		  "example", "api_transform_entity");
  sil_acis_check (api_subtract (B3, B2),
		  "example", "subtract");

  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (35.0 / 180 * PI ,
						       vector (0, 0, 1))),
		  "example", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B1),
		  "example", "subtract");
  sil_acis_check(api_set_material (B1,
				   0.7, 0.0, 0.0, 1.0,
				   0.7, 0.0, 0.0, 1.0,
				   0.9, 0.2, 0.2, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   30.0),
		 "example", "set material");
  while (!sil_add_symbol ("CutBlockDouble", (void *)B1, "acis", true)) {
    sil_del_symbol ("CutBlockDouble");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example12 ()
{
  BODY *B, *C;

  sil_acis_check (api_solid_cylinder_cone (position(0,
						    0, 
						    -3), 
					   position(0,
						    0,
						    0), 
					   13.5,13.5,13.5, NULL, B),
		  "example", "solid cylinder cone");

  sil_acis_check (api_solid_cylinder_cone (position(0,
						    0, 
						    0), 
					   position(0,
						    0,
						    14.5), 
					   8,8,8, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_unite (C, B),
		  "example", "unite");

  sil_acis_check (api_solid_cylinder_cone (position(0,
						    0, 
						    14.5), 
					   position(0,
						    0,
						    16.5), 
					   6.5,6.5,6.5, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_unite (C, B),
		  "example", "unite");

  sil_acis_check (api_solid_cylinder_cone (position(0,
						    0, 
						    16.5), 
					   position(0,
						    0,
						    19), 
					   8,8,8, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_unite (C, B),
		  "example", "unite");

  sil_acis_check (api_solid_cylinder_cone (position(0,
						    0, 
						    19), 
					   position(0,
						    0,
						    23), 
					   5.5,5.5,5.5, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_unite (C, B),
		  "example", "unite");


  sil_acis_check (api_solid_cylinder_cone (position(10.5,
						    0, 
						    -3),
					   position(10.5,
						    0,
						    0), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position(0,
						    10.5, 
						    -3),
					   position(0,
						    10.5,
						    0), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position(-10.5,
						    0, 
						    -3),
					   position(-10.5,
						    0,
						    0), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position(0,
						    -10.5, 
						    -3),
					   position(0,
						    -10.5,
						    0), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");


  sil_acis_check (api_solid_cylinder_cone (position(0,
						    9, 
						    5),
					   position(0,
						    6,
						    5), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");


  sil_acis_check (api_solid_cylinder_cone (position(0,
						    -9, 
						    5),
					   position(0,
						    -6,
						    5), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");


  sil_acis_check (api_solid_cylinder_cone (position(9,
						    0, 
						    5),
					   position(6,
						    0,
						    5), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position(-9,
						    0, 
						    5),
					   position(-6,
						    0,
						    5), 
					   1.25,1.25,1.25, NULL, C),
		  "example", "solid cylinder cone");
  sil_acis_check (api_subtract (C, B),
		  "example", "subtract");


  while (!sil_add_symbol ("Connector", (void *)B, "acis", true)) {
    sil_del_symbol ("Connector");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example13 ()
{
  BODY *B, *B2;

  sil_acis_check (api_make_prism (6.0, 10.0, 10.0, 8, B),
		  "example13", "make prism");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example13", "make prism");
  sil_acis_check (api_subtract (B2, B),
		  "example13", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example13", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (90.0 / 180.0 * PI, 
						       vector (0, 0, 2))),
		  "example13", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example13", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example13", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (180.0 / 180.0 * PI, 
						       vector (0, 0, 2))),
		  "example13", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example13", "subtract");

  sil_acis_check (api_solid_cylinder_cone (position (2.0, 0.0, 5.0),
					   position (10.0, 0.0, 3.0),
					   2.0, 2.0, 2.0, NULL, B2),
		  "example13", "make prism");
  sil_acis_check (api_transform_entity (B2, 
					rotate_transf (270.0 / 180.0 * PI, 
						       vector (0, 0, 1))),
		  "example13", "api_transform_entity");
  sil_acis_check (api_subtract (B2, B),
		  "example13", "subtract");

  sil_acis_check (api_set_material (B,
				    0.7, 0.0, 0.0, 1.0,
				    0.7, 0.0, 0.0, 1.0,
				    0.8, 0.3, 0.3, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    20.0),
		  "example13", "set material");

  while (!sil_add_symbol ("HalfConicalRegBlock", (void *)B, "acis", true)) {
    sil_del_symbol ("ConicalRegBlock");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void example14 ()
{
  BODY *b1, *c1;
  
  sil_acis_check (api_solid_block (position (-10, -10, -2),
				   position (10, 10, 2), 
				   b1),
		  "example", "solid block");
  for (int j = 0; j < 8; ++j) {
    position p1 (cos (PI / 4.0 * j) * 7,
		 sin (PI / 4.0 * j) * 7,
		 -2);
    position p2 (cos (PI / 4.0 * j) * 7,
		 sin (PI / 4.0 * j) * 7,
		 2);

    sil_acis_check (api_solid_cylinder_cone (p1, p2, 1.5, 1.5, 1.5, NULL, c1),
		    "example", "solid cylinder cone");
    sil_acis_check (api_subtract (c1, b1),
		    "example", "subtract");
  }
  sil_acis_check(api_set_material (b1,
				   0.7, 0.0, 0.7, 1.0,
				   0.7, 0.0, 0.7, 1.0,
				   0.9, 0.0, 0.9, 1.0,
				   0.0, 0.0, 0.0, 0.0,
				   1.0),
		 "example", "set material");
  while (!sil_add_symbol ("CircleHoleBlock", (void *)b1, "acis", true)) {
    sil_del_symbol ("CircleHoleBlock");
  }

  sil_refacet ();
  sil_updated ();

  return;
}


void cube ()
{
  BODY *B = NULL;

  sil_acis_check (api_make_cuboid (10.0, 10.0, 10.0, B),
		  "example", "api_make_cuboid");

  while (!sil_add_symbol ("Cube", (void *)B, "acis", true)) {
    sil_del_symbol ("Cube");
  }

  sil_refacet ();
  sil_updated ();

  return;
}

void inter_test ()
{
  FACE *F1 = new FACE (NULL, NULL, 
  		       (SURFACE *)new CONE (position (0.0, 0.0, 0.0),
  					    unit_vector (1.0, 0.0, 0.0),
  					    vector (0.0,0.0,5.0),
  					    1.0, 0.0, 1.0), FORWARD);
  SHELL *S1 = new SHELL (F1, NULL, NULL);
  LUMP *L1 = new LUMP (S1, NULL);
  BODY *B1 = new BODY (L1);

  FACE *F2 = new FACE (NULL, NULL, 
		       (SURFACE *)new CONE (position (0.0, 0.0, 0.0),
					    unit_vector (0.0, 1.0, 0.0),
					    vector (5.0, 0.0, 0.0),
					    1.0, 0.0, 1.0), FORWARD);
  SHELL *S2 = new SHELL (F2, NULL, NULL);
  LUMP *L2 = new LUMP (S2, NULL);
  BODY *B2 = new BODY (L2);

  sil_acis_check (api_intersect (B2, B1), "example", "api_intersect");

  while (!sil_add_symbol ("Inter", (void *)B1, "acis", true)) {
    sil_del_symbol ("Inter");
  }

  sil_refacet ();
  sil_updated ();
}

void tetra ()
{
  BODY *body = NULL;

  // Macro to Ensure current BULLENTIN_BOARD has been created 
  API_BEGIN;

  // create points
  APOINT* p0 = new APOINT (0.0, 0.0, 0.0);
  APOINT* p1 = new APOINT (10.0, 0.0, 0.0);
  APOINT* p2 = new APOINT (0.0, 10.0, 0.0);
  APOINT* p3 = new APOINT (0.0, 0.0, 10.0);

  // create straights
  STRAIGHT* s0 = new STRAIGHT (p0->coords (),
			       normalise (p1->coords () - p0->coords ()));
  STRAIGHT* s1 = new STRAIGHT (p1->coords (), 
			       normalise (p2->coords () - p1->coords ()));
  STRAIGHT* s2 = new STRAIGHT (p2->coords (),
			       normalise (p0->coords () - p2->coords ()));
  STRAIGHT* s3 = new STRAIGHT (p2->coords (),
			       normalise (p3->coords () - p2->coords ()));
  STRAIGHT* s4 = new STRAIGHT (p0->coords (),
			       normalise (p3->coords () - p0->coords ()));
  STRAIGHT* s5 = new STRAIGHT (p3->coords (),
			       normalise (p1->coords () - p3->coords ()));
  
  // create surfaces
  PLANE* pln0 = new PLANE (p0->coords (), unit_vector (0.0, 0.0, -1.0));
  PLANE* pln1 = new PLANE (p0->coords (), unit_vector (-1.0, 0.0, 0.0));
  PLANE* pln2 = new PLANE (p0->coords (), unit_vector (0.0, -1.0, 0.0));
  PLANE* pln3 = new PLANE (p1->coords (), normalise (vector (1.0, 1.0, 1.0)));
  
  // create verticies
  VERTEX* v0 = new VERTEX (p0);
  VERTEX* v1 = new VERTEX (p1);
  VERTEX* v2 = new VERTEX (p2);
  VERTEX* v3 = new VERTEX (p3);
  
  // create edges
  EDGE* e0 = new EDGE (v0, v1, s0, FORWARD);
  EDGE* e1 = new EDGE (v1, v2, s1, FORWARD);
  EDGE* e2 = new EDGE (v2, v0, s2, FORWARD);
  EDGE* e3 = new EDGE (v2, v3, s3, FORWARD);
  EDGE* e4 = new EDGE (v0, v3, s4, FORWARD);
  EDGE* e5 = new EDGE (v3, v1, s5, FORWARD);

  // create loops of coedges
  // xy plane
  COEDGE* c0 = new COEDGE (e0, REVERSED, NULL, NULL);
  COEDGE* c2 = new COEDGE (e2, REVERSED, c0, NULL);
  COEDGE* c1 = new COEDGE (e1, REVERSED, c2, c0);
  c0->set_previous (c1);
  LOOP* l0 = new LOOP (c0, NULL);

  // yz plane
  c2 = new COEDGE (e2, FORWARD, NULL, NULL);
  COEDGE* c4 = new COEDGE (e4, FORWARD, c2, NULL);
  COEDGE* c3 = new COEDGE (e3, REVERSED, c4, c2);
  c2->set_previous (c3);
  LOOP* l1 = new LOOP (c2, NULL);

  // xz plane
  c0 = new COEDGE (e0, FORWARD, NULL, NULL);
  COEDGE* c5 = new COEDGE (e5, REVERSED, c0, NULL);
  c4 = new COEDGE (e4, REVERSED, c5, c0);
  c0->set_previous (c4);
  LOOP* l2 = new LOOP (c0, NULL);
  
  // canted plane
  c1 = new COEDGE (e1, FORWARD, NULL, NULL);
  c3 = new COEDGE (e3, FORWARD, c1, NULL);
  c5 = new COEDGE (e5, FORWARD, c3, c1);
  c1->set_previous (c5);
  LOOP* l3 = new LOOP (c1, NULL);

  // create faces
  FACE* f0 = new FACE (l0, NULL, pln0, FORWARD); 
  FACE* f1 = new FACE (l1, f0, pln1, FORWARD); 
  FACE* f2 = new FACE (l2, f1, pln2, FORWARD); 
  FACE* f3 = new FACE (l3, f2, pln3, FORWARD); 

  // create shells
  SHELL* sh0 = new SHELL (f3, NULL, NULL);
  LUMP* lu0 = new LUMP (sh0, NULL);
  
  // create body
  body = new BODY (lu0);

  // close current BULLETIN_BOARD
  API_END;

  while (!sil_add_symbol ("tetra", body, "acis", true)) {
    sil_del_symbol ("tetra");
  }
  sil_acis_check (api_set_material (body,
				    0.0, 1.0, 0.0, 1.0,
				    0.0, 1.0, 0.0, 1.0,
				    0.0, 0.0, 0.0, 0.0,
				    0.0, 0.0, 0.0, 0.0,
				    1.0),
		  "tetra", "set material");

  sil_refacet ();
  sil_updated ();

  return;
}

SIL_PLUGIN ()
{
  sil_register_tool (&example1, "example1", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 1.", "acis", "acis");
  sil_register_tool (&example2, "example2", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 2.", "acis", "acis");
  sil_register_tool (&example3, "example3", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 3.", "acis", "acis");
  sil_register_tool (&example4, "example4", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 4.", "acis", "acis");
  sil_register_tool (&example5, "example5", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 5.", "acis", "acis");
  sil_register_tool (&example6, "example6", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 6.", "acis", "acis");
  sil_register_tool (&example7, "example7", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 7.", "acis", "acis");
  sil_register_tool (&example8, "example8", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 8", "acis", "acis");
  sil_register_tool (&example9, "example9", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "ACIS example 9.", "acis", "acis");
  sil_register_tool (&example10, "example10", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "AXIS example 10.", "acis", "acis");
  sil_register_tool (&example11, "example11", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "AXIS example 11.", "acis", "acis");
  sil_register_tool (&example12, "example12", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "AXIS example 12.", "acis", "acis");
  sil_register_tool (&example13, "example13", "0.1",
		     "Frank C Langbein", "2001 Frank C Langbein",
		     "ACIS example 13.", "acis", "acis");
  sil_register_tool (&example14, "example14", "0.1",
		     "Frank C Langbein", "2001 Frank C Langbein",
		     "ACIS example 14.", "acis", "acis");
  sil_register_tool (&cube, "cube", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "Tetra example.", "acis", "acis");
  sil_register_tool (&inter_test, "inter_test", "0.1",
		     "Frank C Langbein", "2000 Frank C Langbein",
		     "Intersection test.", "acis", "acis");
  sil_register_tool (&tetra, "tetra", "0.1",
		     "Corney", "Corney",
		     "Tetra example.", "acis", "acis");

  return;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
