// things.cc: ACIS things for testing.
//
// Copyright (C) 1999-2000 Bruce I Mills <B.I.Mills@cs.cf.ac.uk>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: things.cc 1 2005-08-10 23:35:07Z frank $
//
// * Original code desinged and implemented by Bruce I Mills 
//   <B.I.Mills@cs.cf.ac.uk> for sil 0.1.4.
// * Modifications for sil 0.1.5 and cleanups by Frank C Langbein 
//   <frank@langbein.org>.

#include "config.h"

RCSID (things, "$Id: things.cc 1 2005-08-10 23:35:07Z frank $");

#include "acis/acis.hxx"

#include "interface.hxx"
#include "xmath.hxx"
#include "list.hxx"
#include "acis/light.hxx"
#include "acis/silapi.hxx"
#include "base/base.hxx"

// List of things generated by the tools below:
//
// thing01 ... cube with hole and slot
// thing02 ... table strut with C2
// thing03 ... semi-stelated cube-octa-hedron
// thing04 ... pulley block with Dcm
// thing05 ... bracket with D2m
// thing06 ... table strut with C4
// thing07 ... swivel mount with D2m
// thing08 ... transverse bracket with D2i
// thing09 ... fascinating accident: what is it?
// thing10 ... jelly mold former with D8m
// thing11 ... spoked half wheel with A4
// thing12 ... rotary sheet cutter with [D6m L4]
// thing13 ... three way bracket with tetrahedral ??
// thing14 ... pentane, with D5m
// thing15 ... chair base with C5m
// thing16 ... structural spacer with D4m
// thing17 ... Wheel with Dcm
// thing18 ... wishbone axel with C2m
// thing19 ... double blade blank with D2
// thing20 ... optical filter holder with D3m
// thing21 ... drip tray with C4m
// thing22 ... clip fitting with C8m
// thing23 ... base for British standard plug
// thing24 ... a key
//
// Symmetries:
//
// D2m ...... thing05
// D4m ...... thing16
// Dcm ...... thing04
// D6m ...... thing12
// D8m ...... thing10
// D5m ...... thing14
// D2 ....... thing19
// D3 ....... 
// D4 .......
// D3m ...... thing20
// D2i ...... thing08
// C2m ...... thing18
// Ccm ...... 
// C4m ...... thing21
// C5m ...... thing15
// C3m ...... 
// C8m ...... thing22
// C2 ....... thing02

// Shorthand for putting an object into the data base that is
// maintained by sil. Uses default everything, except the
// BODY being inserted, and the name to use for it.
void makeit (BODY *B, char *s)
{
  sil_acis_check (api_set_material (B,
				    0.5, 0.5, 0.5, 1.0,	// R G B alpha ambient
				    0.0, 0.7, 0.0, 1.0, // R G B alpha diffuse
				    0.2, 0.0, 0.0, 0.0, // R G B alpha specular
				    0.1, 0.0, 0.0, 0.0, // R G B alpha emission
				    1.0),
		  "things", "set material");
  while (!sil_add_symbol (s, (void *)B, "acis", true))
    sil_del_symbol (s);
  sil_refacet ();
  sil_updated ();
  return;
}

void thing01 ()
{
  BODY *main_cube;
  BODY *main_hole;
  BODY *hole1;
  BODY *hole2;
  BODY *hole3;
  BODY *hole4;
  BODY *slot;
  BODY *lhand;
  BODY *rhand;

  position pts[20];
 
  // Create an object that can be expressed as a modified cube.
  // A bevel on the top edge, a main hole from front to back,
  // with four small holes in the corners of the front face.
  // A square patch on the left and right face, and a slot 
  // cut in the top face.
  // 
  // From the front ...
  //     
  //       ___________.   .___________       .......  top face
  //      /           +---+           \      .......
  //     /                             \     .......  middle edge
  //     |                             |
  //     |   o                     o   |
  //   +-|              **             |-+
  //   | |           *      *          | |
  //   | |                             | |
  //   | |         *          *        | |
  //   | |                             | |
  //   | |           *      *          | |
  //   +-|              **             |-+
  //     |   o                     o   |
  //     |                             |
  //     |_____________________________|     ....... bottom face

  double cl = 20.0;            // length of the sides of the basic cube
  double mr = cl / 7.0;        // radius of the main hole
  double sr = cl / 40.0;       // radius of the small holes
  double sd = 3.0 * cl / 10.0; // distance of small holes from central axis
  double ph = 4.0 * cl / 7.0;  // height of side patches

  // Create the main cube, complete with upper face bevel
  
  pts[ 0] = position (-cl / 2.0, -cl / 2.0, -cl / 2.0); // bottom face
  pts[ 1] = position (-cl / 2.0,  cl / 2.0, -cl / 2.0);
  pts[ 2] = position ( cl / 2.0,  cl / 2.0, -cl / 2.0);
  pts[ 3] = position ( cl / 2.0, -cl / 2.0, -cl / 2.0);
  
  pts[ 4] = position (-cl / 2.0, -cl / 2.0, 2.0 * cl / 5.0); // middle edge
  pts[ 5] = position (-cl / 2.0,  cl / 2.0, 2.0 * cl / 5.0);
  pts[ 6] = position ( cl / 2.0,  cl / 2.0, 2.0 * cl / 5.0);
  pts[ 7] = position ( cl / 2.0, -cl / 2.0, 2.0 * cl / 5.0);
  
  pts[ 8] = position (-2.0 * cl / 5.0, -2.0 * cl / 5.0, cl / 2.0); // top face
  pts[ 9] = position (-2.0 * cl / 5.0,  2.0 * cl / 5.0, cl / 2.0);
  pts[10] = position ( 2.0 * cl / 5.0,  2.0 * cl / 5.0, cl / 2.0);
  pts[11] = position ( 2.0 * cl / 5.0, -2.0 * cl / 5.0, cl / 2.0);
  
  create_convex (pts, 12, &main_cube);
  
  // and the slot on the top face
  
  sil_acis_check (api_solid_block (position (cl, -cl / 10.0, cl),
				   position (-cl, cl / 10.0, cl - 1),
				   slot),
		  "things", "api_solid_block");
  
  // Create all the holes, going from the front to back face
  
  create_cylinder (position (0.0, 0.0, 0.0),
		   unit_vector(1.0, 0.0, 0.0),
		   cl,
		   mr,
		   &main_hole);

  create_cylinder (position (0.0, sd, -sd),
		   unit_vector (1.0, 0.0, 0.0),
		   cl,
		   sr,
		   &hole1);
  
  create_cylinder (position (0.0, -sd, -sd),
		   unit_vector (1.0, 0.0, 0.0),
		   cl,
		   sr,
		   &hole2);

  create_cylinder (position (0.0, sd, sd),
		   unit_vector (1.0, 0.0, 0.0),
		   cl,
		   sr,
		   &hole3);
  
  create_cylinder (position(0.0, -sd, sd),
		   unit_vector(1.0, 0.0, 0.0),
		   cl,
		   sr,
		   &hole4);

  // Create the left and right side patches
  
  sil_acis_check (api_solid_block (position (-ph / 2.0, cl / 2.0, ph / 2.0),
				   position ( ph / 2, cl / 2.0 + 1.0, 
					      -ph / 2.0),
				   lhand),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (-ph / 2.0, -cl / 2.0, ph / 2.0),
				   position (ph / 2.0, -(cl / 2.0 + 1.0),
					     -ph / 2.0),
				   rhand),
		  "things", "api_solid_block");
    
  // Now put it all together to make the object
  
  sil_acis_check (api_subtract (main_hole,main_cube),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (hole1,main_cube),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (hole2,main_cube),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (hole3,main_cube),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (hole4,main_cube),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (slot,main_cube),
		  "things", "api_subtract");
  sil_acis_check (api_unite (lhand,main_cube),
		  "things", "api_unit");
  sil_acis_check (api_unite (rhand,main_cube),
		  "things", "api_unit");
  
  makeit (main_cube, "thing01");

  return;
}

void thing02 ()
{
  BODY *vert;
  BODY *hori;
  BODY *serif1;
  BODY *serif2;
  BODY *pad1;
  BODY *pad2;
  BODY *pad3;
  BODY *pad4;
  
  double thick = 2.0;
  double leg_len = 40.0;
  double hor_len = 10.0;
  double ser_len = 10.0;
  
  // Table strut
  
  sil_acis_check (api_solid_block (position ( thick, thick, 0.0),
				   position (-thick, -thick, leg_len),
				   vert),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (hor_len, thick, 0.0),
				   position (-hor_len, -thick, thick),
				   hori),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (hor_len, thick, 0.0),
				   position (hor_len + 2.0 * thick,
					     -ser_len, thick),
				  serif1),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position ( -hor_len, -thick, 0.0),
				   position (-(hor_len + 2.0 * thick),
					     ser_len, thick),
				   serif2),
		  "things", "api_solid_block");
  
  create_sphere(position (hor_len + thick, -(ser_len - thick), 0.0),
		position (hor_len + thick, -ser_len, 0.0),
		&pad1);
  
  create_sphere(position (hor_len +       thick, 0.0, 0.0),
		position (hor_len + 2.0 * thick, 0.0, 0.0),
		&pad2);
  
  create_sphere(position (-(hor_len +       thick), 0.0, 0.0),
		position (-(hor_len + 2.0 * thick), 0.0, 0.0),
		&pad3);
  
  create_sphere(position (-(hor_len + thick), ser_len - thick, 0.0),
		position (-(hor_len + thick), ser_len, 0.0),
		&pad4);
  
  sil_acis_check (api_unite (vert,hori),
		  "things", "api_unite");
  sil_acis_check (api_unite (serif1,hori),
		  "things", "api_unite");
  sil_acis_check (api_unite (serif2,hori),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad1,hori),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad2,hori),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad3,hori),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad4,hori),
		  "things", "api_unite");
  
  makeit (hori, "thing02");
  
  return;
}

void thing03 ()
{
  BODY *main_cube;
  position pts[20];
  
  // The following describes a topological semi stellated
  // cube-octohedron. This object can be constructed as follows,
  // select a quadrilatril hexahedron (for example a cube) and pick a
  // spot on each edge, not either of the end points. Now for each
  // pair of points selected on edges with a common vertex, draw a
  // line between them. Around each corner we have draw a triangle, so
  // each corner is now free to move, so move them to produce the
  // final figure.
  //
  // The actual parameterisation used here does not cover the space of
  // all such objects, because right now I can't think of a nice
  // parametrisation for it. However in principle it could be done by
  // picking 8 corner points as below, plus a particular 9 vertices on
  // a distorted cub-ocothedron.
    
  // These points are the corner points of a distorted cube. 
  // Topologically what we have is a quadrilatril hexahedron.
  // There is not requirement for the faces to be planar, as
  // they do not form a face of the final object, only the 
  // corners are really significant.
  
  double corner = 10.0;
  double middle = 8.0;
  
  double daaa1 = 0.15;
  double daab1 = 0.26;
  double daba1 = 0.37;
  double dabb1 = 0.43;
  double dbaa1 = 0.51;
  double dbab1 = 0.34;
  double dbba1 = 0.22;
  double dbbb1 = 0.16;

  double daaa2 = 0.58;
  double daab2 = 0.33;
  double daba2 = 0.21;
  double dabb2 = 0.66;
  double dbaa2 = 0.14;
  double dbab2 = 0.93;
  double dbba2 = 0.22;
  double dbbb2 = 0.21;

  BODY *ulf;
  BODY *ulb;
  BODY *urf;
  BODY *urb;
  BODY *dlf;
  BODY *dlb;
  BODY *drf;
  BODY *drb;
  
  // the extra corners for the semi-stellation
  
  position paaa ( corner+daaa1,  corner + daaa2,  corner);
  position paab ( corner+daab1,  corner + daab2, -corner);
  position paba ( corner+daba1, -corner + daba2,  corner);
  position pabb ( corner+dabb1, -corner + dabb2, -corner);
  position pbaa (-corner+dbaa1,  corner + dbaa2,  corner);
  position pbab (-corner+dbab1,  corner + dbab2, -corner);
  position pbba (-corner+dbba1, -corner + dbba2,  corner);
  position pbbb (-corner+dbbb1, -corner + dbbb2, -corner);
  
  // the corners of a topological octohedral-cut cube
  
  double twaa0 = 0.5;
  double twab0 = 0.5;
  double twba0 = 0.5;
  double twbb0 = 0.5;

  double twa0a = 0.5;
  double twa0b = 0.5;
  double twb0a = 0.5;
  double twb0b = 0.5;

  double tw0aa = 0.5;
  double tw0ab = 0.5;
  double tw0ba = 0.5;
  double tw0bb = 0.5;

  position paa0 ( middle,  middle, twaa0);
  position pab0 ( middle, -middle, twab0);
  position pba0 (-middle,  middle, twba0);
  position pbb0 (-middle, -middle, twbb0);
  
  position pa0a ( middle, twa0a,  middle);
  position pa0b ( middle, twa0b, -middle);
  position pb0a (-middle, twb0a,  middle);
  position pb0b (-middle, twb0b, -middle);
  
  position p0aa ( tw0aa,  middle,  middle);
  position p0ab ( tw0ab,  middle, -middle);
  position p0ba ( tw0ba, -middle,  middle);
  position p0bb ( tw0bb, -middle, -middle);
  
  // assuming it is convex, this make the inner cube-octohedron
  
  pts[ 0] = paa0;
  pts[ 1] = pab0;
  pts[ 2] = pba0;
  pts[ 3] = pbb0;
  
  pts[ 4] = pa0a;
  pts[ 5] = pa0b;
  pts[ 6] = pb0a;
  pts[ 7] = pb0b;
  
  pts[ 8] = p0aa;
  pts[ 9] = p0ab;
  pts[10] = p0ba;
  pts[11] = p0bb;

  create_convex (pts, 12, &main_cube);
  
  // Now just make each of the eight tetrahedrons for the corners
  
  pts[0] = paba; 
  pts[1] = pab0; 
  pts[2] = pa0a; 
  pts[3] = p0ba; 
  create_convex (pts, 4, &ulf);

  pts[0] = pbba;
  pts[1] = pbb0;
  pts[2] = pb0a;
  pts[3] = p0ba;
  create_convex (pts, 4, &ulb);
  
  pts[0] = paaa; 
  pts[1] = paa0; 
  pts[2] = pa0a; 
  pts[3] = p0aa; 
  create_convex (pts, 4, &urf);
  
  pts[0] = pbaa; 
  pts[1] = pba0; 
  pts[2] = pb0a; 
  pts[3] = p0aa; 
  create_convex (pts, 4, &urb);
  
  pts[0] = pabb; 
  pts[1] = pab0; 
  pts[2] = pa0b; 
  pts[3] = p0bb; 
  create_convex (pts, 4, &dlf);
  
  pts[0] = pbbb; 
  pts[1] = pbb0; 
  pts[2] = pb0b; 
  pts[3] = p0bb; 
  create_convex (pts, 4, &dlb);

  pts[0] = paab; 
  pts[1] = paa0; 
  pts[2] = pa0b; 
  pts[3] = p0ab; 
  create_convex (pts, 4, &drf);
  
  pts[0] = pbab; 
  pts[1] = pba0; 
  pts[2] = pb0b; 
  pts[3] = p0ab; 
  create_convex (pts, 4, &drb);
  
  sil_acis_check (api_unite (ulf, main_cube), "things", "api_unite");
  sil_acis_check (api_unite (ulb, main_cube), "things", "api_unite");
  sil_acis_check (api_unite (urf, main_cube), "things", "api_unite");
  sil_acis_check (api_unite (urb, main_cube), "things", "api_unite");
  sil_acis_check (api_unite (dlf, main_cube), "things", "api_unite");
  sil_acis_check (api_unite (dlb, main_cube), "things", "api_unite");
  sil_acis_check (api_unite (drf, main_cube), "things", "api_unite");
  sil_acis_check (api_unite (drb, main_cube), "things", "api_unite");
  
  makeit (main_cube, "thing03");
  
  return;
}

void thing04 ()
{
  BODY *torus_1;
  BODY *torus_2;
  BODY *torus_3;
  BODY *tube;
  BODY *ctube;
  BODY *hole;
  BODY *lcap;
  BODY *rcap;
  BODY *B;
  
  sil_acis_check (api_solid_block (position (0.1, 0.1, 0.1),
				   position (0.0, 0.0, 0.0),
				   B),
		  "things", "api_solid_block");
  
  double lcent = 10.0;
  double mcent = 0.0;
  double rcent = -10.0;

  double rada = 20.0;
  double radb = 5.0;
  double radh = 10.0;

  double xdirn = 0.0;
  double ydirn = 1.0;
  double zdirn = 1.0;

  unit_vector dirn (xdirn, ydirn, zdirn);
  unit_vector perp = ortho (dirn);
  
  position ltcent (lcent * dirn.x (), lcent * dirn.y (), lcent * dirn.z ());
  position mtcent (mcent * dirn.x (), mcent * dirn.y (), mcent * dirn.z ());
  position rtcent (rcent * dirn.x (), rcent * dirn.y (), rcent * dirn.z ());
  
  create_torus (ltcent, dirn, rada, radb, &torus_1);
  create_torus (mtcent, dirn, rada, radb, &torus_2);
  create_torus (rtcent, dirn, rada, radb, &torus_3);
  
  // make left end cap
  
  create_sphere (position (ltcent.x () * 4.0, ltcent.y () * 4.0,
			   ltcent.z () * 4.0),
		 position ((rada + radb) * perp.x () + 
			   (rcent - radb) * dirn.x (),
			   (rada + radb) * perp.y () + 
			   (rcent - radb) * dirn.y (),
			   (rada + radb) * perp.z () + 
			   (rcent - radb) * dirn.z ()),
		 &lcap);
  
  section (position (0.0, 0.0, 0.0), position (1.0, 0.0, 0.0),
	   position (0.0, -1.0, 1.0), position (0.0, -1.0, -1.0), lcap);
  
  // make right end cap
  
  create_sphere (position (rtcent.x () * 4.0, rtcent.y () * 4.0,
			   rtcent.z () * 4.0),
		 position ((rada + radb) * perp.x () +
			   (lcent + radb) * dirn.x (),
			   (rada + radb) * perp.y () + 
			   (lcent + radb) * dirn.y (),
			   (rada + radb) * perp.z () +
			   (lcent + radb) * dirn.z ()),
		 &rcap);
  
  section (position (0.0, 0.0, 0.0), position (1.0, 0.0, 0.0),
	   position (0.0, -1.0, 1.0), position (0.0, 1.0, 1.0), rcap);
  
  create_cylinder (position (0.0, 0.0, 0.0), dirn, lcent - rcent, rada, &tube);
  create_cylinder (position (0.0, 0.0, 0.0), dirn, 100.0, rada, &ctube);
  create_cylinder (position (0.0, 0.0, 0.0), dirn, 100.0, radh, &hole);
 
  sil_acis_check (api_unite (lcap, B),
		  "things", "api_unite");
  sil_acis_check (api_unite (rcap, B),
		  "things", "api_unite");
  sil_acis_check (api_intersect (ctube, B),
		  "things", "api_intersect");
  sil_acis_check (api_unite (tube, B),
		  "things", "api_unite");
  sil_acis_check (api_subtract (torus_1, B),
		  "things", "api_subtracta");
  sil_acis_check (api_subtract (torus_2, B),
		  "things", "api_subtracta");
  sil_acis_check (api_subtract (torus_3, B),
		  "things", "api_subtracta");
  sil_acis_check (api_subtract (hole, B),
		  "things", "api_subtracta");
  
  makeit (B, "thing04");
  
  return;
}

void thing05 ()
{
  BODY *main_body;
  BODY *cylpp;
  BODY *cylpm;
  BODY *cylmp;
  BODY *cylmm;
  BODY *slot1;
  BODY *slot2;
  
  double box_dep = 20.0;
  double box_wid = 10.0;
  double box_hig = 1.0;
  double blk = 100.0;

  // create block to do some sculpting on
  
  sil_acis_check (api_solid_block (position (-blk, -blk, -blk),
				   position (blk, blk, blk), 
				   main_body),
		  "things", "api_solid_block");
  
  // cut a potentially wonky quadralatril hexahedron
  
  section (position (box_dep,  box_wid,  box_hig),
	   position (box_dep, -box_wid,  box_hig),
	   position (box_dep,  box_wid, -box_hig),
	   position (0.0, 0.0, 0.0),
	   main_body);
  
  section (position (-box_dep,  box_wid,  box_hig),
	   position (-box_dep, -box_wid,  box_hig),
	   position (-box_dep,  box_wid, -box_hig),
	   position (0.0, 0.0, 0.0),
	   main_body);

  section (position ( box_dep, box_wid, box_hig),
	   position (-box_dep, box_wid, box_hig),
	   position ( box_dep, box_wid,-box_hig),
	   position (0.0, 0.0, 0.0),
	   main_body);
  
  section (position ( box_dep,-box_wid, box_hig),
	   position (-box_dep,-box_wid, box_hig),
	   position ( box_dep,-box_wid,-box_hig),
	   position (0.0, 0.0, 0.0),
	   main_body);
  
  section (position ( box_dep, box_wid, box_hig),
	   position ( box_dep,-box_wid, box_hig),
	   position (-box_dep, box_wid, box_hig),
	   position (0.0, 0.0, 0.0),
	   main_body);
  
  section (position ( box_dep, box_wid,-box_hig),
	   position ( box_dep,-box_wid,-box_hig),
	   position (-box_dep, box_wid,-box_hig),
	   position (0.0, 0.0, 0.0),
	   main_body);
    
  // drill four holes
  
  create_cylinder (position (2.0 * box_dep / 3.0, 2.0 * box_wid / 3.0, 0.0),
		   unit_vector (0.0, 0.0, 1.0),
		   2.0 * box_hig, box_wid / 5.0, &cylpp);
  
  create_cylinder (position (2.0 * box_dep / 3.0, -2.0 * box_wid / 3.0, 0.0),
		   unit_vector (0.0, 0.0, 1.0),
		   2.0 * box_hig, box_wid / 5.0, &cylpm);
  
  create_cylinder (position (-2.0 * box_dep / 3.0, 2.0 * box_wid / 3.0, 0),
		   unit_vector (0.0, 0.0, 1.0),
		   2.0 * box_hig, box_wid / 5.0, &cylmp);
  
  create_cylinder (position (-2.0 * box_dep / 3.0, -2.0 * box_wid / 3.0, 0),
		   unit_vector (0.0, 0.0, 1.0),
		   2.0 * box_hig, box_wid / 5.0, &cylmm);
  
  // create a slot in each end
  
  sil_acis_check (api_solid_block (position (-box_dep, (9.0 * box_wid) / 10.0,
					     (2.0 * box_hig) / 3.0),
				   position (-(9.0 * box_dep) / 10.0,
					     -(9.0 * box_wid) / 10.0,
					     -(2.0 * box_hig) / 3.0),
				   slot1),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (box_dep, (9.0 * box_wid) / 10.0,
					     (2.0 * box_hig) / 3.0),
				   position ((9.0 * box_dep) / 10.0,
					     -(9.0 * box_wid) / 10.0,
					     -(2.0 * box_hig) / 3.0),
				   slot2),
		  "things", "api_solid_block");
  
  sil_acis_check (api_subtract (cylpp,main_body),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (cylpm,main_body),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (cylmp,main_body),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (cylmm,main_body),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (slot1,main_body),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (slot2,main_body),
		  "things", "api_subtract");

  BODY *bx;
  sil_acis_check (api_solid_block (position (-1.0, -1.0, -1.0),
				   position (1.0, 1.0, 1.0),
				   bx),
		  "things", "api_solid_block");
  sil_acis_check (api_unite (main_body,bx),
		  "things", "api_unite");

  makeit (bx, "thing05");

  return;
}

void thing06 ()
{
  BODY *vert;
  BODY *hori1;
  BODY *hori2;
  BODY *serif11;
  BODY *serif12;
  BODY *serif21;
  BODY *serif22;
  BODY *pad11;
  BODY *pad12;
  BODY *pad13;
  BODY *pad14;
  BODY *pad21;
  BODY *pad22;
  BODY *pad23;
  BODY *pad24;
  
  double thick = 2.0;
  double leg_len = 40.0;
  double hor_len = 10.0;
  double ser_len = 10.0;
  
  // Table strut
  
  sil_acis_check (api_solid_block (position ( thick, thick,0),
				   position (-thick,-thick,leg_len),
				   vert),
		  "things", "api_solid_block");

  sil_acis_check (api_solid_block (position ( hor_len, thick,0),
				   position (-hor_len,-thick,thick),
				   hori1),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (hor_len, thick,0),
				   position (hor_len+2*thick,-ser_len, thick),
				   serif11),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (-hor_len,-thick,0),
				   position (-(hor_len+2*thick),
					     ser_len, thick),
				   serif12),
		  "things", "api_solid_block");

  sil_acis_check (api_solid_block (position (-thick, hor_len, 0),
				   position ( thick, -hor_len, thick),
				   hori2),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (-thick,hor_len        , 0),
				   position (ser_len,hor_len+2*thick, thick),
				   serif21),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (thick, -hor_len,0),
				   position (-ser_len, 
					     -(hor_len+2*thick),thick),
				   serif22),
		  "things", "api_solid_block");
  
  create_sphere (position (hor_len+thick,-(ser_len-thick),0),
		 position (hor_len+thick,-(ser_len),0),
		 &pad11);
  
  create_sphere (position (hor_len+1*thick,0,0),
		 position (hor_len+2*thick,0,0),
		 &pad12);
  
  create_sphere (position (-(hor_len+1*thick),0,0),
		 position (-(hor_len+2*thick),0,0),
		 &pad13);
  
  create_sphere (position (-(hor_len+thick),(ser_len-thick),0),
		 position (-(hor_len+thick),(ser_len),0),
		 &pad14);
  
  create_sphere (position ((ser_len-thick),hor_len+thick,0),
		 position (ser_len,hor_len+thick,0),
		 &pad21);
  
  create_sphere (position (0,hor_len+1*thick,0),
		 position (0,hor_len+2*thick,0),
		 &pad22);

  create_sphere (position (0,-(hor_len+1*thick),0),
		 position (0,-(hor_len+2*thick),0),
		 &pad23);
  
  create_sphere (position (-(ser_len-thick),-(hor_len+thick),0),
		 position (-(ser_len),-(hor_len+thick),0),
		 &pad24);
  
  sil_acis_check (api_unite (hori1,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (serif11,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (serif12,vert),
		  "things", "api_unite");
  
  sil_acis_check (api_unite (hori2,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (serif21,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (serif22,vert),
		  "things", "api_unite");
  
  sil_acis_check (api_unite (pad11,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad12,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad13,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad14,vert),
		  "things", "api_unite");
  
  sil_acis_check (api_unite (pad21,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad22,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad23,vert),
		  "things", "api_unite");
  sil_acis_check (api_unite (pad24,vert),
		  "things", "api_unite");
  
  makeit (vert, "thing06");
  
  return;
}

void thing07 ()
{
  BODY *base;
  BODY *flange;
  BODY *hole;
  BODY *bolt1;
  BODY *bolt2;
  BODY *bolt3;
  BODY *bolt4;
  
  double box_dep = 10.0;
  double box_wid = 30.0;
  double box_hig = 1.25;
  double frat = 3.0;

  sil_acis_check (api_solid_block (position (box_dep,box_wid,box_hig),
				   position (-box_dep,-box_wid,-box_hig),
				   base),
		  "things", "api_solid_block");
  
  create_cylinder (position (0,0,0),
		   unit_vector (1,0,0),
		   frat*box_hig,
		   box_wid,
		   &flange);
  
  section (position (0,0,0),
	   position (box_wid,box_hig,0),
	   position (box_wid,-box_hig,0),
	   position (0,0,box_hig),
	   flange);
  
  create_cylinder (position (0,0,box_wid/2),
		   unit_vector (1,0,0),
		   frat*box_hig,
		   box_wid/6,
		   &hole);
  
  sil_acis_check (api_subtract (hole,flange),
		  "things", "api_subtract"); 
  sil_acis_check (api_unite (flange,base),
		  "things", "api_unite");
  
  create_cylinder (position (3*box_dep/5,4*box_wid/5,0),
		   unit_vector (0,0,1),
		   2*box_hig,
		   box_dep/10,
		   &bolt1);
  
  create_cylinder (position (3*box_dep/5,-4*box_wid/5,0),
		   unit_vector (0,0,1),
		   2*box_hig,
		   box_dep/10,
		   &bolt2);
  
  create_cylinder (position (-3*box_dep/5,4*box_wid/5,0),
		   unit_vector (0,0,1),
		   2*box_hig,
		   box_dep/10,
		   &bolt3);
  
  create_cylinder (position (-3*box_dep/5,-4*box_wid/5,0),
		   unit_vector (0,0,1),
		   2*box_hig,
		   box_dep/10,
		   &bolt4);
  
  sil_acis_check (api_subtract (bolt1,base),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (bolt2,base),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (bolt3,base),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (bolt4,base),
		  "things", "api_subtract");
  
  makeit (base, "thing07");
  
  return;
}

void thing08 ()
{
  BODY *base1;
  BODY *base2;
  BODY *bolt1;
  BODY *hole1;
  BODY *bolt2;
  BODY *hole2;
  
  position pts[20];
  
  double box_dep = 9.0;
  double box_wid = 27.0;
  double box_hig = 0.875;
  
  pts[0] = position (-box_dep,-box_wid,-box_hig);
  pts[1] = position ( box_dep,-box_wid,-box_hig);
  pts[2] = position ( 0      , box_wid,-box_hig);
  
  pts[3] = position (-box_dep,-box_wid, box_hig);
  pts[4] = position ( box_dep,-box_wid, box_hig);
  pts[5] = position ( 0      , box_wid, box_hig);
   
  create_convex (pts,6,&base1);
  
  pts[0] = position (-box_hig, box_wid,-box_dep);
  pts[1] = position (-box_hig, box_wid, box_dep);
  pts[2] = position (-box_hig,-box_wid, 0      );
  
  pts[3] = position ( box_hig, box_wid,-box_dep);
  pts[4] = position ( box_hig, box_wid, box_dep);
  pts[5] = position ( box_hig,-box_wid, 0      );
  
  create_convex (pts,6,&base2);
 
  create_cylinder (position (0,-box_wid,0),
		   unit_vector (1,0,0),
		   2*box_dep,
		   4*box_hig,
		   &bolt1);
  
  create_cylinder (position (0,-box_wid,0),
		   unit_vector (1,0,0),
		   2*box_dep,
		   3*box_hig,
		   &hole1);

  create_cylinder (position (0,box_wid,0),
		   unit_vector (0,0,1),
		   2*box_dep,
		   4*box_hig,
		   &bolt2);
 
  create_cylinder (position (0,box_wid,0),
		   unit_vector (0,0,1),
		   2*box_dep,
		   3*box_hig,
		   &hole2);

  sil_acis_check (api_unite (base2,base1),
		  "things", "api_unite");
  sil_acis_check (api_unite (bolt1,base1),
		  "things", "api_unite");
  sil_acis_check (api_unite (bolt2,base1),
		  "things", "api_unite");
  sil_acis_check (api_subtract (hole1,base1),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (hole2,base1),
		  "tings", "api_subtract");
  
  makeit (base1, "thing08");
  
  return;
}

void thing09 ()
{
  BODY *base;
  BODY *spoke1;
  BODY *spoke2;
  BODY *spoke3;
  BODY *spoke4;
  
  double CosPiOn8;
  double SinPiOn8;
  
  double box_dep = 2.0;
  double box_wid = 6.0;
  double spoke_len = 5.0;

  CosPiOn8 = sqrt (( 2+sqrt (6))/4);
  SinPiOn8 = sqrt ((-2+sqrt (6))/4);
  
  create_cylinder (position (0,0,0),
		   unit_vector (1,0,0),
		   box_dep,
		   box_wid,
		   &base);
  
  create_cylinder (position (0,(spoke_len/2)*CosPiOn8,(spoke_len/2)*SinPiOn8),
		   unit_vector (0,CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_wid,
		   &spoke1);
  
  create_cylinder (position (0, (spoke_len/2)*SinPiOn8,(spoke_len/2)*CosPiOn8),
		   unit_vector (0,CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_wid,
		   &spoke2);

  create_cylinder (position (0, -(spoke_len/2)*SinPiOn8,
			     (spoke_len/2)*CosPiOn8),
		   unit_vector (0,CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_wid,
		   &spoke3);
  
  create_cylinder (position (0,-(spoke_len/2)*CosPiOn8,(spoke_len/2)*SinPiOn8),
		   unit_vector (0,CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_wid,
		   &spoke4);
  
  sil_acis_check (api_unite (spoke1, base),
		  "things", "api_unite");
  sil_acis_check (api_unite (spoke2, base),
		  "things", "api_unite");
  sil_acis_check (api_unite (spoke3, base),
		  "things", "api_unite");
  sil_acis_check (api_unite (spoke4, base),
		  "tings", "api_unite");
  
  makeit (base, "thing09");
  
  return;
}

void thing10 ()
{
  BODY *base;
  BODY *spoke1;
  BODY *spoke2;
  BODY *spoke3;
  BODY *spoke4;
  BODY *spoke5;
  BODY *spoke6;
  BODY *spoke7;
  BODY *spoke8;

  double CosPiOn8;
  double SinPiOn8;

  double box_dep = 2.0;
  double box_wid = 6.0;
  double spoke_len = 5.0;

  CosPiOn8 = sqrt (( 1+sqrt (2))/2);  // proportional to
  SinPiOn8 = sqrt ((-1+sqrt (2))/2);  // proportional to
  
  create_cylinder (position (0,0,0),
		   unit_vector (1,0,0),
		   box_dep,
		   box_wid,
		   &base);

  create_cylinder (position (0,(spoke_len/2)*CosPiOn8,(spoke_len/2)*SinPiOn8),
		   unit_vector (0,CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke1);
  
  create_cylinder (position (0, (spoke_len/2)*SinPiOn8,(spoke_len/2)*CosPiOn8),
		   unit_vector (0,SinPiOn8,CosPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke2);

  create_cylinder (position (0, -(spoke_len/2)*SinPiOn8,
			     (spoke_len/2)*CosPiOn8),
		   unit_vector (0,-SinPiOn8,CosPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke3);
  
  create_cylinder (position (0,-(spoke_len/2)*CosPiOn8,(spoke_len/2)*SinPiOn8),
		   unit_vector (0,-CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke4);
  
  create_cylinder (position (0,(spoke_len/2)*CosPiOn8,-(spoke_len/2)*SinPiOn8),
		   unit_vector (0,CosPiOn8,-SinPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke5);
  
  create_cylinder (position (0, (spoke_len/2)*SinPiOn8,
			     -(spoke_len/2)*CosPiOn8),
		   unit_vector (0,SinPiOn8,-CosPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke6);
  
  create_cylinder (position (0, -(spoke_len/2)*SinPiOn8,
			     -(spoke_len/2)*CosPiOn8),
		   unit_vector (0,-SinPiOn8,-CosPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke7);
  
  create_cylinder (position (0,-(spoke_len/2)*CosPiOn8,
			     -(spoke_len/2)*SinPiOn8),
		   unit_vector (0,-CosPiOn8,-SinPiOn8),
		   spoke_len,
		   box_dep,
		   &spoke8);

  api_unite (spoke1,base);
  api_unite (spoke2,base);
  api_unite (spoke3,base);
  api_unite (spoke4,base);
  api_unite (spoke5,base);
  api_unite (spoke6,base);
  api_unite (spoke7,base);
  api_unite (spoke8,base);

  makeit (base,"thing10");
  
  return;
}

void thing11 ()
{
  BODY *base;
  BODY *spoke1;
  BODY *spoke2;
  BODY *spoke3;
  BODY *spoke4;
 
  double CosPiOn8;
  double SinPiOn8;
  
  double box_dep = 2.0;
  double box_wid = 6.0;
  double spoke_len = 15.0;

  CosPiOn8 = sqrt (( 1+sqrt (2))/2); // proportional to
  SinPiOn8 = sqrt ((-1+sqrt (2))/2); // proportional to
  
  create_cylinder (position (0,0,0),
		   unit_vector (1,0,0),
		   box_dep,
		   box_wid,
		   &base);
  
  create_cylinder (position (0,(spoke_len/2)*CosPiOn8,(spoke_len/2)*SinPiOn8),
		   unit_vector (0,CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_dep/2,
		   &spoke1);
  
  create_cylinder (position (0, (spoke_len/2)*SinPiOn8,(spoke_len/2)*CosPiOn8),
		   unit_vector (0,SinPiOn8,CosPiOn8),
		   spoke_len,
		   box_dep/2,
		   &spoke2);
  
  create_cylinder (position (0, -(spoke_len/2)*SinPiOn8,
			     (spoke_len/2)*CosPiOn8),
		   unit_vector (0,-SinPiOn8,CosPiOn8),
		   spoke_len,
		   box_dep/2,
		   &spoke3);
  
  create_cylinder (position (0,-(spoke_len/2)*CosPiOn8,(spoke_len/2)*SinPiOn8),
		   unit_vector (0,-CosPiOn8,SinPiOn8),
		   spoke_len,
		   box_dep/2,
		   &spoke4);
  
  api_unite (spoke1,base);
  api_unite (spoke2,base);
  api_unite (spoke3,base);
  api_unite (spoke4,base);


  section (position (0,0,0),
	   position (1,0,0),
	   position (0,1,0),
	   position (0,0,1),
	   base);
  
  makeit (base,"thing11");
  
  return;
}

void thing12 ()
{
  BODY *base;
  BODY *cutter1;
  BODY *cutter2;
  BODY *cutter3;
  BODY *cutter4;
  BODY *axis;
  
  double axis_rad = 2.0;
  double whole_rad = 5.0;
  double cutter_len = 2.0;
  double cutter_rad = 8.0;
  double cutter_space = 4.0;
  double whole_len = 4.0 * cutter_space;

  create_cylinder (position (0,0,0),
		   unit_vector (0,1,0),
		   whole_len,
		   whole_rad,
		   &base);
  
  create_cylinder (position (0,-1.5*cutter_space,0),
		   unit_vector (0,1,0),
		   cutter_len,
		   cutter_rad,
		   &cutter1);

  create_cylinder (position (0,-0.5*cutter_space,0),
		   unit_vector (0,1,0),
		   cutter_len,
		   cutter_rad,
		   &cutter2);
  
  create_cylinder (position (0, 0.5*cutter_space,0),
		   unit_vector (0,1,0),
		   cutter_len,
		   cutter_rad,
		   &cutter3);
  
  create_cylinder (position (0, 1.5*cutter_space,0),
		   unit_vector (0,1,0),
		   cutter_len,
		   cutter_rad,
		   &cutter4);
  
  position pts[12];
  
  pts[ 1] = position ( axis_rad/2 ,-whole_len/2, -sqrt (3.0)*axis_rad/2 );
  pts[ 0] = position ( axis_rad   ,-whole_len/2, 0*axis_rad );
  pts[ 2] = position ( axis_rad/2 ,-whole_len/2, sqrt (3.0)*axis_rad/2 );
  pts[ 4] = position (-axis_rad/2 ,-whole_len/2, sqrt (3.0)*axis_rad/2 );
  pts[ 3] = position (-axis_rad   ,-whole_len/2, 0*axis_rad );
  pts[ 5] = position (-axis_rad/2 ,-whole_len/2, -sqrt (3.0)*axis_rad/2 );
  pts[ 6] = position ( axis_rad/2 , whole_len/2, -sqrt (3.0)*axis_rad/2 );
  pts[ 7] = position ( axis_rad   , whole_len/2, 0*axis_rad );
  pts[ 8] = position ( axis_rad/2 , whole_len/2, sqrt (3.0)*axis_rad/2 );
  pts[ 9] = position (-axis_rad/2 , whole_len/2, sqrt (3.0)*axis_rad/2 );
  pts[10] = position (-axis_rad   , whole_len/2, 0*axis_rad );
  pts[11] = position (-axis_rad/2 , whole_len/2, -sqrt (3.0)*axis_rad/2 );
  
  api_solid_block (position (-whole_rad,-whole_len,-whole_rad),
		  position ( whole_rad, whole_len, whole_rad),
		  axis);
  
  cut_convex (pts,12,axis);
  
  api_unite (cutter1,base);
  api_unite (cutter2,base);
  api_unite (cutter3,base);
  api_unite (cutter4,base);

  api_subtract (axis,base);
  
  makeit (base,"thing12");
  
  return;
}

void thing13 ()
{
  BODY *base;
  BODY *hole1;
  BODY *hole2;
  BODY *hole3;
  
  double len = 4.0;
  double hole_rad = 1.0;

  api_solid_block (position (-len,-len,-len),
		   position ( len, len, len),
		   base);
  
  create_cylinder (position ( (len-2*hole_rad),0, (len-2*hole_rad)),
		   unit_vector (0,1,0),
		   2*len,
		   hole_rad,
		   &hole1);

  create_cylinder (position (-(len-2*hole_rad),-(len-2*hole_rad),0),
		   unit_vector (0,0,1),
		   2*len,
		   hole_rad,
		   &hole2);
  
  create_cylinder (position (0, (len-2*hole_rad),-(len-2*hole_rad)),
		   unit_vector (1,0,0),
		   2*len,
		   hole_rad,
		   &hole3);

  api_subtract (hole1,base);
  api_subtract (hole2,base);
  api_subtract (hole3,base);

  makeit (base,"thing13");

  return;
}

void thing14 ()
{
  BODY *sphere1;
  BODY *sphere2;
  BODY *sphere3;
  BODY *sphere4;
  BODY *sphere5;
  
  double Tan1PiOn5;
  double Tan3PiOn5;
  
  Tan1PiOn5 = sqrt (5.0 + 2.0*sqrt (5.0));
  Tan3PiOn5 = sqrt (5.0 - 2.0*sqrt (5.0));
  
  position pts[10];
  unit_vector vec[5];
  
  vec[0] = unit_vector ( 1.0, 0.0,      0.0);
  vec[1] = unit_vector ( 1.0, Tan1PiOn5,0.0);
  vec[2] = unit_vector ( 1.0,-Tan1PiOn5,0.0);
  vec[3] = unit_vector (-1.0, Tan3PiOn5,0.0);
  vec[4] = unit_vector (-1.0,-Tan3PiOn5,0.0);

  pts[0] = position (10*vec[0].x (),10*vec[0].y (), 5);
  pts[1] = position (10*vec[1].x (),10*vec[1].y (), 5);
  pts[2] = position (10*vec[2].x (),10*vec[2].y (), 5);
  pts[3] = position (10*vec[3].x (),10*vec[3].y (), 5);
  pts[4] = position (10*vec[4].x (),10*vec[4].y (), 5);

  pts[5] = position (10*vec[0].x (),10*vec[0].y (),-5);
  pts[6] = position (10*vec[1].x (),10*vec[1].y (),-5);
  pts[7] = position (10*vec[2].x (),10*vec[2].y (),-5);
  pts[8] = position (10*vec[3].x (),10*vec[3].y (),-5);
  pts[9] = position (10*vec[4].x (),10*vec[4].y (),-5);
  
  create_sphere (pts[0],position (0,0,0),&sphere1);
  create_sphere (pts[1],position (0,0,0),&sphere2);
  create_sphere (pts[2],position (0,0,0),&sphere3);
  create_sphere (pts[3],position (0,0,0),&sphere4);
  create_sphere (pts[4],position (0,0,0),&sphere5);
  
  api_unite (sphere2,sphere1);
  api_unite (sphere3,sphere1);
  api_unite (sphere4,sphere1);
  api_unite (sphere5,sphere1);
  
  makeit (sphere1,"thing14");
  
  return;
}

void thing15 ()
{
  BODY *base;
  BODY *leg[5];
  position cpts[8];
  position pts[10];
  unit_vector vec[5];
  int i;
  
  double bs = 5.0;    // base radius, sort of
  double ts = 2.0;    // half base thickness
  double droop = 2.0; // how many times ts to droop the legs
  double ll = 4.0;    // how many time bs the legs are

  double Tan1PiOn5;
  double Tan3PiOn5;
  
  Tan1PiOn5 = sqrt (5.0 + 2.0*sqrt (5.0));
  Tan3PiOn5 = sqrt (5.0 - 2.0*sqrt (5.0));
  
  vec[0] = unit_vector ( 1.0, 0.0,      0.0);
  vec[1] = unit_vector ( 1.0, Tan1PiOn5,0.0);
  vec[2] = unit_vector (-1.0, Tan3PiOn5,0.0);
  vec[3] = unit_vector (-1.0,-Tan3PiOn5,0.0);
  vec[4] = unit_vector ( 1.0,-Tan1PiOn5,0.0);
 
  pts[0] = position (bs*vec[0].x (),bs*vec[0].y (), ts);
  pts[1] = position (bs*vec[1].x (),bs*vec[1].y (), ts);
  pts[2] = position (bs*vec[2].x (),bs*vec[2].y (), ts);
  pts[3] = position (bs*vec[3].x (),bs*vec[3].y (), ts);
  pts[4] = position (bs*vec[4].x (),bs*vec[4].y (), ts);
  
  pts[5] = position (bs*vec[0].x (),bs*vec[0].y (),-ts);
  pts[6] = position (bs*vec[1].x (),bs*vec[1].y (),-ts);
  pts[7] = position (bs*vec[2].x (),bs*vec[2].y (),-ts);
  pts[8] = position (bs*vec[3].x (),bs*vec[3].y (),-ts);
  pts[9] = position (bs*vec[4].x (),bs*vec[4].y (),-ts);
  
  create_convex (pts,10,&base);  

  // ---------------------------------------------------------------
  
  for (i=0;i<5;i++) {
    cpts[0] = pts[0+(i+0)%5];
    cpts[1] = pts[0+(i+1)%5];
    cpts[2] = pts[5+(i+0)%5];
    cpts[3] = pts[5+(i+1)%5];
    
    cpts[4] = position (pts[0+(i+0)%5].x ()*ll + pts[0+(i+1)%5].x ()*(ll+1),
		       pts[0+(i+0)%5].y ()*ll + pts[0+(i+1)%5].y ()*(ll+1), 
		       pts[0+(i+0)%5].z ()+droop*ts);
    
    cpts[5] = position (pts[0+(i+1)%5].x ()*ll + pts[0+(i+0)%5].x ()*(ll+1), 
		       pts[0+(i+1)%5].y ()*ll + pts[0+(i+0)%5].y ()*(ll+1), 
		       pts[0+(i+1)%5].z ()+droop*ts);
    
    cpts[6] = position (pts[5+(i+0)%5].x ()*ll + pts[5+(i+1)%5].x ()*(ll+1),
		       pts[5+(i+0)%5].y ()*ll + pts[5+(i+1)%5].y ()*(ll+1), 
		       pts[5+(i+0)%5].z ()+droop*ts);
    
    cpts[7] = position (pts[5+(i+1)%5].x ()*ll + pts[5+(i+0)%5].x ()*(ll+1), 
		       pts[5+(i+1)%5].y ()*ll + pts[5+(i+0)%5].y ()*(ll+1), 
		       pts[5+(i+1)%5].z ()+droop*ts);

    create_convex (cpts,8,&leg[i]);
  }

  api_unite (leg[0],base);
  api_unite (leg[1],base);
  api_unite (leg[2],base);
  api_unite (leg[3],base);
  api_unite (leg[4],base);
 
  makeit (base,"thing15");
 
  return;
}

void thing16 ()
{
  BODY *strut;
  BODY *flange1;
  BODY *flange2;
  BODY *hole;
  BODY *bolt1;
  BODY *bolt2;
  BODY *bolt3;
  BODY *bolt4;
  
  // The following defines a pipe with a flat square flange on
  // either end. Each flange has a bolt hole in each corner
  
  double sl = 10.0;    // length of the strut
  double sr = 3.0;     // radius of strut
  double st = 1.0;     // thickness of strut material
  double fl = 5.0;     // size of the flange
  double ft = 1.0;     // thickness of the flange
  double bs = 0.5;     // radius of bolt holes

  create_cylinder (position (0,0,0),
		   unit_vector (0,0,1),
		   sl,
		   sr,
		   &strut);

  create_cylinder (position (0,0,0),
		   unit_vector (0,0,1),
		   sl,
		   sr-st,
		   &hole);

  create_cylinder (position ( (fl-2*bs), (fl-2*bs), 0),
		   unit_vector (0,0,1),
		   sl,
		   bs,
		   &bolt1);
  
  create_cylinder (position ( (fl-2*bs),-(fl-2*bs), 0),
		   unit_vector (0,0,1),
		   sl,
		   bs,
		   &bolt2);
  
  create_cylinder (position (-(fl-2*bs), (fl-2*bs), 0),
		   unit_vector (0,0,1),
		   sl,
		   bs,
		   &bolt3);
  
  create_cylinder (position (-(fl-2*bs),-(fl-2*bs), 0),
		   unit_vector (0,0,1),
		   sl,
		   bs,
		   &bolt4);
  
  api_solid_block (position (fl,fl,sl/2),
		   position (-fl,-fl,sl/2-ft),
		   flange1);
  
  api_solid_block (position (fl,fl,-sl/2),
		   position (-fl,-fl,-(sl/2-ft)),
		   flange2);
  
  api_unite (flange1,strut);
  api_unite (flange2,strut);
  api_subtract (hole,strut);
  api_subtract (bolt1,strut);
  api_subtract (bolt2,strut);
  api_subtract (bolt3,strut);
  api_subtract (bolt4,strut);
  
  makeit (strut,"thing16");
  
  return;
}

void thing17 ()
{
  BODY *wheel;
  BODY *hole1;
  BODY *hole2;
  
  // The following is basically just a sphere with two diametrically
  // opposed cylindrical depressions in it, the idea is that it is
  // a wheel, the forked holder for this is also in the collection.
  
  double rad = 10.0;  // the radius of the ball
  double dep = 2.0;   // the depth of the holes
  double hrad = 2.0;  // the radius of the hole

  create_sphere (position (0,0,0),
		 position (rad,0,0),
		 &wheel);
  
  create_cylinder (position ((rad-dep/2),0,0),
		   unit_vector (1,0,0),
		   dep,
		   hrad,
		   &hole1);
  
  create_cylinder (position (-(rad-dep/2.0),0,0),
		   unit_vector (1,0,0),
		   dep,
		   hrad,
		   &hole2);

  api_subtract (hole1,wheel);
  api_subtract (hole2,wheel);
  
  makeit (wheel,"thing17");
  
  return;
}

void thing18 ()
{
  BODY *handle;
  BODY *upper1;
  BODY *upper2;
  BODY *lower1;
  BODY *lower2;
  BODY *axel1;
  BODY *axel2;

  double hl = 10.0;   // length of handle
  double hr = 2.0;    // radius of handle
  double ul = 10.0;   // length of upper
  double ur = 2.0;    // radius of upper
  double ll = 8.0;    // length of lower
  double lr = 2.0;    // radius of lower
  double al = 5.0;    // length of axel
  double ar = 1.0;    // radius of axel

  double hmx = 0.0;
  double hmy = 0.0;
  double hmz = hl / 2.0;

  double umx = 0.0;
  double umy = ul / sqrt (8.0);
  double umz = ul / sqrt (8.0);

  double lmx = 0.0;
  double lmy = ul / sqrt (2.0);
  double lmz = ul / sqrt (2.0) + ll / 2.0 - ur /sqrt (8.0);

  double amx = 0.0;
  double amy = ul / sqrt (2.0) - al / 2.0;
  double amz = ul / sqrt (2.0) + ll - 2.0 * ar;

  create_cylinder (position (0,0,hl/2),
		   unit_vector (hmx, hmy, hmz),
		   hl,
		   hr,
		   &handle);

  create_cylinder (position (umx,umy,-umz),
		   unit_vector (0,1,-1),
		   ul + (lr/sqrt (8)),
		   ur,
		   &upper1);

  create_cylinder (position (umx,-umy,-umz),
		   unit_vector (0,-1,-1),
		   ul + (lr/sqrt (8)),
		   ur,
		   &upper2);

  create_cylinder (position (lmx,lmy,-lmz),
		   unit_vector (0,0,-1),
		   ll,
		   lr,
		   &lower1);
  
  create_cylinder (position (lmx,-lmy,-lmz),
		   unit_vector (0,0,-1),
		   ll,
		   lr,
		   &lower2);

  create_cylinder (position (amx,amy,-amz),
		   unit_vector (0,1,0),
		   al,
		   ar,
		   &axel1);
  
  create_cylinder (position (amx,-amy,-amz),
		   unit_vector (0,1,0),
		   al,
		   ar,
		   &axel2);

  api_unite (upper1,handle);
  api_unite (upper2,handle);
  api_unite (lower1,handle);
  api_unite (lower2,handle);
  api_unite (axel1,handle);
  api_unite (axel2,handle);
  
  makeit (handle,"thing18");
  
  return;
}

void thing19 ()
{
  BODY *knub;
  BODY *wing1;
  BODY *wing2;
  BODY *cut1;
  BODY *cut2;
    
  double kr = 3.0; // radius of knub
  double kh = 2.0; // knub height
  double wr = 6.0; // radius of wing
  double wh = 1.0; // wing height

  double kmx = 0.0;
  double kmy = 0.0;
  double kmz = 0.0;

  double wmx = 0.0;
  double wmy = wr - kr;
  double wmz = 0.0;
  
  create_cylinder (position (kmx,kmy,kmz),
		   unit_vector (0,0,1),
		   kh,
		   kr,
		   &knub);

  create_cylinder (position (wmx,wmy,wmz),
		   unit_vector (0,0,1),
		   wh,
		   wr,
		   &wing1);
  
  section (position (0,0,0),
	   position (0,1,0),
	   position (0,0,1),
	   position (1,0,0),
	   wing1);
  
  create_cylinder (position (wmx,-wmy,wmz),
		   unit_vector (0,0,1),
		   wh,
		   wr,
		   &wing2);
  
  section (position (0,0,0),
	   position (0,1,0),
	   position (0,0,1),
	   position (-1,0,0),
	   wing2);

  create_cylinder (position (0,(kr+wmy+wr)/2,0),
		   unit_vector (0,0,1),
		   wh,
		   (wmy+wr-kr)/2,
		   &cut1);
  
  create_cylinder (position (0,-(kr+wmy+wr)/2,0),
		   unit_vector (0,0,1),
		   wh,
		   (wmy+wr-kr)/2,
		   &cut2);

  api_unite (wing1,knub);
  api_unite (wing2,knub);
  api_subtract (cut1,knub);
  api_subtract (cut2,knub);
  
  makeit (knub,"thing19");
  
  return;
}

void thing20 ()
{
  BODY *plate;
  BODY *hole[6];
  BODY *slot[3];
  
  position pts[12];
  unit_vector u[12];
  
  double pr = 5.0 * 5.0;
  double pt = 1.0 * 5.0;
  double hp = 2.5 * 5.0;
  double hr = 1.0 * 5.0;
  double sl = 1.0 * 5.0;
  double sw = 0.2 * 5.0;

  u[ 0] = unit_vector ( 2,       0, 1);
  u[ 1] = unit_vector ( 1, sqrt (3), 1);
  u[ 2] = unit_vector (-1, sqrt (3), 1);
  u[ 3] = unit_vector (-2,       0, 1);
  u[ 4] = unit_vector (-1,-sqrt (3), 1);
  u[ 5] = unit_vector ( 1,-sqrt (3), 1);
  
  u[ 6] = unit_vector ( 2,       0,-1);
  u[ 7] = unit_vector ( 1, sqrt (3),-1);
  u[ 8] = unit_vector (-1, sqrt (3),-1);
  u[ 9] = unit_vector (-2,       0,-1);
  u[10] = unit_vector (-1,-sqrt (3),-1);
  u[11] = unit_vector ( 1,-sqrt (3),-1);
  

  for (int i = 0; i < 12; ++i)
    pts[i] = position (u[i].x () * pr, u[i].y () * pr, u[i].z () * pt);
  create_convex (pts, 12, &plate);
  
  for (int i = 0; i < 6; ++i)
    create_cylinder (position (u[i].x () * hp, u[i].y () * hp, u[i].z () * hp),
		     unit_vector (0.0, 0.0, 1.0),
		     2.0 * pr,
		     hr,
		     &(hole[i]));
  
  for (int i = 0; i < 6; ++i)
    api_subtract (hole[i], plate);
  
  for (int i = 0; i < 3; ++i) {
    unit_vector d1 ((u[(2 * i + 0) % 6].x () + u[(2 * i + 1) % 6].x ()) / 2,
		    (u[(2 * i + 0) % 6].y () + u[(2 * i + 1) % 6].y ()) / 2,
		    0);
    unit_vector d2 (u[(2 * i + 2) % 6].x (), u[(2 * i + 2) % 6].y (), 0);
    
    position p ((pr * u[(2 * i + 0) % 6].x () + 
		 pr * u[6 + (2 * i + 1) % 6].x ()) / 2,
		(pr * u[(2 * i + 0) % 6].y () + 
		 pr * u[6 + (2 * i + 1) % 6].y ()) / 2,
		(pr * u[(2 * i + 0) % 6].z () + 
		 pr * u[6 + (2 * i + 1) % 6].z ()) / 2);
    
    pts[0] = position (p.x () + sl * d1.x () + sw * d2.x (),
		       p.y () + sl * d1.y () + sw * d2.y (), pt);
    pts[1] = position (p.x () + sl * d1.x () - sw * d2.x (),
		       p.y () + sl * d1.y () - sw * d2.y (), pt);
    pts[2] = position (p.x () - sl * d1.x () + sw * d2.x (),
		       p.y () - sl * d1.y () + sw * d2.y (), pt);
    pts[3] = position (p.x () - sl * d1.x () - sw * d2.x (),
		       p.y () - sl * d1.y () - sw * d2.y (), pt);
    
    pts[4] = position (p.x () + sl * d1.x () + sw * d2.x (),
		       p.y () + sl * d1.y () + sw * d2.y (), -pt);
    pts[5] = position (p.x () + sl * d1.x () - sw * d2.x (),
		       p.y () + sl * d1.y () - sw * d2.y (), -pt);
    pts[6] = position (p.x () - sl * d1.x () + sw * d2.x (),
		       p.y () - sl * d1.y () + sw * d2.y (), -pt);
    pts[7] = position (p.x () - sl * d1.x () - sw * d2.x (),
		       p.y () - sl * d1.y () - sw * d2.y (), -pt);
    
    create_convex (pts, 8, &(slot[i]));
  }
  
  for (int i = 0; i < 3; ++i)
    api_subtract (slot[i], plate);
  
  makeit (plate, "thing20");
  
  return;
}

void thing21 ()
{
  double br = 3.0 * 5.0;
  double bt = 0.5 * 5.0;
  double hr = 2.8 * 5.0;
  double hd = 0.4 * 5.0;

  BODY *base;
  sil_acis_check (api_solid_block (position (br, br, bt),
				   position (-br, -br, -bt),
				   base),
		  "things", "api_solid_block");
  BODY *hole;
  sil_acis_check (api_solid_block (position (hr, hr, bt),
				   position (-hr, -hr, -(bt - hd)),
				   hole),
		  "things", "api_solid_block");

  sil_acis_check (api_subtract (hole, base),
		  "things", "api_subtract");
  
  makeit (base, "thing21");
  
  return;
}

void thing22 ()
{
  double pt = 2.0;
  double ph = 5.0;
  double pr = 30.0;

  double alpha = 1.0 / sqrt (2.0);
  double beta = 1.0 + alpha;

  unit_vector u[8];
  u[0] = unit_vector (beta, alpha, 0);
  u[1] = unit_vector (alpha, beta, 0);
  u[2] = unit_vector (-alpha, beta, 0);
  u[3] = unit_vector (-beta, alpha, 0);
  u[4] = unit_vector (-beta, -alpha, 0);
  u[5] = unit_vector (-alpha, -beta, 0);
  u[6] = unit_vector (alpha, -beta, 0);
  u[7] = unit_vector (beta, -alpha, 0);
  
  position pts[16];
  for (int i = 0; i < 16; ++i)
    pts[i] = position (pr * u[i % 8] . x (),
		       pr * u[i % 8] . y (),
		       ph * (1.0 - (double)i / 8.0) - pt * ((double)i / 8.0));
  BODY *plate;
  create_convex (pts, 16, &plate);
  
  for (int i = 0; i < 16; ++i)
    pts[i] = position (1.1*pr*(u[i%8].x ()+u[(i+1)%8].x ())/2,
		      1.1*pr*(u[i%8].y ()+u[(i+1)%8].y ())/2,ph*(i/8));
  BODY *hole;
  create_convex (pts, 16, &hole);
  
  sil_acis_check (api_subtract (hole,plate),
		  "things", "api_subtract");
  
  makeit (plate, "thing22");
  
  return;
}

void thing23 ()
{
  BODY *sqr1;
  BODY *cap1a;
  BODY *cap1b;
  BODY *cap1c;
  BODY *chan1;
  BODY *chan2;
  BODY *chan3;
  BODY *chan4;
  BODY *smooth1;
  BODY *smooth2;
  
  double lt = 1.0;   // thickness of each layer
  double lw = 14.0;  // width of the layer
  double lh = 11.0;  // height of rectangular part of layer

  sil_acis_check (api_solid_block (position (0.0, 0.0, 0.0),
				   position (lw, lh, lt),
				   sqr1),
		  "things", "api_solid_block");
  
  create_cylinder (position (7.0 * lw / 14.0, lh, lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   7.0 * lw / 14.0,
		   &cap1a);
  create_cylinder (position (7.0 * lw / 14.0, lh, lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   5.0 * lw / 14.0,
		   &cap1b);
  create_cylinder (position (7.0 * lw / 14.0, lh, lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   3.0 * lw / 14.0,
		   &cap1c);
  
  sil_acis_check (api_subtract (cap1b, cap1a),
		  "things", "api_subtract");
  sil_acis_check (api_unite (cap1c, cap1a),
		  "things", "api_unite");
  
  sil_acis_check (api_unite (cap1a, sqr1),
		  "things", "api_unite");
  
  sil_acis_check (api_solid_block (position (6.0 * lw / 14.0, 0.0 * lh / 11.0,
					    0.0),
				  position (8.0 * lw / 14.0, 12.0 * lh / 11.0,
					    lt),
				  chan1),
		 "things", "api_solid_block");
  sil_acis_check (api_subtract (chan1, sqr1),
		  "things", "api_subtract");
  
  sil_acis_check (api_solid_block (position (2.0 * lw / 14.0, 2.0 * lh / 11.0,
					     0.0),
				   position (4.0 * lw / 14.0, 12.0 * lh / 11.0,
					     lt),
				   chan2),
		  "things", "api_solid_block");
  sil_acis_check (api_subtract (chan2, sqr1),
		 "things", "api_subtract");
  
  sil_acis_check (api_solid_block (position (10.0 * lw / 14.0, 2.0 * lh / 11.0,
					     0.0),
				   position (12.0 * lw / 14.0, 
					     12.0 * lh / 11.0, lt),
				   chan3),
		  "things", "api_solid_block");
  sil_acis_check (api_subtract (chan3,sqr1),
		  "things", "api_subtract");
  
  sil_acis_check (api_solid_block (position (4.0 * lw / 14.0, 6.0 * lh / 11.0,
					     0),
				   position (6.0 * lw / 14.0, 10.0 * lh / 11.0,
					     lt),
				   chan4),
		  "things", "api_solid_block");
  sil_acis_check (api_subtract (chan4, sqr1),
		  "things", "api_subtract");
  
  create_cylinder (position (5.0 * lw / 14.0, 6.0 * lh / 11.0, lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   1.0 * lw / 14.0,
		   &smooth1);
  
  create_cylinder (position (5.0 * lw / 14.0, 10.0 * lh / 11.0, lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   1.0 * lw / 14.0,
		   &smooth2);
  
  sil_acis_check (api_unite (smooth1, sqr1),
		  "things", "api_unite");
  sil_acis_check (api_unite (smooth2,sqr1),
		  "things", "api_unite");

  // ------------------- Layer one is complete ----------------------
  
  BODY *sqr2;
  BODY *cap2;
  BODY *chan11;
  BODY *chan12;
  BODY *chan13;
  BODY *chan14;
  
  sil_acis_check (api_solid_block (position (0.0, 0.0, 0.0),
				   position (lw, lh, -lt),
				   sqr2),
		  "things", "api_solid_block");
  
  create_cylinder (position (7.0 * lw / 14.0, lh, -lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   7.0 * lw / 14.0,
		   &cap2);
  sil_acis_check (api_unite (cap2, sqr2),
		  "things", "api_unite");
  
  sil_acis_check (api_solid_block (position (10.0 * lw / 14.0 ,6.0 * lh / 11.0,
					     0.0),
				   position (12.0 * lw / 14.0, 
					     10.0 * lh / 11.0, -lt),
				   chan11),
		  "things", "api_solid_block");
  sil_acis_check (api_solid_block (position (10.0 * lw / 14.0, 2.0 * lh / 11.0,
					     0.0),
				   position (12.0 * lw / 14.0,
					     4.0 * lh / 11.0, -lt),
				   chan12),
		  "things", "api_solid_block");
  sil_acis_check (api_solid_block (position (6.0 * lw / 14.0, 11.0 * lh / 11.0,
					     0.0),
				   position (8.0 * lw / 14.0,
					     13.0 * lh / 11.0, -lt),
				   chan13),
		  "things", "api_solid_block");
  sil_acis_check (api_solid_block (position (2.0 * lw / 14.0, 2.0 * lh / 11.0,
					     0.0),
				   position (4.0 * lw / 14.0,
					     4.0 * lh / 11.0, -lt),
				   chan14),
		  "things", "api_solid_block");

  sil_acis_check (api_subtract (chan11, sqr2),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (chan12, sqr2),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (chan13, sqr2),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (chan14, sqr2),
		  "things", "api_subtract");
  sil_acis_check (api_unite (sqr2,sqr1),
		  "things", "api_unite");
  
  // --------------------- layer two is complete---------------------
  
  BODY *sqr3;
  BODY *cap3;
  BODY *chan31;
  BODY *chan32;
  BODY *chan33;
  
  sil_acis_check (api_solid_block (position (0.0, 0.0, -lt),
				   position (lw, lh, -2.0 * lt),
				   sqr3),
		  "things", "api_solid_block");

  create_cylinder (position (7.0 * lw / 14.0, lh, -3.0 * lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   7.0 * lw / 14.0,
		   &cap3);
  
  sil_acis_check (api_unite (cap3,sqr3),
		  "things", "api_unite");
  
  sil_acis_check (api_solid_block (position (10.0 * lw / 14.0,
					     2.0 * lh / 11.0, -lt),
				   position (12.0 * lw / 14.0,
					     4.0 * lh / 11.0, -2.0 * lt),
				   chan31),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (6.0 * lw / 14.0,
					     11.0 * lh / 11.0, -lt),
				   position (8.0 * lw / 14.0, 
					     13.0 * lh / 11.0, -2.0 * lt),
				   chan32),
		  "things", "api_solid_block");
  
  sil_acis_check (api_solid_block (position (2.0 * lw / 14.0, 2.0 * lh / 11.0,
					     -lt),
				   position (4.0 * lw / 14.0, 4.0 * lh / 11.0,
					     -2.0 * lt),
				   chan33),
		  "things", "api_solid_block");
  
  sil_acis_check (api_subtract (chan31, sqr3),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (chan32,sqr3),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (chan33,sqr3),
		  "things", "api_subtract");
  sil_acis_check (api_unite (sqr3,sqr1),
		  "things", "api_unite");
  
  // ------------------- layer 3 is complete --------------------------
  
  BODY *rim;
  BODY *rim_hole;
  BODY *cap;
  BODY *cap_hole;
  BODY *chan;
  
  sil_acis_check (api_solid_block (position (1.0 * lw / 14.0,
					     1.0 * lh / 11.0, lt),
				   position (13.0 * lw / 14.0,
					     10.0 * lh / 11.0, 2.0 * lt),
				   rim),
		  "things", "api_solid_block");
  
  create_cylinder (position (7.0 * lw / 14.0, lh, 3.0 * lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   6.0 * lw / 14.0,
		   &cap);
  
  create_cylinder (position (7.0 * lw / 14.0, lh, 3.0 * lt / 2.0),
		   unit_vector (0.0, 0.0, 1.0),
		   lt,
		   5.0 * lw / 14.0,
		   &cap_hole);
  
  sil_acis_check (api_solid_block (position (2.0 * lw / 14.0, 
					     2.0 * lh / 11.0, lt),
				   position (12.0 * lw / 14.0,
					     10.0 * lh / 11.0 ,2.0 * lt),
				   rim_hole),
		  "things", "api_solid_block");
  sil_acis_check (api_solid_block (position (6*lw/14, 0*lh/11,lt),
				   position (8*lw/14,12*lh/11,2*lt),
				   chan),
		  "things", "api_solid_block");
  
  sil_acis_check (api_subtract (chan, rim),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (cap_hole, cap),
		  "things", "api_subtract");
  sil_acis_check (api_unite (cap,rim),
		  "things", "api_unite");
  sil_acis_check (api_subtract (rim_hole, rim),
		  "things", "api_subtract");
  sil_acis_check (api_unite (rim, sqr1),
		  "things", "api_unite");

  makeit (sqr1, "thing23");

  return;
}

// A bit of technical geometric jiggery-pokery.
// 
// Suppose that we have a cylinder C of radius r, and a line L, that
// is parallel to the axis, A, of C, and a distance r+d away from A.
// Given that d>0 there must be two planes Pu and Pd that contain L
// and are tangent to the cylinder. The problem of finding these planes
// is essentially a two dimensional problem.
// 
// 
// Given a circle in the plane, centre x, and a point in the plane, 
// outside the circle, centre y, find the two points at which the lines
// tangent to the circle, and going through y, intersect the circle.
// 
//          .
//              .
//                  .
//                *   * .    
//            *          .t .
//          *           .   *  .
//                     .          .
//        *           .       *      .
//                   .                  .
//       *          x          *          y
// 
//        *                   *
// 
//          *                *
//            *            *
//                *    * 
// 
// Let the distance from `x' to `y' be `alpha', and the distance 
// from `t' to y' be beta, and the `vertical' distance from the
// line xy to `t', be `b', and the horixontal distance from `x'
// to the projection of `t', be `a'
// 
// Then, using similar triangles in the right angle triangle xty,
// 
// we get ....
// 
// 
// a = r^2/alpha
// b = sqrt (r^2-a^2)
// 
// The rest of the routine is just handling the 3D issues.
// 
// note: the routine is based on the assumption that u and v
// are orthogonal, if they are not the result refers to
// a skewed cylinder, the amount of skew being indicated by
// the amount of non orthogonality ('scuse the term) between
// u and v.

void find_tangent_points (position p,    // a point on the axis of the cylinder
			  unit_vector u, // the direction of the axis
			  double r,      // the radius of the cylinder
			  unit_vector v, // the direction to proceed from p
			  double alpha,  // the distance to move out
			  position *t1,  // upper return value
			  position *t2)  // lower return value
{
  double a = r * r / alpha;
  double b = sqrt (r * r - a * a);
  
  vector dy = b * (u * v);
  vector dx = a * v;

  *t1 = p + dx + dy;
  *t2 = p + dx - dy;

  return;
}


// Create a barge shaped object.
// 
// 
//       +-----------+-----+   .
//      /|              * /    .
//     / +-----------+ * /     .
//    / /           /   /      .
//   +-----------+-----+       .
//   |           |   *         .
//   +-----------+ *           .
// 
// Which is basically a box with a bit of a cylinder stuck on
// the front for a prow. However there are some details that
// need to be sorted out to get it to hold together.

void create_barge (unit_vector H,      // height co-ordinate axis
		   unit_vector B,      // bredth co-ordinate axis
		   unit_vector D,      // depth co-ordinate axis
		   position p,         // mid point cargo hold
		   double h,           // height of cargo hold
		   double b,           // width of cargo hold
		   double d,           // length of cargo hold
		   double prow,        // length of the prow
		   BODY **barge)
{
  position pts[8];
  pts[0] =  p + h * H / 2.0 + b * B / 2.0 + d * D / 2.0;
  pts[1] =  p + h * H / 2.0 + b * B / 2.0 - d * D / 2.0;
  pts[2] =  p + h * H / 2.0 - b * B / 2.0 + d * D / 2.0;
  pts[3] =  p + h * H / 2.0 - b * B / 2.0 - d * D / 2.0;
  pts[4] =  p - h * H / 2.0 + b * B / 2.0 + d * D / 2.0;
  pts[5] =  p - h * H / 2.0 + b * B / 2.0 - d * D / 2.0;
  pts[6] =  p - h * H / 2.0 - b * B / 2.0 + d * D / 2.0;
  pts[7] =  p - h * H / 2.0 - b * B / 2.0 - d * D / 2.0;
  
  create_convex (pts, 8, barge);
  
  double alpha = ((h*h+prow*prow)/(2*h));
  
  position cc = p - h * H / 2.0 + d * D / 2.0 + alpha * H;
  unit_vector cd = B;
  double cr = alpha;
  double cl = b;

  BODY *cyl;
  create_cylinder (cc, cd, cl, cr, &cyl);
  
  api_unite (cyl, *barge);

  section (p + h * H / 2.0 + b * B / 2.0 + d * D / 2.0,
	   p + h * H / 2.0 - b * B / 2.0 + d * D / 2.0,
	   p + h * H / 2.0 - d * D / 2.0,
	   p,
	   *barge);
  return;
}

void thing24 ()
{
  // Think of the key as roughly as follows:
  // 
  //                                  __________
  //                                 |          |
  //        ____________________-----|     __   |
  //      ( ------------------------      |  |  |
  //       \ -----------------------      |  |  |
  //        \___________________          |  |  |
  //                            |____     |__|  |
  //                                 |          |
  //                                 |__________|
  // 
  // Going from left to right we have three basic section, the leader,
  // the base and the handle. The leader has channels on it that are
  // basically \__/ in cross section, but at the end near the handle
  // they dont just come to a blank stop, rather they peter out with
  // a cylindrical surface (see the function that generates barges).
  //  
  // Now from left to right there are three basic lengths, the length
  // of the leader, the base and the handle. But we also have to worry
  // about the length of the barges. Heights on the leader are measured
  // in terms of mbh, which is the minumum barge height (nominal).
  
  double mbh = 0.33;         // nominal minimum barge height
  double Bl = 5.0;           // base length
  double Ll = 20.0;          // leader length
  double Hl = 15.0;          // handle length

  double kh = 14.5 * mbh;    // height of the base
  double kl = Ll + Bl;       // length of leader + base
  double kt = 1.0;           // key thickness
  double bp = 3.0 * mbh;     // parge prow length
  double bl = kl - bp;       // length of barges
  double bt = 0.33;          // nominal dept of barges
  
  BODY *thing;
  sil_acis_check (api_solid_block (position (0.0, 0.0, -2.0 * mbh),
				   position (kt, kl, kh),
				   thing),
		  "things", "api_solid_block");

  BODY *barge1;
  create_barge (unit_vector (1.0, 0.0, 0.0),
		unit_vector (0.0, 0.0, 1.0),
		unit_vector (0.0, 1.0, 0.0),
		position (kt - bt / 2.0, bl / 2.0, 4.0 * mbh / 2.0),
		bt, 4.0 * mbh, bl, bp,
		&barge1);
  BODY *barge3;
  create_barge (unit_vector (1.0, 0.0, 0.0),
		unit_vector (0.0, 0.0, 1.0),
		unit_vector (0.0, 1.0, 0.0),
		position (kt - bt / 2.0, bl / 2.0,
			  10.5 * mbh + 4.0 * mbh / 6.0),
		2.0 * bt, 4.0 * mbh / 3.0, bl, bp,
		&barge3);
  BODY *barge4;
  create_barge (unit_vector (1.0, 0.0, 0.0),
		unit_vector (0.0, 0.0, 1.0),
		unit_vector (0.0, 1.0, 0.0),
		position (kt - bt / 2.0, bl / 2.0,
			  5.5 * mbh + 4.0 * mbh / 6.0),
		2.0 * bt, 4.0 * mbh / 3.0, bl, bp,
		&barge4);
  BODY *barge6;
  create_barge (unit_vector (1.0, 0.0, 0.0),
		unit_vector (0.0, 0.0, 1.0),
		unit_vector (0.0, 1.0, 0.0),
		position (kt - bt / 2.0, bl / 2.0, kh - mbh),
		1.0 * bt, mbh, bl, bp,
		&barge6);
  BODY *barge2;
  create_barge (unit_vector (-1.0, 0.0, 0.0),
		unit_vector (0.0, 0.0, 1.0),
		unit_vector (0.0, 1.0, 0.0),
		position (bt / 2.0, bl / 2.0, 3.0 * mbh / 2.0),
		bt, 4.0 * mbh, bl, bp,
		&barge2);
  BODY *barge5;
  create_barge (unit_vector (-1.0, 0.0, 0.0),
		unit_vector (0.0, 0.0, 1.0),
		unit_vector (0.0, 1.0, 0.0),
		position (bt / 2.0, bl / 2.0, 8.0 * mbh + 4.0 * mbh / 6.0),
		2.0 * bt, 4.0 * mbh / 3.0, bl, bp,
		&barge5);
  BODY *barge7;
  create_barge (unit_vector (-1.0, 0.0, 0.0),
		unit_vector (0.0, 0.0, 1.0),
		unit_vector (0.0, 1.0, 0.0),
		position (bt / 2.0, bl / 2.0, kh - mbh),
		1.0 * bt, mbh, bl ,bp,
		&barge7);

  // ---- Now round off the leading end of the key -----
  
  position t1;
  position t2;
  position p (kt / 2.0, kh / 5.0, 6.0 * kh / 9.0);
  position q (kt / 2.0, -3.0 * kh / 20.0, 7.0 * kh / 9.0);
  unit_vector u (1.0, 0.0, 0.0);
  unit_vector v ((q - p) . x (), (q - p) . y (), (q - p) . z ());
  double r = p.y ();
  double alpha = (q - p ) . len ();
  find_tangent_points (p, u, r, v, alpha, &t1, &t2);

  section (q + u, q - u, t1, p, thing);
  section (q + u, q - u, t2, p, thing);

  BODY *fillet;
  BODY *cyl;
  sil_acis_check (api_copy_body (thing, fillet),
		  "things", "api_copy_body");
  section (q + u, q - u, t1, p, fillet);
  section (q + u, q - u, t2, p, fillet);
  section (p + u, p - u, t1, q, fillet);
  section (p + u, p - u, t2, q, fillet);
  create_cylinder (p, u, 2.0 * kt, r, &cyl);
  sil_acis_check (api_subtract (cyl, fillet),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (fillet, thing),
		  "things", "api_subtract");

  // ---- Now form the base by cutting away material ----
  
  BODY *cut1;
  BODY *cut2;
  sil_acis_check (api_solid_block (position (0.0, 0.0, 0.0),
				   position (kt, Ll, -3.0 * mbh),
				   cut1),
		  "things", "api_solid_block");
  sil_acis_check (api_solid_block (position (0.0, 0.0, kh),
				   position (kt, Ll, kh - 3.0 * mbh / 2.0),
				   cut2),
		  "things", "api_solid_block");

  sil_acis_check (api_subtract (cut1,thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (cut2,thing),
		  "things", "api_subtract");

  // ---- Now add the handle -----

  double hcr = Hl / 5.0;  // radius of round corners of handle
  double Hh = 3.0 * kh;   // height of handle

  position ho (0,kl,-kh);   // origin for handle calculations

  BODY *hand1;
  sil_acis_check (api_solid_block (ho + vector (0.0, 0.0, hcr),
				   ho + vector (kt, Hl, Hh - hcr),
				   hand1),
		  "things", "api_solid_block");
  BODY *hand2;
  sil_acis_check (api_solid_block (ho + vector (0, hcr, 0),
				   ho + vector (kt, Hl - hcr, Hh),
				   hand2),
		  "things", "api_solid_block");

  sil_acis_check (api_unite (hand1, thing),
		  "things", "api_unite");
  sil_acis_check (api_unite (hand2, thing),
		  "things", "api_unite");
  
  BODY *corn1;
  create_cylinder (ho + vector (kt / 2.0, hcr, hcr),
		   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hcr,
		   &corn1);
  BODY *corn2;
  create_cylinder (ho + vector (kt / 2.0, hcr, Hh - hcr),
		   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hcr,
		   &corn2);
  BODY *corn3;
  create_cylinder (ho + vector (kt / 2.0, Hl - hcr, hcr),
		   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hcr,
		   &corn3);
  BODY *corn4;
  create_cylinder (ho + vector (kt / 2.0, Hl - hcr, Hh - hcr),
		   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hcr,
		   &corn4);
  
  sil_acis_check (api_unite (corn1, thing),
		  "things", "api_unite");
  sil_acis_check (api_unite (corn2, thing),
		  "things", "api_unite");
  sil_acis_check (api_unite (corn3, thing),
		  "things", "api_unite");
  sil_acis_check (api_unite (corn4, thing),
		  "things", "api_unite");
  
  // ---- Now put a hole in the handle ----
 
  double hole_h = 6.0 * Hh / 16.0;    // height of hole
  double hole_l = 7.0 * Hl / 27.0;    // length of hole
  double hole_r = Hh / 10.0;          // radius of corners

  position hole_o = ho + vector (0.0, 3.0 * Hl / 5.0, (Hh - hole_h) / 2.0);

  BODY *hole1;  
  sil_acis_check (api_solid_block (hole_o + vector (0.0, 0.0, hole_h - hole_r),
				   hole_o + vector (kt, hole_l, hole_r),
				   hole1),
		  "things", "api_solid_block");
  
  BODY *hole2;
  sil_acis_check (api_solid_block (hole_o+vector (0.0, hole_l - hole_r, 0),
				   hole_o+vector (kt, hole_r, hole_h),
				   hole2),
		  "things", "api_solid_block");

  BODY *corn5;  
  create_cylinder (hole_o + vector (kt / 2.0, hole_r, hole_r),
				   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hole_r,
		   &corn5);
  BODY *corn6;  
  create_cylinder (hole_o + vector (kt / 2.0, hole_r, hole_h - hole_r),
		   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hole_r,
		   &corn6);
  BODY *corn7;
  create_cylinder (hole_o + vector (kt / 2.0, hole_l - hole_r, hole_r),
		   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hole_r,
		   &corn7);
  BODY *corn8;
  create_cylinder (hole_o + vector (kt / 2.0, hole_l - hole_r,
				    hole_h - hole_r),
		   unit_vector (1.0, 0.0, 0.0),
		   kt,
		   hole_r,
		   &corn8);
  
  sil_acis_check (api_subtract (hole1, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (hole2, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (corn5, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (corn6, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (corn7, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (corn8, thing),
		  "things", "api_subtract");

  // ----- Now cut the barges in the key -----

  sil_acis_check (api_subtract (barge1, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (barge2, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (barge3, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (barge4, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (barge5, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (barge6, thing),
		  "things", "api_subtract");
  sil_acis_check (api_subtract (barge7, thing),
		  "things", "api_subtract");
  
  makeit (thing, "thing24");
  
  return;
}

SIL_PLUGIN ()
{
  sil_register_tool (&thing01, "thing01", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "Square block from survey paper", 
		     "acis", "acis");
  sil_register_tool (&thing02, "thing02", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "Table base with C2", 
		     "acis", "acis");
  sil_register_tool (&thing03, "thing03", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "Semi-stelated cube-octohedron", 
		     "acis", "acis");
  sil_register_tool (&thing04, "thing04", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "Pulley Block with coaxial tori", 
		     "acis", "acis");
  sil_register_tool (&thing05, "thing05", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "bracket plate with D2m", 
		     "acis", "acis");
  sil_register_tool (&thing06, "thing06", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "table base with C4", 
		     "acis", "acis");
  sil_register_tool (&thing07, "thing07", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "swing mount with D2m", 
		     "acis", "acis");
  sil_register_tool (&thing08, "thing08", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "transverse bracket with D2i", 
		     "acis", "acis");
  sil_register_tool (&thing09, "thing09", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "some odd shape, causes funny behaviour from ACIS", 
		     "acis", "acis");
  sil_register_tool (&thing10, "thing10", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "jelly mold former with D8m", 
		     "acis", "acis");
  sil_register_tool (&thing11, "thing11", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "half wheel sprocket with A8m", 
		     "acis", "acis");
  sil_register_tool (&thing12, "thing12", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "strip cutter with D6m", 
		     "acis", "acis");
  sil_register_tool (&thing13, "thing13", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "three way bracket", 
		     "acis", "acis");
  sil_register_tool (&thing14, "thing14", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "pentane molecule with D5m", 
		     "acis", "acis");
  sil_register_tool (&thing15, "thing15", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "chair base with C5", 
		     "acis", "acis");
  sil_register_tool (&thing16, "thing16", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "metal spacer", 
		     "acis", "acis");
  sil_register_tool (&thing17, "thing17", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "spherical wheel", 
		     "acis", "acis");
  sil_register_tool (&thing18, "thing18", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "Y-axle", 
		     "acis", "acis");
  sil_register_tool (&thing19, "thing19", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "double bladed cutter blank", 
		     "acis", "acis");
  sil_register_tool (&thing20, "thing20", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "six way optical component holder", 
		     "acis", "acis");
  sil_register_tool (&thing21, "thing21", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "square tray", 
		     "acis", "acis");
  sil_register_tool (&thing22, "thing22", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "8 sided clip", 
		     "acis", "acis");
  sil_register_tool (&thing23, "thing23", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "British standard electric plug base", 
		     "acis", "acis");
  sil_register_tool (&thing24, "thing24", "0.1",
		     "Bruce I Mills", "2000 Bruce I Mills",
		     "door key", 
		     "acis", "acis");

  return; 
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
