// scanner.cxx: Scanner module implementation.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: scanner.cc 1 2005-08-10 23:35:07Z frank $

#include "config.h"

RCSID (scanner, "$Id: scanner.cc 1 2005-08-10 23:35:07Z frank $");

#include "interface.hxx"
#include "xstring.hxx"
#include "xmath.hxx"
#include "scanner/scanner_module.hxx"
#include "scanner/scanner.hxx"
#include "scanner/points.hxx"

class TriangResult {
public:
  TriangResult (unsigned long a, unsigned long b, unsigned long c) {
    v1 = a;
    v2 = b;
    v3 = c;
  }
  unsigned long v1, v2, v3;
};

Material ids_material[15] = {
  Material (1.0, 0.0, 0.0, 1.0,
	    1.0, 0.0, 0.0, 1.0,
	    0.2, 0.0, 0.0, 1.0,
	    0.2, 0.0, 0.0, 1.0,
	    1.0),
  Material (0.0, 1.0, 0.0, 1.0,
	    0.0, 1.0, 0.0, 1.0,
	    0.0, 0.2, 0.0, 1.0,
	    0.0, 0.2, 0.0, 1.0,
	    1.0),
  Material (0.0, 0.0, 1.0, 1.0,
	    0.0, 0.0, 1.0, 1.0,
	    0.0, 0.0, 0.2, 1.0,
	    0.0, 0.0, 0.2, 1.0,
	    1.0),
  Material (1.0, 1.0, 0.0, 1.0,
	    1.0, 1.0, 0.0, 1.0,
	    0.2, 0.2, 0.0, 1.0,
	    0.2, 0.2, 0.0, 1.0,
	    1.0),
  Material (0.0, 1.0, 1.0, 1.0,
	    0.0, 1.0, 1.0, 1.0,
	    0.0, 0.2, 0.2, 1.0,
	    0.0, 0.2, 0.2, 1.0,
	    1.0),
  Material (1.0, 0.0, 1.0, 1.0,
	    1.0, 0.0, 1.0, 1.0,
	    0.2, 0.0, 0.2, 1.0,
	    0.2, 0.0, 0.2, 1.0,
	    1.0),
  Material (1.0, 0.5, 0.0, 1.0,
	    1.0, 0.5, 0.0, 1.0,
	    0.2, 0.1, 0.0, 1.0,
	    0.2, 0.1, 0.0, 1.0,
	    1.0),
  Material (0.0, 1.0, 0.5, 1.0,
	    0.0, 1.0, 0.5, 1.0,
	    0.0, 0.2, 0.1, 1.0,
	    0.0, 0.2, 0.1, 1.0,
	    1.0),
  Material (1.0, 0.0, 0.5, 1.0,
	    1.0, 0.0, 0.5, 1.0,
	    0.2, 0.0, 0.1, 1.0,
	    0.2, 0.0, 0.1, 1.0,
	    1.0),
  Material (0.5, 1.0, 0.0, 1.0,
	    0.5, 1.0, 0.0, 1.0,
	    0.1, 0.2, 0.0, 1.0,
	    0.1, 0.2, 0.0, 1.0,
	    1.0),
  Material (0.0, 0.5, 1.0, 1.0,
	    0.0, 0.5, 1.0, 1.0,
	    0.0, 0.1, 0.2, 1.0,
	    0.0, 0.1, 0.2, 1.0,
	    1.0),
  Material (0.5, 0.0, 1.0, 1.0,
	    0.5, 0.0, 1.0, 1.0,
	    0.1, 0.0, 0.2, 1.0,
	    0.1, 0.0, 0.2, 1.0,
	    1.0),
  Material (1.0, 0.5, 0.5, 1.0,
	    1.0, 0.5, 0.5, 1.0,
	    0.2, 0.1, 0.1, 1.0,
	    0.2, 0.1, 0.1, 1.0,
	    1.0),
  Material (0.5, 1.0, 0.5, 1.0,
	    0.5, 1.0, 0.5, 1.0,
	    0.1, 0.2, 0.1, 1.0,
	    0.1, 0.2, 0.1, 1.0,
	    1.0),
  Material (0.5, 0.5, 1.0, 1.0,
	    0.5, 0.5, 1.0, 1.0,
	    0.1, 0.1, 0.2, 1.0,
	    0.1, 0.1, 0.2, 1.0,
	    1.0),
};

SIL_MODULE ()
{
  return new ScannerModule;
}

const char *ScannerModule::nm = "scanner";

const char *ScannerModule::fl_formats[7] = {
  "asc", "ASCII scanner point file",
  "off", "OFF mesh (triangular only)",
  "mesh", "ASCII mesh file",
  NULL 
};

ScannerModule::ScannerModule ()
{
}

ScannerModule::~ScannerModule ()
{
}

void ScannerModule::rollback_del (void *object)
{
  if (object)
    delete (Scanner *)object;
  return;
}

void ScannerModule::draw_object (void *object, SimpleList<void> const &hirefs)
{
  if (((Scanner *)object)->is ("RangeMesh")) {
    RangeMesh *M = (RangeMesh *)object;
    if (M->num_triangles () > 0) {
      if (sil_get_draw_solid () && M->normal_x ()) {
	Material *m = M->material ();
	if (!m) {
	  m = new Material ();
	  M->set_material (m);
	}
	Material *last_m = m;
	bool inv = hirefs.is_member (object);
	unsigned long i = 0;
	while (i < M->num_triangles ()) {
	  if (M->ids () && M->ids () [i] > 0) {
	    Material *mm = ids_material + ((M->ids () [i] - 1) % 15);
	    if (mm != last_m) {
	      mm->gl_material (inv);
	      last_m = mm;
	    }
	  } else if (M->ids ()) {
	    if (m != last_m) {
	      m->gl_material (inv);
	      last_m = m;
	    }
	  } else {
	    if (m != last_m) {
	      m->gl_material (inv);
	      last_m = m;
	    }
	  }
	  glBegin(GL_POLYGON);
	  unsigned long v = M->triangle_v1 () [i];
	  if (M->project_x ()) {
	    glNormal3f (M->normal_x () [v], M->normal_y () [v],
	    		M->normal_z () [v]);
	    glVertex3d (M->project_x () [v], M->project_y () [v],
			M->project_z () [v]);
	    v = M->triangle_v2 () [i];
	    glNormal3f (M->normal_x () [v], M->normal_y () [v],
			M->normal_z () [v]);
	    glVertex3d (M->project_x () [v], M->project_y () [v],
			M->project_z () [v]);
	    v = M->triangle_v3 () [i++];
	    glNormal3f (M->normal_x () [v], M->normal_y () [v],
			M->normal_z () [v]);
	    glVertex3d (M->project_x () [v], M->project_y () [v],
			M->project_z () [v]);
	  } else {
	    glNormal3f (M->normal_x () [v], M->normal_y () [v],
			M->normal_z () [v]);
	    glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
	    v = M->triangle_v2 () [i];
	    glNormal3f (M->normal_x () [v], M->normal_y () [v],
			M->normal_z () [v]);
	    glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
	    v = M->triangle_v3 () [i++];
	    glNormal3f (M->normal_x () [v], M->normal_y () [v],
			M->normal_z () [v]);
	    glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
	  }
	  glEnd ();
	}
      } else {
	Material *m = M->material ();
	if (!m) {
	  m = new Material ();
	  M->set_material (m);
	}
	Material *last_m = m;
	bool inv = hirefs.is_member (object);
	m->gl_material_bright (inv);
	unsigned long v;
	unsigned long i = 0;
	while (i < M->num_triangles ()) {
	  if (M->ids () && M->ids () [i] > 0) {
	    Material *mm = ids_material + ((M->ids () [i] - 1) % 15);
	    if (mm != last_m) {
	      mm->gl_material_bright (inv);
	      last_m = mm;
	    }
	  } else if (M->ids ()) {
	    if (m != last_m) {
	      m->gl_material_bright (inv);
	      last_m = m;
	    }
	  } else {
	    if (m != last_m) {
	      m->gl_material (inv);
	      last_m = m;
	    }
	  }
	  if (M->neighbour1 () [i] > i) {
	    if (M->neighbour2 () [i] > i) {
	      if (M->neighbour3 () [i] > i) {
		glBegin(GL_LINE_LOOP);
		v = M->triangle_v1 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v2 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v3 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		glEnd ();
	      } else {
		glBegin(GL_LINE_STRIP);
		v = M->triangle_v2 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v3 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v1 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		glEnd ();
	      }
	    } else {
	      if (M->neighbour3 () [i] > i) {
		glBegin(GL_LINE_STRIP);
		v = M->triangle_v1 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v2 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v3 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		glEnd ();
	      } else {
		glBegin(GL_LINE_STRIP);
		v = M->triangle_v2 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v3 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		glEnd ();
	      }
	    }
	  } else {
	    if (M->neighbour2 () [i] > i) {
	      if (M->neighbour3 () [i] > i) {
		glBegin(GL_LINE_STRIP);
		v = M->triangle_v3 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v1 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v2 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		glEnd ();
	      } else {
		glBegin(GL_LINE_STRIP);
		v = M->triangle_v3 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		v = M->triangle_v1 () [i];
		if (M->normal_x ())
		  glNormal3f (M->normal_x () [v], M->normal_y () [v],
			      M->normal_z () [v]);
		glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
		glEnd ();

	      }
	    } else if (M->neighbour3 () [i] > i) {
	      glBegin(GL_LINE_STRIP);
	      v = M->triangle_v1 () [i];
	      if (M->normal_x ())
		glNormal3f (M->normal_x () [v], M->normal_y () [v],
			    M->normal_z () [v]);
	      glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
	      v = M->triangle_v2 () [i];
	      if (M->normal_x ())
		glNormal3f (M->normal_x () [v], M->normal_y () [v],
			    M->normal_z () [v]);
	      glVertex3d (M->x () [v], M->y () [v], M->z () [v]);
	      glEnd ();
	    }
	  }
	  ++i;
	}
      }
    }
  } else if (((Scanner *)object)->is ("RangePoints")) {
    RangePoints *P = (RangePoints *)object;
    Material *m = P->material ();
    if (!m) {
      m = new Material ();
      P->set_material (m);
    }
    m->gl_material_bright (hirefs.is_member (object));
    Material *pm = NULL;
    bool pmat = false;
    glMatrixMode (GL_MODELVIEW);
    glPushMatrix ();
    double x, y, z, rp, rt, rs;
    P->translation (x, y, z);
    glTranslated (x, y, z);
    P->rotation (rp, rt, rs);
    glRotatef (rs / PI * 180.0, 0.0, 0.0, -1.0);
    glRotatef (rt / PI * 180.0, -1.0, 0.0, 0.0);
    glRotatef (rp / PI * 180.0, 0.0, 0.0, -1.0);
    glBegin (GL_POINTS);
    RangePoint *p = P->first ();
    while (p) {
      pm = p->material ();
      if (pm) {
	pm->gl_material_bright ();
	pmat = true;
      } else if (pmat) {
	m->gl_material_bright ();
	pmat = false;
      }
      glVertex3d (p->x (), p->y (), p->z ());
      p = P->next ();
    }
    glEnd ();
    glPopMatrix ();
  } else   if (((Scanner *)object)->is ("RangePoint")) {
    RangePoint *p = (RangePoint *)object;
    Material *m = p->material ();
    if (!m) {
      m = new Material ();
      p->set_material (m);
    }
    m->gl_material_bright (hirefs.is_member (object));
    glBegin (GL_POINTS);
    glVertex3d (p->x (), p->y (), p->z ());
    glEnd ();
  }
  return;
}

void ScannerModule::del_object (void *object)
{
  if (object)
    delete (Scanner *)object;
  return;
}

void ScannerModule::load_file (FILE *fp, const char *format, bool ext,
			       bool replace)
{
  if (strcmp (format, "asc") == 0) {
    double x, y, z;
    RangePoints *P = new RangePoints ();
    int i = 0;
    while (!feof(fp)) {
      x = y = z = 0.0;
      if (fscanf (fp, "%lf %lf %lf\n", &x, &y, &z)) {
	P->add (x, y, z);
	++ i;
      }
    }
    char nn[27];
    unsigned long tn = sil_tmp_name ();
    snprintf (nn, 26, "Points_%lu", tn);
    while (!sil_add_symbol (nn, P, "scanner", true)) {
      unsigned long tnn = sil_tmp_name  ();
      if (tnn == tn) {
	sil_msg_error ("scanner",
		       "can't add symbol; no tmp_name symbols left");
	break;
      }
      snprintf (nn, 26, "Points_%lu", tnn);
    }
    snprintf (nn, 26, "%d points read", i);
    sil_msg ("open", nn);
  } else if (strcmp (format, "mesh") == 0) {
    while (!feof (fp)) {
      char *str = new char[8192];
      if (!fgets (str, 8191, fp))
	break;
      unsigned long npts, ntrgs, nreg;
      char fN, fP, fI;
      if (sscanf (str, "Mesh %lu %lu %lu %c%c%c\n", &npts, &ntrgs, &nreg,
		  &fN, &fP, &fI) == 6) {
	RangeMesh *M = new RangeMesh (npts, ntrgs);
	if (fN == '+')
	  M->init_normals ();
	if (fP == '+')
	  M->init_project ();
	if (fI == '+')
	  M->init_ids ();
	unsigned long l = 0;
	while (l < npts) {
	  double x, y, z, px, py, pz, nx, ny, nz;
	  fscanf (fp, "%lf %lf %lf %lf %lf %lf %lf %lf %lf\n",
		  &x, &y, &z, &nx, &ny, &nz, &px, &py, &pz);
	  M->x () [l] = x;
	  M->y () [l] = y;
	  M->z () [l] = z;
	  if (M->normal_x ()) {
	    M->normal_x () [l] = nx;
	    M->normal_y () [l] = ny;
	    M->normal_z () [l] = nz;
	  }
	  if (M->project_x ()) {
	    M->project_x () [l] = px;
	    M->project_y () [l] = py;
	    M->project_z () [l] = pz;
	  }
	  ++l;
	}
	unsigned short mid = 0;
	l = 0;
	while (l < ntrgs) {
	  unsigned long v1, v2, v3, n1, n2, n3;
	  unsigned short i;
	  fscanf (fp, "%lu %lu %lu %lu %lu %lu %hu", &v1, &v2, &v3, 
		  &n1, &n2, &n3, &i);
	  M->triangle_v1 () [l] = v1;
	  M->triangle_v2 () [l] = v2;
	  M->triangle_v3 () [l] = v3;
	  M->neighbour1 () [l] = n1;
	  M->neighbour2 () [l] = n2;
	  M->neighbour3 () [l] = n3;
	  if (M->ids ()) {
	    M->ids () [l] = i;
	    if (i > mid)
	      mid = i;
	  }
	  ++l;
	}
	M->max_id () = mid;
	l = 0;
	while (l < nreg) {
	  do{ 
	    if (!fgets (str, 8191, fp)) {
	      sil_msg_warn ("Scanner", "could not read all regions");
	      break;
	    }
	  } while (strncmp (str, "region", 6) != 0);
	  str[strlen(str) - 1] = '\0';
	  M->region () . add_last (new string (str));
	  do {
	    if (!fgets (str, 8191, fp)) {
	      sil_msg_warn ("Scanner", "could not read all regions");
	      break;
	    }
	  } while (strncmp (str, "g ", 2) != 0);
	  str[strlen(str) - 1] = '\0';
	  M->classify () . add_last (new string (str));
	  ++l;
	}
	delete []str;
	bool add = true;
	char nn[27];
	unsigned long tn = sil_tmp_name ();
	snprintf (nn, 26, "Mesh_%lu", tn);
	while (!sil_add_symbol (nn, M, "scanner", true)) {
	  unsigned long tnn = sil_tmp_name  ();
	  if (tnn == tn) {
	    sil_msg_error ("scanner",
			   "can't add symbol; no tmp_name symbols left");
	    add = false;
	    break;
	  }
	  snprintf (nn, 26, "Mesh_%lu", tnn);
	}
	if (add) {
	  sil_msg ("open", "Added " + static_cast<string> (nn) + ": " +
		   as_string (npts) + " points, " + as_string (ntrgs) +
		   " triangles.");
	} else {
	  delete M;
	}
      }
    }

  } else if (strcmp (format, "off") == 0) {

    char str[256];
    if (!fgets (str, 255, fp)) {
      sil_msg_error ("Scanner", "Cannot read OFF file.");
      return;
    }
    if (strncmp (str, "OFF", 3) != 0) {
      sil_msg_error ("Scanner", "Not an OFF file.");
      return;
    }
    
    unsigned long npts, ntrgs, d;
    if (fscanf (fp, "%lu %lu %lu", &npts, &ntrgs, &d) != 3) {
      sil_msg_error ("Scanner", "Corrupted OFF file.");
      return;
    }

    RangePoints *NPts = new RangePoints ();
    unsigned long l = 0;
    while (l < npts) {
      double x, y, z;
      if (fscanf (fp, "%lf %lf %lf", &x, &y, &z) == 3) {
	NPts->add (x, y, z);
	++l;
      } else {
	sil_msg_error ("Scanner", "Corrupted OFF file.");
	delete NPts;
	return;
      }
    }

    npts--;
    l = 0;
    SimpleList<TriangResult> trgs;
    while (l < ntrgs) {
      unsigned long h, v1, v2, v3;
      if (fscanf (fp, "%lu %lu %lu %lu", &h, &v1, &v2, &v3) == 4) {
	// Adjust numbers as we reverse the order of the points.
	v1 = npts - v1;
	v2 = npts - v2;
	v3 = npts - v3;
	trgs.add_last (new TriangResult (v1, v2, v3));
	++l;
      } else {
	h = 0;
      }
      if (h != 3) {
	sil_msg_error ("Scanner", 
		       "OFF file does not contain a triangular mesh.");
	trgs.purge ();
	delete NPts;
	return;
      }
    }

    if (trgs.length () == 0) {
      sil_msg_error ("Scanner", "Empty OFF mesh.");
      delete NPts;
      trgs.purge ();
      return;
    }

    RangeMesh *rm = new RangeMesh (NPts, trgs.length ());
    delete NPts;

    TriangResult *tr = trgs.unlink_first ();
    l = 0;
    while (tr) {
      rm->triangle_v1 () [l] = tr->v1;
      rm->triangle_v2 () [l] = tr->v2;
      rm->triangle_v3 () [l++] = tr->v3;
      delete tr;
      tr = trgs.unlink_first ();
    }

    rm->init_neighbours ();

    bool add = true;
    char nn[27];
    unsigned long tn = sil_tmp_name ();
    snprintf (nn, 26, "OFF_%lu", tn);
    while (!sil_add_symbol (nn, rm, "scanner", true)) {
      unsigned long tnn = sil_tmp_name  ();
      if (tnn == tn) {
	sil_msg_error ("scanner",
		       "can't add symbol; no tmp_name symbols left");
	add = false;
	break;
      }
      snprintf (nn, 26, "OFF_%lu", tnn);
    }
    if (add) {
      sil_msg ("open", "Added " + static_cast<string> (nn) + ": " +
	       as_string (npts + 1) + " points, " + as_string (ntrgs) +
	       " triangles.");
    } else {
      delete rm;
    }

  } else {
    sil_msg_warn ("scanner", "unsupported file format");
    return;
  }
  return;
}

void ScannerModule::save_file (FILE *fp, const char *format, bool ext)
{
  if (strcmp (format, "asc") == 0) {
    double x,y,z;
    const Symbol *s = sil_first_symbol ();
    while (s) {
      if (sil_symbol_is (s, "scanner")) {
	Scanner *S = (Scanner *)sil_get_symbol_data (s);
	if (S->is ("RangePoints")) {
	  RangePoints *P = (RangePoints *)S;
	  double rp, rt, rs;
	  P->rotation (rp, rt, rs);
	  double tx, ty, tz;
	  P->translation (tx, ty, tz);
	  bool r = P->first (x, y, z);
	  while (r) {
	    double xx =
	      ( cos (rs) * cos (rp) - cos (rt) * sin (rp) * sin (rs)) * x +
	      ( cos (rs) * sin (rp) + cos (rt) * cos (rp) * sin (rs)) * y +
	      ( sin (rs) * sin (rt)                                 ) * z;
	    double yy =
	      (-sin (rs) * cos (rp) - cos (rt) * sin (rp) * cos (rs)) * x +
	      (-sin (rs) * sin (rp) + cos (rt) * cos (rp) * cos (rs)) * y +
	    ( cos (rs) * sin (rt)                                 ) * z;
	    double zz = 
	      ( sin (rt) * sin (rp)) * x +
	      (-sin (rt) * cos (rp)) * y +
	      ( cos (rt))            * z;
	    fprintf (fp, "%f %f %f\n", xx + tx, yy + ty, zz + tz);
	    r = P->next (x, y, z);
	  }
	}
      }
      s = sil_next_symbol ();
    }
  } else if (strcmp (format, "mesh") == 0) {
    const Symbol *s = sil_first_symbol ();
    while (s) {
      if (sil_symbol_is (s, "scanner")) {
	Scanner *S = (Scanner *)sil_get_symbol_data (s);
	if (S->is ("RangeMesh")) {
	  RangeMesh *M = (RangeMesh *)S;
	  unsigned long n = M->num_points ();
	  fprintf (fp, "Mesh %lu %lu %lu %c%c%c\n", n, M->num_triangles (),
		   M->region () . length (),
		   M->normal_x () ? '+' : '-',
		   M->project_x () ? '+' : '-',
		   M->ids () ? '+' : '-');
	  unsigned long l = 0;
	  while (l < n) {
	    fprintf (fp, "%f %f %f", M->x () [l], M->y () [l], M->z () [l]);
	    if (M->normal_x ())
	      fprintf (fp, " %f %f %f", M->normal_x () [l], 
		       M->normal_y () [l], M->normal_z () [l]);
	    else
	      fprintf (fp, " 0 0 0");
	    if (M->project_x ())
	      fprintf (fp, " %f %f %f\n", M->project_x () [l], 
		       M->project_y () [l], M->project_z () [l]);
	    else
	      fprintf (fp, " 0 0 0\n");
	    ++l;
	  }
	  n = M->num_triangles ();
	  l = 0;
	  while (l < n) {
	    fprintf (fp, "%lu %lu %lu %lu %lu %lu",
		     M->triangle_v1 () [l],
		     M->triangle_v2 () [l],
		     M->triangle_v3 () [l],
		     M->neighbour1 () [l],
		     M->neighbour2 () [l],
		     M->neighbour3 () [l]);
	    if (M->ids ())
	      fprintf (fp, " %hu\n", M->ids () [l]);
	    else
	      fprintf (fp, " 0\n");
	    ++l;
	  }
	  string *reg = M->region () . first ();
	  string *cla = M->classify () . first ();
	  while (reg) {
	    fprintf (fp, "%s\n%s\n", reg->c_str (), cla->c_str ());
	    reg = M->region () . next ();
	    cla = M->classify () . next ();
	  }
	}
      }
      s = sil_next_symbol ();
    }
  } else {
    sil_msg_warn ("Scanner", "unsupported file format");
    return;
  }
  return;
}

const char **ScannerModule::file_formats () const
{
  return fl_formats;
}

const char *ScannerModule::name () const
{
  return nm;
}

/// Local Variables:
/// exe-comm: "../sil"
/// compile-command: "cd .. && make"
/// End:
