// light.hxx: Light base class.
//
// Copyright (C) 1999-2001 Frank C Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
// $Id: sil_light.hxx,v 1.8.2.1 2001/08/16 16:19:05 scmfcl Exp $
//

#ifndef SIL_LIGHT_HXX
#define SIL_LIGHT_HXX

#include "interface.hxx"
#include "renderer.hxx"

// General class for SiL lights. Any Light class in the modules should
// be derived from this class.
class SiL_Light
{
public:
  SiL_Light (GLfloat ar=0.0, GLfloat ag=0.0, GLfloat ab=0.0, GLfloat aa=1.0,
	     GLfloat dr=1.0, GLfloat dg=1.0, GLfloat db=1.0, GLfloat da=1.0,
	     GLfloat sr=1.0, GLfloat sg=1.0, GLfloat sb=1.0, GLfloat sa=1.0,
	     GLfloat x=0.0, GLfloat y=0.0, GLfloat z=1.0, GLfloat w=0.0,
	     GLfloat sx=0.0, GLfloat sy=0.0, GLfloat sz=-1.0,
	     GLfloat se=0.0,
	     GLfloat sc=180.0,
	     GLfloat ca=1.0,
	     GLfloat la=0.0,
	     GLfloat qa=0.0) {
    ambient[0] = ar;
    ambient[1] = ag;
    ambient[2] = ab;
    ambient[3] = aa;
    diffuse[0] = dr;
    diffuse[1] = dg;
    diffuse[2] = db;
    diffuse[3] = da;
    specular[0] = sr;
    specular[1] = sg;
    specular[2] = sb;
    specular[3] = sa;
    pos[0] = x;
    pos[1] = y;
    pos[2] = z;
    pos[3] = w;
    spot_direction[1] = sx;
    spot_direction[2] = sy;
    spot_direction[3] = sz;
    spot_exponent = se;
    spot_cutoff = sc;
    constant_attenuation = ca;
    linear_attenuation = la;
    quadratic_attenuation = qa;
  };
  inline void set (GLfloat ar=0.0, GLfloat ag=0.0, GLfloat ab=0.0, 
		   GLfloat aa=1.0,
		   GLfloat dr=1.0, GLfloat dg=1.0, GLfloat db=1.0,
		   GLfloat da=1.0,
		   GLfloat sr=1.0, GLfloat sg=1.0, GLfloat sb=1.0,
		   GLfloat sa=1.0,
		   GLfloat x=0.0, GLfloat y=0.0, GLfloat z=1.0,
		   GLfloat w=0.0,
		   GLfloat sx=0.0, GLfloat sy=0.0, GLfloat sz=-1.0,
		   GLfloat se=0.0,
		   GLfloat sc=180.0,
		   GLfloat ca=1.0,
		   GLfloat la=0.0,
		   GLfloat qa=0.0) {
    ambient[0] = ar;
    ambient[1] = ag;
    ambient[2] = ab;
    ambient[3] = aa;
    diffuse[0] = dr;
    diffuse[1] = dg;
    diffuse[2] = db;
    diffuse[3] = da;
    specular[0] = sr;
    specular[1] = sg;
    specular[2] = sb;
    specular[3] = sa;
    pos[0] = x;
    pos[1] = y;
    pos[2] = z;
    pos[3] = w;
    spot_direction[1] = sx;
    spot_direction[2] = sy;
    spot_direction[3] = sz;
    spot_exponent = se;
    spot_cutoff = sc;
    constant_attenuation = ca;
    linear_attenuation = la;
    quadratic_attenuation = qa;
  };
  inline void gl_render () {
    GLenum light_id = GL_FALSE;
    if (!Renderer::gl_lights[0]) {
      light_id = GL_LIGHT0;
      Renderer::gl_lights[0] = true;
    } else if (!Renderer::gl_lights[1]) {
      light_id = GL_LIGHT1;
      Renderer::gl_lights[1] = true;
    } else if (!Renderer::gl_lights[2]) {
      light_id = GL_LIGHT2;
      Renderer::gl_lights[2] = true;
    } else if (!Renderer::gl_lights[3]) {
      light_id = GL_LIGHT3;
      Renderer::gl_lights[3] = true;
    } else if (!Renderer::gl_lights[4]) {
      light_id = GL_LIGHT4;
      Renderer::gl_lights[4] = true;
    } else if (!Renderer::gl_lights[5]) {
      light_id = GL_LIGHT5;
      Renderer::gl_lights[5] = true;
    } else if (!Renderer::gl_lights[6]) {
      light_id = GL_LIGHT6;
      Renderer::gl_lights[6] = true;
    } else if (!Renderer::gl_lights[7]) {
      light_id = GL_LIGHT7;
      Renderer::gl_lights[7] = true;
    } else {
      sil_msg_warn ("light", "cannot add another OpenGL light source");
      return;
    }
    glEnable (light_id);
    glLightfv (light_id, GL_DIFFUSE, diffuse);
    glLightfv (light_id, GL_AMBIENT, ambient);
    glLightfv (light_id, GL_SPECULAR, specular); 
    glLightfv (light_id, GL_POSITION, pos);
    glLightfv (light_id, GL_SPOT_DIRECTION, spot_direction);
    glLightf (light_id, GL_SPOT_EXPONENT, (GLfloat)spot_exponent);
    glLightf (light_id, GL_SPOT_CUTOFF, (GLfloat)spot_cutoff);
    glLightf (light_id, GL_CONSTANT_ATTENUATION, 
	      (GLfloat)constant_attenuation);
    glLightf (light_id, GL_LINEAR_ATTENUATION, 
	      (GLfloat)linear_attenuation);
    glLightf (light_id, GL_QUADRATIC_ATTENUATION, 
	      (GLfloat)quadratic_attenuation);
  };
  inline const GLfloat *get_ambient () const {
    return ambient;
  };
  inline const GLfloat *get_diffuse () const {
    return diffuse;
  };
  inline const GLfloat *get_specular () const {
    return specular;
  };
  inline const GLfloat *get_position () const {
    return pos;
  };
  inline const GLfloat *get_spot_direction () const {
    return spot_direction;
  };
  inline const GLfloat get_spot_exponent () const {
    return spot_exponent;
  };
  inline const GLfloat get_spot_cutoff () const {
    return spot_cutoff;
  };
  inline const GLfloat get_constant_attenuation () const {
    return constant_attenuation;
  };
  inline const GLfloat get_linear_attenuation () const {
    return linear_attenuation;
  };
  inline const GLfloat get_quadratic_attenuation () const {
    return quadratic_attenuation;
  };
private:
  GLfloat ambient[4];
  GLfloat diffuse[4];
  GLfloat specular[4];
  GLfloat pos[4];
  GLfloat spot_direction[4];
  GLfloat spot_exponent;
  GLfloat spot_cutoff;
  GLfloat constant_attenuation;
  GLfloat linear_attenuation;
  GLfloat quadratic_attenuation;
}; 

#endif // !SIL_LIGHT_HXX

/// Local Variables:
/// exe-comm: "./sil"
/// compile-command: "make"
/// End:
